package com.sas.cfgwizard.migration
/**
 * The groovy script <code>listMigratedObjects</code> is used to extract information from the migration schema found in 
 * a migration package created by the SAS Migration Utility.  The migration schema contains information about the deployed 
 * components in the original system's configuration.
 * 
 * Input:
 *   migrationPackageLoc || Ant Project property 'config.migration.smu.file':  
 *      The migration package directory.
 *   migrationSchemaLoc || Ant Project property 'config.migration.schema.file':  
 *      The migration schema file.
 *   tierToConfigure || Ant Project property 'plan.machine.selected.name':  
 *      The name of the plan file machine being migrated.  Should map to a newTier in the migration schema.
 *   project12byte:  
 *      The 12byte of the product being configured.
 *   loadLocationProperties:  
 *      Enable the setting of host, port, and path properties.  True by default.
 *   loadSMUProperties:  
 *      Enable the setting of smu.properties.  False by default.  Not currently implemented.
 *   debug:
 *      Enable debug logging.  True by default.  
 *
 * @author <A HREF="mailto:Kyle.Hia@sas.com">Kyle Hia</A>
 * @version $Revision: 1.1.4.1 $
 */

def outProps = new Properties()

// Use this for normal problems arising in the script.
class MigrationPackageException extends Exception {
	MigrationPackageException(String msg) { super(msg) }
}
class MigrationSchemaException extends Exception {
	MigrationSchemaException(String msg) { super(msg) }
}

println "Class is: " + this.class.name

// Values obtained from ANT
def migrationPackageLoc = attributes["migrationPackageLoc"]
if (migrationPackageLoc==null) migrationPackageLoc = project.getProperty("config.migration.smu.file")

def migrationSchemaLoc = attributes["migrationSchemaLoc"]
if (migrationSchemaLoc==null) migrationSchemaLoc = project.getProperty("config.migration.schema.file")

def tierToConfigure = attributes["tierToConfigure"]
if (tierToConfigure==null) tierToConfigure = project.getProperty("plan.machine.selected.name")

def product = attributes["product12byte"]
if (product==null) throw new Exception("The listMigratedObjects() macro requires a product 12-byte")

def loadLocationProperties = true
if (attributes["loadLocationProperties"]) loadLocationProperties = attributes["loadLocationProperties"].equalsIgnoreCase("true") || "1"==attributes["loadLocationProperties"]

def loadSMUProperties = false
if (attributes["loadSMUProperties"]) loadSMUProperties = attributes["loadSMUProperties"].equalsIgnoreCase("true") || "1"==attributes["loadSMUProperties"]

def debug =  true
if (attributes["debug"]) debug = attributes["debug"].equalsIgnoreCase("true") || "1"==attributes["debug"]

// First handle the non-migration case
def projIsMigration = project.getProperty("config.migration.is_selected")
if (projIsMigration==null || projIsMigration.equalsIgnoreCase("false") || "0"==projIsMigration) {
	// Set some basic properties and be done.
	project.setNewProperty(product+".has.migration","0")
	project.setNewProperty(product+".migrated.object.count","0")
	project.setNewProperty(product+".listMigratedObjects.error.code","0")
	project.setNewProperty(product+".listMigratedObjects.error.msg","")
	return
}

def errorMsg = ""
def errorCode = 0

def CONFIGURLPREFIX = "sasconfigobj"

try {
	if (debug) {
		println "Called with the following: "
		println "   migrationPackageLoc=${migrationPackageLoc}"
		println "   migrationSchemaLoc=${migrationSchemaLoc}"
		println "   product=${product}"
		println "   tierToConfigure=${tierToConfigure}"
		println "   loadLocationProperties=${loadLocationProperties}"
		println "   debug=${debug}"
		println ""
	}

	def migrationPackageDir = new File(migrationPackageLoc)
	if (!migrationPackageDir || !migrationPackageDir.exists() || !migrationPackageDir.isDirectory()) throw new MigrationPackageException("Could not find the migration package")
	def schemaFile = new File(migrationSchemaLoc)
	if (!schemaFile || !schemaFile.exists() || !schemaFile.isFile()) throw new MigrationPackageException("Could not find the migration schema file: ${migrationSchemaLoc}")
	println "Parsing migration schema at: ${schemaFile.canonicalPath}";

	def ms = new XmlParser().parseText(schemaFile.getText("UTF-8"));

	def deploymentList = []
	def selectedTierName
	def selectedTierHostName
	Node selectedProduct
	def selectedProductName
	Node selectedTier

	//Need new metadata properties for every tier
	if("${product}"=="metadatasrv"){
		println "Retrieving metadatasrv information"
		def Node metaNode = ms.newTiers.tier.products.product.find {it.'@id'?.equalsIgnoreCase("metadatasrv")}
		if (metaNode)
			selectedTier = metaNode.parent().parent()
	}
	else{
		selectedTier = ms.newTiers[0].tier.find { it.'@name'?.equalsIgnoreCase(tierToConfigure) }
	}

	if (selectedTier) {
		selectedTierName = selectedTier.'@name'
		selectedTierHostName = selectedTier.'@host'
		if (!selectedTierHostName) throw new MigrationSchemaException("${selectedTierName} does not have a host attribute.")

		selectedProduct = selectedTier.products[0].product.find { it.'@id'?.equalsIgnoreCase(product) }

		if (selectedProduct) {
			selectedProductName = selectedProduct.'@id'

			deploymentList = selectedProduct.deployments[0].deployment
		}
	}

	def sasversion = ms.'@sasver'
	if (debug) { println "Migrating from SAS version: ${sasversion}" }
	outProps["config.migration.sas.version"]="${sasversion}"

	def nDeployments = deploymentList.size()
	if (debug) { println "Number of deployments: ${nDeployments}" }

	def isSingleton = false
	if (1==nDeployments) {
		def singletonAttr = deploymentList[0].'@singleton'
		if (singletonAttr?.equalsIgnoreCase("true") || singletonAttr?.equals("1")) isSingleton = true
	}

	// Values provided back to ANT
	outProps["${product}.migrated.object.count"]="${nDeployments}"
	if (nDeployments > 0) {
		outProps["${product}.has.migration"]="true"
	}
	else {
		outProps["${product}.has.migration"]="false"
	}

	def propNamePrefixList = ""
	def URIDropDownList = ""
	def labelDropDownList = ""
	def combinedList = ""
	def Node original
	def Node originalTier
	def originalTierName
	def File tierDir
	def File productDir

	/**
	 * In 9.1.3, many objects did not yet have an FQID. We allow them to specify the URL as any unique ID that can also
	 * be tolerated as an ANT property name. The SMU framework will generate a standard UUID when no URL is supplied.
	 * Then we create an expression which will be meaningful after the configure target has created the conventional FQID properties:
	 * The expression will look like: "@metadatasrv.configobj.fqid@". We also create the property name segment so that
	 * ANT properties for each such deployment can have a unique property name. For singletons without an 8.8 object ID,
	 * the property name is just "configobj".
	 */
	def getURIExpression = { theURI, thePropNameId ->
		if (theURI.length()==17 && theURI[8]=="." && theURI.substring(0,7) ==~ /[A-Za-z0-9]+/ && theURI.substring(9,16) ==~ /[A-Za-z0-9]+/) {
			return theURI
		}
		else {
			return "@${selectedProductName}.${thePropNameId}.fqid@"
		}
	}

	/**
	 * Set the original depoloyment matching a given URL.  Set the originalTier and originalTierName.
	 * Get the original OS arch and type properties and add them to the output properties.
	 */
	def setOriginalProperties = { theURI ->
		original = ms.originalTiers.tier.products.product.deployments.deployment.find {theURI.equalsIgnoreCase(it.'@URL')}
		if (!original) throw new MigrationSchemaException("Could not find the original configurable object matching URL: ${theURI}.")
		originalTier = original.parent().parent().parent().parent()
		originalTierName = originalTier.'@name'
		if (!originalTierName) throw new MigrationSchemaException("The tier name is missing for configurable object: ${theURI}.")

		def originalOS = originalTier.'@os'
		if (debug) { println "Migrating from OS: ${originalOS}" }
		outProps["config.migration.os.original.type"]="${originalOS}"

		def originalOSArch = originalTier.'@osarch'
		if (debug) { println "Migrating from OS arch: ${originalOSArch}" }
		outProps["config.migration.os.original.arch"]="${originalOSArch}"
	}

	/**
	 * Get all of the ports or paths (type) for a deployment on the original tier.  Match them by the sdwprop with the matching new tier
	 * deployment and set the current and prior output properties.
	 */
	def setLocationProperties = { newDeploymentNode, type, propNamePrefix ->
		original."${type}".each {
			def Node oldPortNode = it
			def oldPort = oldPortNode.text()
			def id = oldPortNode.'@sdwprop'
			def Node newPortNode = newDeploymentNode."${type}".find { it.'@sdwprop'.equalsIgnoreCase(id) }
			if (!newPortNode) throw new MigrationSchemaException("Could not find matching new ${type} defintion for ${id} of ${product}-${selectedTierName}")
			outProps["${propNamePrefix}.prior.${id}"]= oldPort
			outProps["${propNamePrefix}.${id}"]= newPortNode.text()
			outProps["${propNamePrefix}.${id}.URL"]= newPortNode.'@URL'
		}
	}

	/**
	 * Set the current and old host output properties.  Search for port and path properties to set.
	 */
	def handleLocationProperties = { newDeploymentNode, propNamePrefix ->
		if (loadLocationProperties) {
			// Define old and new hostnames
			outProps["${propNamePrefix}.host"]=selectedTierHostName

			def originalTierHostName = originalTier.'@host'
			if (!originalTierHostName) throw new MigrationSchemaException("${originalTierName} does not have a host attribute.")
			outProps["${propNamePrefix}.prior.host"]=originalTierHostName

			setLocationProperties newDeploymentNode, 'port', propNamePrefix
			setLocationProperties newDeploymentNode, 'path', propNamePrefix
		}
	}

	/**
	 * Get and verify the tier directory and product directory in the migration package 
	 */
	def setDirectories = {
		tierDir = new File(migrationPackageDir, originalTierName)
		if (!tierDir || !tierDir.exists() || !tierDir.isDirectory()) throw new MigrationPackageException("Could not find the tier subdirectory for: ${originalTierName}.")
		productDir = new File(tierDir, product)
		if (!productDir || !productDir.exists() || !productDir.isDirectory()) throw new MigrationPackageException("Could not find the product subdirectory for: ${originalTierName}/${product}.")
	}

	// Singletons generally just have a URL.  Labels are optional.  Singletons do not have subdirectories.
	if (isSingleton) {
		def Node newDeploymentNode = deploymentList[0]
		def theURI = newDeploymentNode.'@URL'
		if (!theURI) throw new MigrationSchemaException("missing URL attribute for a ${product} configurable object on ${selectedTierName}.")

		def thePropNameId = "configobj"
		def theURIExpr = getURIExpression (theURI, thePropNameId)
		def theLabel = selectedProductName  /* The label will never be surfaced. */

		// Now find the original deployment of this configurable object
		setOriginalProperties theURI

		def propNamePrefix = "${product}.migrated.object.${thePropNameId}"
		propNamePrefixList = propNamePrefix   // there is only one in the list
		URIDropDownList = theURI
		labelDropDownList = theLabel
		combinedList = theLabel + "=" + theURIExpr

		handleLocationProperties newDeploymentNode, propNamePrefix
		setDirectories()

		outProps["${propNamePrefix}.dir.name"]="."
		outProps["${propNamePrefix}.label"]=theLabel
		outProps["${propNamePrefix}.dir"]=productDir.canonicalPath
	}
	// Non-singletons require labels and subdirectories.
	else {
		def int count = 0
		deploymentList.each {
			def int ordinal = count+1  // 1-based ordinal
			def Node newDeploymentNode = it
			def theURI = it.'@URL'
			if (!theURI) throw new MigrationSchemaException("missing URL attribute")
			def theLabel = it.'@label'
			if (!theLabel) throw new MigrationSchemaException("missing label attribute")

			def thePropNameId = "configobj" + ordinal
			def propNamePrefix = "${product}.migrated.object.${thePropNameId}"
			def theURIExpr = getURIExpression (theURI, thePropNameId)

			if (count<nDeployments-1) {
				propNamePrefixList += (propNamePrefix +  "|")
				URIDropDownList += (theURI +  "|")
				labelDropDownList += (it.'@label' +  " | ")
				combinedList += ( it.'@label' + "=" + theURIExpr +  "|")
			}
			else {
				propNamePrefixList += propNamePrefix
				URIDropDownList += theURI
				labelDropDownList += it.'@label'
				combinedList += ( it.'@label' + "=" + theURIExpr)
			}

			// Now find the original deployment of this configurable object
			setOriginalProperties theURI

			setDirectories()
			def subDirName = original.'@packageSubdir'
			def File theDir = new File(productDir, subDirName)
			if (!theDir || !theDir.exists() || !theDir.isDirectory()) throw new MigrationPackageException("Could not find the subdirectory for: ${originalTierName}/${product}/${subDirName}.")

			handleLocationProperties newDeploymentNode, propNamePrefix

			outProps["${propNamePrefix}.dir.name"]=subDirName
			outProps["${propNamePrefix}.label"]=theLabel
			outProps["${propNamePrefix}.dir"]=theDir.canonicalPath

			count++
		}
	}

	if (nDeployments>0) {
		outProps["${product}.migrated.object.property.name.prefix.list"]=propNamePrefixList
		outProps["${product}.migrated.object.URI.list"]=URIDropDownList
		outProps["${product}.migrated.object.label.list"]=labelDropDownList
		// This name defined by SDW.
		outProps["${product}.migration.list"]=combinedList
	}
}
catch (MigrationSchemaException msex) {
	errorMsg = "Error in migration schema ${migrationSchemaLoc}: " + msex.message
	errorCode = 10
	msex.printStackTrace()
}
catch (MigrationPackageException mpex) {
	errorMsg = "Error in the migration package ${migrationPackageLoc}: " + mpex.message
	errorCode = 10
	mpex.printStackTrace()
}
catch (Throwable t) {
	errorMsg = "Unexpected error while processing the migration schema: " + t.message
	errorCode = 10
	t.printStackTrace()
}
outProps["${product}.listMigratedObjects.error.msg"]= errorMsg
outProps["${product}.listMigratedObjects.error.code"]=errorCode.toString()

if (debug) {
	println "\n\n---- Property settings for ANT ----"
	def keys = outProps.keySet().sort()
	keys.each{ println "   ${it} = ${outProps[it]}" }
}

outProps.each { project.setNewProperty(it.key, it.value) }

