// Copyright (c) 2010, SAS Institute Inc., Cary, NC, USA. All Rights Reserved

// Script used to update the Name Description attributes of an
// Application SoftwareComponent.

// Attributes:
// 
// 1. resultproperty
//    Name of an Ant property which will be defined by this script
//    to specify an int value representing the return code.
//    This attribute is required.
//     0 : success
//    -1 : initialization failure
//    -2 : Metadata Server connection failure
//    -3 : SoftwareComponent search failure
//    -4 : SoftwareComponent search returned no results
//    -5 : SoftwareComponent search returned more than one result
//    -6 : update SoftwareComponent attribute failure
// 2. outputproperty
//    Name of an Ant property which will be defined by this script
//    to specify a message summarizing its completion status.
//    This attribute is optional.
// 3. outputpropertyfqid
//    Name of an Ant property which will be defined by this script
//    to specify the FQID of the SoftwareComponent object.
//    This attribute is required.
//    (e.g. "A5ZBO87U.AV00000G")
// 4. softwarecomponentname
//    The name of the SoftwareComponent whose PublicType is 'Application'
//    whose Name and Description attributes are to be updated.
//    This XMLSelect should uniquely identify one SoftwareComponent when used in the 
//    XMLSelect where xxx is the name of the SoftwareComponent.
//    "SoftwareComponent[@Name='xxx' and @PublicType='Application']"
//    This attribute is required.
//    (e.g. "Web Infra Platform Services 9.2")
// 5. host
//    Metadata Server host.
//    If this attribute is not specified, then the host will default
//    to the value of the "SMU.host.metadata" property.
//    This attribute is optional.
//    (e.g. "xxxx.xx.xx.com")
// 6. port
//    Metadata Server port.
//    If this attribute is not specified, then the port will default
//    to the value of the "SMU.port.metadata" property.
//    (e.g. "8561")
// 7. user
//    Metadata Server connection user ID.
//    If this attribute is not specified, then the user ID will default
//    to the value of the "SMU.user" property.
//    This attribute is optional.
//    (e.g. "myuserid")
// 8. password
//    Metadata Server connection user password.
//    If this attribute is not specified, then the password will default
//    to the value of the "SMU.password" property.
//    This attribute is optional.
//    (e.g. "mypassword")
// 9. newname
//    The new value for the SoftwareComponent.Name attribute.
//    (e.g. "Web Infra Platform Services 9.3")
//10. newversion
//    The new value for the SoftwareComponent.SoftwareVersion attribute
//    (e.g. "9.3")


import java.rmi.RemoteException;
import java.util.Iterator;
import java.util.List;

import com.sas.metadata.remote.CMetadata;
import com.sas.metadata.remote.MdException;
import com.sas.metadata.remote.MdFactory;
import com.sas.metadata.remote.MdFactoryImpl;
import com.sas.metadata.remote.MdOMRConnection;
import com.sas.metadata.remote.MdObjectStore;
import com.sas.metadata.remote.MdOMIUtil;
import com.sas.metadata.remote.MetadataObjects;
import com.sas.metadata.remote.SoftwareComponent;
import com.sas.metadata.remote.Tree;

int rc = -1;
String returnMsg = "Unable to initialize.";

AntBuilder ant = new AntBuilder();

logger =
{
   level, msg -> ant.echo(level:level, message:msg);
}


// name of the Ant property which will be set a return status message.
String outputproperty_msg = attributes["outputproperty"];

// name of the Ant property which will be set to the return code.
//  0 : success
// -1 : initialization failure
// -2 : Metadata Server connection failure
// -3 : SoftwareComponent search failure
// -4 : SoftwareComponent search returned no results
// -5 : SoftwareComponent search returned more than one result
// -6 : set SoftwareComponent attribute failure
// -7 : persist SoftwareComponent attribute failure

String outputproperty_returncode = attributes["resultproperty"];
logger("info", "resultproperty=" + outputproperty_returncode);

// Name of the SoftwareComponent which is associated to a
// TCPIPConnection via a Connection URI.
String softwareComponentName = attributes["softwarecomponentname"];
logger("info", "softwarecomponentname=" + softwareComponentName);

// Metadata Server connection user ID.
// If this attribute is not specified, then the user ID will default
// to the value of the "SMU.user" property.
String metadataUser = attributes["user"];
logger("info", "user=" + metadataUser);

// Metadata Server connection user password.
// If this attribute is not specified, then the password will default
// to the value of the "SMU.password" property.
String metadataPassword = attributes["password"];
logger("info", "password=***");

// Metadata Server host.
// If this attribute is not specified, then the host will default
// to the value of the "SMU.host.metadata" property.
String metadataHost = attributes["host"];
logger("info", "host=" + metadataHost);

// Metadata Server port.
// If this attribute is not specified, then the port will default
// to the value of the "SMU.port.metadata" property.
String metadataPort = attributes["port"];
logger("info", "port=" + metadataPort);

// name of the Ant property which will be set to the FQID of the
// TCPIPConnection metadata object which was updated
// (e.g. "A5ZBO87U.AV00000G")
String outputproperty_fqid = attributes["outputpropertyfqid"];
logger("info", "outputpropertyfqid=" + outputproperty_fqid);

// Application SoftwareComponent's new name
// (e.g. "Web Infra Platform Services 9.3")
String newName = attributes["newname"];
logger("info", "newname=" + newName);

// Software version
String softwareVersion = attributes["newversion"];
logger("info", "newversion=" + softwareVersion);

MdFactory mdFactory = null;

int majorVersion = 0;
int minorVersion = 0;

try {
   final String[] versions = softwareVersion.split("\\.");
   if (versions.length > 0)
   {
      majorVersion = Integer.parseInt(versions[0]);
      logger("info", "Major Version=" + majorVersion);
   }
   
   if (versions.length > 1)
   {
      minorVersion = Integer.parseInt(versions[1]);
      logger("info", "Minor Version=" + minorVersion);
   }
}
catch (NumberFormatException e) {
   logger("debug", "Unable to determine major and minor version by parsing software version " + softwareVersion);
   majorVersion = 0;
   minorVersion = 0;
}

try
{
   
   rc = -2;
   returnMsg = "Unable to connect to the Metadata Server";
   // create connection to the Metadata Server

   mdFactory = new MdFactoryImpl(false);
   MdOMRConnection mdOMRConnection = mdFactory.getConnection();
   mdOMRConnection.makeOMRConnection(
      (metadataHost == null)
         ? project.getProperty("SMU.host.metadata")
         : metadataHost,
      (metadataPort == null)
         ? project.getProperty("SMU.port.metadata")
         : metadataPort,
      (metadataUser == null)
         ? project.getProperty("SMU.user")
         : metadataUser,
      (metadataPassword == null)
         ? project.getProperty("SMU.password")
         : metadataPassword);
   MdObjectStore objectStore = mdFactory.createObjectStore();
   MdOMIUtil mdOMIUtil = mdFactory.getOMIUtil();


   rc = -3;
   // query SoftwareComponent from the Metadata Server

   String options = "<XMLSelect search=\"SoftwareComponent[@Name='" + softwareComponentName + "' and @PublicType='Application']\"/>";
   returnMsg = "Search failed for: " + options;
   List softwareComponents = mdOMIUtil.getMetadataObjectsSubset(
      objectStore,
      mdOMIUtil.getFoundationReposID(),
      MetadataObjects.SOFTWARECOMPONENT,
      MdOMIUtil.OMI_GET_METADATA
         | MdOMIUtil.OMI_ALL_SIMPLE
         | MdOMIUtil.OMI_TEMPLATE
         | MdOMIUtil.OMI_XMLSELECT,
      options);

   if (softwareComponents.isEmpty())
   {
      rc = -4;
      returnMsg = "The search " + options + " did not return a SoftwareComponent.";
      // search failed to return a SoftwareComponent
      throw new IllegalStateException(returnMsg);
   }
   
   if (softwareComponents.size() > 1)
   {
      rc = -5;
      returnMsg = "The search " + options + " returned multiple SoftwareComponents.";
      // search returned multiple connection URIs
      throw new IllegalStateException(returnMsg);
   }


   rc = -6;
   returnMsg = "Unable to set SoftwareComponent attribute.";
   // read TCPIPConnection attributes
   
   SoftwareComponent softwareComponent = (SoftwareComponent) softwareComponents.get(0);
   String fqid = softwareComponent.getFQID();
   String origName = softwareComponent.getName();
   String origDesc = softwareComponent.getDesc();
   String origSoftwareVersion = softwareComponent.getSoftwareVersion();
   
   boolean hasUpdate = false;
   
   if (null != newName && newName != origName && newName.trim().length() > 0)
   {
      logger("info", "setting SoftwareComponent.Name to: " + newName);
      softwareComponent.setName(newName);
      hasUpdate = true;
   }
   
   if (origDesc.contains(origName))
   {
      String description = origDesc.replace(origName, newName);
      logger("info", "setting SoftwareComponent.Desc to: " + description);
      softwareComponent.setDesc(description);
      hasUpdate = true;
   }
   
   if (null != softwareVersion && softwareVersion != origSoftwareVersion && softwareVersion.trim().length() > 0)
   {
      logger("info", "setting SoftwareComponent.SoftwareVersion to: " + softwareVersion);
      softwareComponent.setSoftwareVersion(softwareVersion);
      if (majorVersion != softwareComponent.getMajor())
      {
         softwareComponent.setMajor(majorVersion);
         hasUpdate = true;
      }
      if (minorVersion != softwareComponent.getMinor())
      {
         softwareComponent.setMinor(minorVersion);
         hasUpdate = true;
      }

   }
   
   for (CMetadata cMetadata : softwareComponent.getTrees())
   {
      Tree tree = (Tree) cMetadata;
      if (softwareComponentName == tree.getName())
      {
         tree.setName(newName);
         String description = tree.getDesc().replace(origName, newName);
         tree.setDesc(description);
         logger("info", "set Tree[Id=" + tree.getFQID() + "][Name=" + newName + "][Desc=" + description + "].");
         
         Tree parentTree = tree.getParentTree();
         if (softwareComponentName == parentTree.getName())
         {
            parentTree.setName(newName);
            parentTree.setSoftwareVersion(softwareVersion);
            description = parentTree.getDesc().replace(origName, newName);
            parentTree.setDesc(description);
            logger("info", "set Tree[Id=" + parentTree.getFQID() + "][Name=" + newName + "][Desc=" + description + "].");
         }
         break;
      }
   }
   
   if (hasUpdate)
   {
      rc = -7;
      returnMsg = "Unable to persist updates.";
      softwareComponent.getObjectStore().updatedMetadataAll();
   }
   
   rc = 0;
   returnMsg = "The SoftwareComponent[ID=" + fqid + "] Application has been updated from [Name=" + origName + "][Description=" + origDesc + "] to [ID=" + fqid + "][Name=" + softwareComponent.getName() + "][Description=" + softwareComponent.getDesc() + "].";
   // success
   
   // set a property whose value specifies the FQID of the
   // updated SoftwareComponent.
   project.setNewProperty(outputproperty_fqid, fqid);

}
catch (Exception e)
{
   logger("error", e.getMessage());
   sw = new StringWriter();
   pw = new PrintWriter(sw);
   e.printStackTrace(pw);
   logger("error", sw.toString());
}
finally
{
   if (null != outputproperty_msg && outputproperty_msg.trim().length() > 0)
   {
      project.setNewProperty(outputproperty_msg, returnMsg);
   }
   project.setNewProperty(outputproperty_returncode, "" + rc);

   logger("info", returnMsg);

   if (null != mdFactory)
   {
      try
      {
         mdFactory.dispose();
      }
      catch (RemoteException e)
      {
         logger("error", e.getMessage());
      }
   }

}