// Copyright (c) 2010, SAS Institute Inc., Cary, NC, USA. All Rights Reserved

// Script used to update the Host, Port, CommunicationProtocol
// and/or Service attributes of a TCPIPConnection.

// Attributes:
//
// 1. resultproperty
//    Name of an Ant property which will be defined by this script
//    to specify an int value representing the return code.
//    This attribute is required.
//     0 : success
//    -1 : initialization failure
//    -2 : Metadata Server connection failure
//    -3 : TCPIPConnection search failure
//    -4 : TCPIPConnection search returned no results
//    -5 : update TCPIPConnection attribute failure
//    -6 : Metadata Server update failure
// 2. outputproperty
//    Name of an Ant property which will be defined by this script
//    to specify a message summarizing its completion status.
//    This attribute is optional.
// 3. tcpipconnectionfqid
//    The FQID of the TCPIPConnection object which will be updated.
//    This attribute is required.
//    (e.g. "A5ZBO87U.AV00000G")
// 4. host
//    TCPIPConnection's new host or "" if not updating the host.
//    This attribute is optional.
//    (e.g. "xxx.yyy.com")
// 5. port
//    TCPIPConnection's new port or "" if not updating the port.
//    This attribute is optional.
//    (e.g. "1234")
// 6. protocol
//    TCPIPConnection's new communication protocol or "" if not updating the protocol.
//    This attribute is optional.
//    (e.g. "http", "https")
// 7. service
//    TCPIPConnection's service or "" if not updating the service.
//    This attribute is optional.
//    (e.g. "/SASSharedApps")


import java.rmi.RemoteException
import java.util.Iterator
import java.util.List

import com.sas.metadata.remote.MdException
import com.sas.metadata.remote.MdFactory
import com.sas.metadata.remote.MdFactoryImpl
import com.sas.metadata.remote.MdOMRConnection
import com.sas.metadata.remote.MdObjectStore
import com.sas.metadata.remote.MdOMIUtil
import com.sas.metadata.remote.MetadataObjects
import com.sas.metadata.remote.TCPIPConnection

int rc = -1
String returnMsg = "Unable to initialize."

AntBuilder ant = new AntBuilder()

logger =
{
   level, msg -> ant.echo(level:level, message:msg)
}


// name of the Ant property which will be set to the FQIDs of the
// TCPIPConnection which were updated or an empty string if no
// TCPIPConnections were updated
String outputproperty_msg = attributes["outputproperty"]

// name of the Ant property which will be set to the return code.
//  0 : success
// -1 : initialization failure
// -2 : Metadata Server connection failure
// -3 : TCPIPConnection search failure
// -4 : TCPIPConnection search returned no results
// -5 : update TCPIPConnection attribute failure
// -6 : Metadata Server update failure
String outputproperty_returncode = attributes["resultproperty"]

// name of the Ant property which will be set to the FQID of the
// TCPIPConnection metadata object which will be updated
// (e.g. "A5ZBO87U.AV00000G")
String fqid = attributes["tcpipconnectionfqid"]

// host (e.g. "xxx.yyy.com")
String host = attributes["host"]

// port (e.g. "1234")
String port = attributes["port"]

// communication protocol (e.g. "http", "https")
String protocol = attributes["protocol"]

// servlet context (e.g. "/SASLogon")
String service = attributes["service"]

boolean hasUpdate = false
MdFactory mdFactory = null

try
{
   rc = -2
   returnMsg = "Unable to connect to the Metadata Server"
   // create connection to the Metadata Server

   mdFactory = new MdFactoryImpl(false)
   MdOMRConnection mdOMRConnection = mdFactory.getConnection()
   mdOMRConnection.makeOMRConnection(
      project.getProperty("iomsrv.metadatasrv.host"),
      project.getProperty("iomsrv.metadatasrv.port"),
      project.getProperty("metadata.connection.userid"),
      project.getProperty("metadata.connection.passwd"))
   MdObjectStore objectStore = mdFactory.createObjectStore()
   MdOMIUtil mdOMIUtil = mdFactory.getOMIUtil()


   rc = -3
   // query TCPIPConnection from the Metadata Server

   String options = "<XMLSelect search=\"TCPIPConnection[@Id='" + fqid + "']]\"/>"
   returnMsg = "Search failed for: " + options
   List tcpipConnections = mdOMIUtil.getMetadataObjectsSubset(
      objectStore,
      mdOMIUtil.getFoundationReposID(),
      MetadataObjects.TCPIPCONNECTION,
      MdOMIUtil.OMI_GET_METADATA
         | MdOMIUtil.OMI_ALL_SIMPLE
         | MdOMIUtil.OMI_TEMPLATE
         | MdOMIUtil.OMI_XMLSELECT,
      options)

   if (tcpipConnections.isEmpty())
   {
      rc = -4
      returnMsg = "The search " + options + " did not return a URI connection."
      // search failed to return a connection URI
      throw new IllegalStateException(returnMsg)
   }

   rc = -5
   returnMsg = "Unable to set TCPIPConnection attribute."
   // update the TCPIPConnection attributes
   
   TCPIPConnection tcpipConnection = (TCPIPConnection) tcpipConnections.get(0)
   String updateMsgPrefix = "Updating TCPIPConnection[Id=" + fqid + "] "

   String fromURI =
      tcpipConnection.getCommunicationProtocol() + "://" + 
      tcpipConnection.getHostName() + ':' +
      tcpipConnection.getPort() + 
      tcpipConnection.getService()
      
   if (null != port
       && port.trim().length() > 0
       && !(port.contains("${") && port.contains("}")) 
       && Integer.parseInt(port) != tcpipConnection.getPort())
   {
      returnMsg = "Port '" + port + "' is invalid."
      tcpipConnection.setPort(port)
      hasUpdate = true
   }
   
   if (null != protocol
      && protocol.trim().length() > 0
      && !(protocol.contains("${") && protocol.contains("}"))
      && protocol != tcpipConnection.getCommunicationProtocol())
   {
      returnMsg = "Protocol '" + protocol + "' is invalid."
      tcpipConnection.setCommunicationProtocol(protocol)
      hasUpdate = true
   }

   if (null != host
      && host.trim().length() > 0
      && !(host.contains("${") && host.contains("}"))
      && host != tcpipConnection.getHostName())
   {
      returnMsg = "Host '" + host + "' is invalid."
      tcpipConnection.setHostName(host)
      hasUpdate = true
   }

   if (null != service
      && service.trim().length() > 0
      && !(service.contains("${") && service.contains("}"))
      && service != tcpipConnection.getService())
   {
      returnMsg = "Service '" + service + "' is invalid."
      tcpipConnection.setService(service)
      hasUpdate = true
   }


   if (hasUpdate)
   {
      rc = -6
      returnMsg = "Unable to persist TCPIPConnection updates to the Metadata Server."
      // persist updates to the Metadata Server

      tcpipConnection.updateMetadataAll()

      String toURI =
         tcpipConnection.getCommunicationProtocol() + "://" + 
         tcpipConnection.getHostName() + ':' +
         tcpipConnection.getPort() + 
         tcpipConnection.getService()
         
      returnMsg = "Updated TCPIPConnection[ID=" + fqid + "][Name='" + tcpipConnection.getName() + "'] from '" + fromURI + "' to '" + toURI + "'."
   }
   else
   {
      returnMsg = "TCPIPConnection[ID=" + fqid + "][Name='" + tcpipConnection.getName() + "'] was not changed from " + fromURI + " since there were no attribute value changes."
      logger(
         "info",
         returnMsg)
      
   }
   
   rc = 0
   // success
}
catch (Exception e)
{
   logger("error", e.getMessage())
   sw = new StringWriter()
   pw = new PrintWriter(sw)
   e.printStackTrace(pw)
   logger("error", sw.toString())
}
finally
{
   if (null != mdFactory)
   {
      try
      {
         mdFactory.dispose()
      }
      catch (RemoteException e)
      {
         logger("error", e.getMessage())
      }
   }
   
   if (null != outputproperty_msg && outputproperty_msg.trim().length() > 0)
   {
      project.setNewProperty(outputproperty_msg, returnMsg)
   }
   project.setNewProperty(outputproperty_returncode, "" + rc)

   logger("info", returnMsg)
   logger("info", outputproperty_returncode + " = " + rc)
}