// Copyright (c) 2002-2011, SAS Institute Inc., Cary, NC, USA, All Rights Reserved

package com.sas.config.util
import groovy.swing.SwingBuilder
import java.awt.BorderLayout as BL
import javax.swing.WindowConstants as WC
import javax.swing.SwingConstants as SC

//*********************************************************************************************
// Class: CredentialsDialog
// Purpose: Provide a pop-up Groovy dialog to solicit all userIds and passwords required to 
// 			configure WAS resources such as WAS Admin credentials, MailSession, LoginModule
//			DataSource, MultiCast, and JMSConnectionFactory. 
//*********************************************************************************************

class CredentialsDialog {
	static final String SEPARATOR = '-'*90
	
	// Class variables for Java Swing (GRAPHIC) support:
	def swing
	def frame
	def getCredentialsDialog
	def enter
	
	// Common class variables
	String userid = new String("")
	String passwd = new String("")
	
	//******************************************************************************************
	// CTOR:
	// Input:
	//		content - list of maps, each of which has 3 keys: "titleText", "userid" and "passwd"
	//				  The values for these three keys are used to build a dynamic table on a 
	//				  Groovy dialog to solicit userIds and passwords required to configure 
	//				  WAS resources.
	//		outmap  - A map that will contain key-value pairs with userIds and passwords.  The
	//				  key name is obtained from the value of "userid" and "passwd" keys in the
	//				  input content list of maps above.
	//		promptMode - "GRAPHIC" or "TEXT" (default)
	//****************************************************************************************** 	
	public CredentialsDialog(def content, def outmap, def promptMode="TEXT") {
		if (promptMode == "GRAPHIC") {
			swing = new SwingBuilder();
	
			// Enter key handler:
			enter = swing.action(name:'Enter', enabled:true,
				closure: {
					content.each { idInfo ->
						def uidKey = idInfo.get('userid')
						userid = swing."${uidKey}".text
						outmap.put(uidKey, userid)
						def pwdKey = idInfo.get('passwd')
						passwd = swing."${pwdKey}".text
						outmap.put(pwdKey, passwd)
					}
					// outmap.each {key,value -> println "Enter: $key=$value"}
					getCredentialsDialog.dispose()	// Close the dialog and return
				}
			)
			
			// Frame and dialog components:
			frame = swing.frame( title:"Resource Configuration Credentials",
								 defaultCloseOperation: WC.EXIT_ON_CLOSE )
			{
				// Dialog component:
				getCredentialsDialog = swing.dialog(title:"Enter credentials",
													owner:frame,
													modal:true )
				{
					// Table of Resource type text and associated userId and password text fields:
					tableLayout(cellpadding:3) {
						tr {
							td(colspan:2, align:SC.LEFT) { label(SEPARATOR) }
						}
						content.each { idInfo ->
							def titleText = idInfo.get('titleText')
							tr {
								td(colspan:2) { label("Enter credentials for: ${titleText}") }
							}
							def uid = idInfo.get('userid')
							tr {
								td { label('Username:') }
								td { textField(id:uid, columns:25, '') }
							}
							def pwd = idInfo.get('passwd')
							tr {
								td { label('Password:') }
								td { passwordField(id:pwd, columns:25, '') }
							}
							tr {
								td(colspan:2, align:SC.LEFT) { label(SEPARATOR) }
							}
						}
						tr {
							td(colspan:2, align:SC.CENTER) { button(defaultButton:true, action:enter, ) }
						}
					}	// End tableLayout
				}	// End dialog
				getCredentialsDialog.setLocationRelativeTo(frame)
				getCredentialsDialog.pack()
				getCredentialsDialog.show()
				getCredentialsDialog.dispose()
			}	// End frame
			frame.dispose()
		}		// End GRAPHIC mode (Java Swing support)
		
		else {	// TEXT mode support
			// Prepare to use the System Console
			java.io.Console console = java.lang.System.console()	// Get system console object associated with the JVM
			if (console != null)  {		// If a console object is available,
				try { 
					console.format("%s","\n*=*=*=*=*=*=*  BEGIN Prompting for credentials *=*=*=*=*=*=*\n\n")
					console.format("%s",SEPARATOR+"\n")
					content.each { idInfo ->	// Process each credential prompt map in the input list (titleText, userid, passwd)
						def titleText = idInfo.get('titleText')
						console.format("%s", "Please enter credentials for: ${titleText}\n")
						
						// Solicit userId for the resource and save the response in the output map
						def uidKey = idInfo.get('userid')
						userid = null
						while (userid == null || userid == "") {
							userid = console.readLine("%s","Enter username: ")
							String resp = console.readLine("%s","Is \"${userid}\" correct? (y/n): ")
							if (resp.substring(0,1).toLowerCase().equals("y")) {
								outmap.put(uidKey, userid)
								console.format("%s","Username \"${userid}\" accepted.\n")
							} else {
								console.format("%s","\nPlease re-enter username.\n")
								console.flush()
								userid = null
							}
						}	// End while
						
						// Solicit password for the resource and save the response in the output map
						def pwdKey = idInfo.get('passwd')
						passwd = null
						while (passwd == null || passwd == "") {
							char [] charPasswd = console.readPassword("%s","Enter password: ")
							if (charPasswd != null) {
								String passwd1 = new String(charPasswd)
								charPasswd = console.readPassword("%s","Re-enter password: ")
								String passwd2 = new String(charPasswd)
								passwd = new String(passwd1)
								if (passwd1.equals(passwd2)) {
									outmap.put(pwdKey, passwd)
									console.format("%s","Password accepted.\n")
								} else {
									console.format("%s","\nPasswords don't match; please try again.\n")
									console.flush()
									passwd = null
								}
							}
						}	// End while
						
						console.format("%s",SEPARATOR+"\n")
					}	// End closure for each map item in the list of maps
					console.format("%s","\n\nn*=*=*=*=*=*=*  END   Prompting for credentials *=*=*=*=*=*=*\n")
				}	// End try block
				catch (IOError e) {
					outmap = null
					println "**ERROR** JVM Console I/O failure; unable to solicit required resource authentication credentials from user."
					e.printStackTrace();
				}
			} 	// End console object is available		
			else  {		// System console object not available for this JVM
				println "**ERROR** JVM Console Object unavailable; unable to solicit required resource authentication credentials from user."
				outmap = null
			}	// End console object not available	
		}	// End TEXT mode support
	}	// End CTOR
	
	/***  Testing only: 
	public static void main(String[] args) {
		def resultMap = [:]
		def idList = [	[titleText:"DataSource SharedServices on SASServer1",userid:"userid0",passwd:"passwd0"], 
						[titleText:"LoginModule PFS",userid:"userid1",passwd:"passwd1"],
						[titleText:"WebSphere Admin Security",userid:"userid2",passwd:"passwd2"]  ]
		def frame = new CredentialsDialog(idList, resultMap, "GRAPHIC");
		resultMap.each { key,value -> println "Main: $key=$value" }
	}
	***/
}	// End CredentialsDialog class