// Copyright (c) 2002-2011, SAS Institute Inc., Cary, NC, USA, All Rights Reserved

package com.sas.config.weblogic;

import java.io.File;
import java.util.List;
import java.util.Map;

import com.sas.config.Resource;

class WebLogicResource extends Resource {
	public static final String TARGETS = "targets"
	public static final String DELETED = "deleted"

	public WebLogicResource() {
		super()
	}
	
	public WebLogicResource(Resource r) { super(r) }
	public WebLogicResource(File propertiesFile) { super(propertiesFile) }	
	
	/**
	 * isDeleted is a helper method to determine if a give resource has been deleted.
	 * 
	 * Criteria:
	 * 	Resource property files will never be deleted.  They will be marked in a way that can easily be tested in 
	 * 	velocity to determine if they need to be included in build command scripts or doc parts.
	 * 	1.	Resource with targets have a targets property.  If this property is an empty string it is considered deleted.  
	 *      The jython delete functions should handle these resources the same way.
	 * 	2.	Resources that do not have targets may have a deleted property.  This value will be true if the resource is deleted.
	 *  3.  Resources that do not match condition 1 or 2 listed above are considered active (non-deleted) resources.
	 */
	public boolean isDeleted() {
		String targets = getProperty(TARGETS)
		String deleted = getProperty(DELETED)
		
		if(targets == null && deleted == null) {
			return false
		}
		
		if(deleted == "true") {
			return true
		}
		
		if(targets != null && targets.trim().length() == 0) {
			return true
		}
		
		return false
	}
	
	protected List<Map<String,String>> getCommandLineChanges(String propertyName) {
		List<Map<String,String>> changes = new ArrayList<Map<String,String>>()
		
		String sChanges = getProperty(propertyName)
		if(sChanges != null && sChanges.trim().length() > 0) {
			sChanges.split(";").each() { entry ->
				Map<String,String> m = new HashMap<String,String>()	
				
				def item = entry.split(",")
				switch(item[0]) {
					case "ADD":
					m.put("type", "New")
					m.put("oldValue", "")
					m.put("newValue", item[1])
					break
					case "REPLACE":
					m.put("type", "Replace")
					m.put("oldValue", item[1])
					m.put("newValue", item[2])
					break
					case "DROP":
					m.put("type", "Non-standard")
					m.put("oldValue", item[1])
					m.put("newValue", item[1])
					break
					default:
					m.put("type", item[0])
					m.put("oldValue", "")
					m.put("newValue", "")
					break
				}
				changes.add(m)
			}
		}
		return changes
	}	
	
	protected List<Map<String,String>> getAttributeChanges(String propertyName) {
		List<Map<String,String>> changes = new ArrayList<Map<String,String>>()
		
		String sChanges = getProperty(propertyName)
		if(sChanges != null && sChanges.trim().length() > 0) {
			sChanges.split(";").each() { entry -> 
				Map<String,String> m = new HashMap<String,String>()		
				def item = entry.split(",")		
				m.put("attribute", item[0])
				m.put("oldValue", item[1])
				m.put("newValue", item[2])
				changes.add(m)
			}
		}		
		return changes		
	}		
}
