// Copyright (c) 2002-2011, SAS Institute Inc., Cary, NC, USA, All Rights Reserved

package com.sas.config.websphere
import com.sas.config.util.CommandProcess
import com.sas.config.util.ResourceProperties

// **************************************************************************************************************
// Class: Application
// Purpose: Encapsulate the attributes and methods used to manage and configure a WAS web application
// Methods:
//		deploy 		- install a webapp from an EAR
//		undeploy	- uninstall a webapp
//		start		- start a webapp
//		stop		- stop a webapp
// ****************************************************************************************************************

class Application extends Resource {
	// Class variables available to all methods:
	CommandProcess cmdProc
	def globalPropsFileName
	def resourcePropsFileName
	def servername
	def appname
	def pathtoear
    def loadorder
    def startingWeight
    def deployws
    def deployejb
    def filepermission
    def classloaderMode
    def classloaderPolicy
	def config_target_name	// SDW-provided calling Ant target name (configure, updateConfigure, updatePasswords, startMidtierServers, unconfigure, changeHost)
	def command 			// Used to hold the command string that will be executed via the Process class
	def options 			// ArrayList used to accumulate command options
	String [] optionsArray 	// String array used to pass options to command 
	String command_output	// Used to hold the stdout output from the command execution
	int    command_retcode  // Used to hold the return code from the command execution	

	// *************************************************************************************
	// CTOR: Initialize global properties object and obtain required global properties 
	// Global properties are fetched and initialized by the base Resource class
	public Application(GlobalProperties parmGblProps) {
		super(parmGblProps)		// Load the global properties using the abstract base class CTOR
		globalPropsFileName = parmGblProps.globalPropsFileName	// Get Global Properties file name
		cmdProc = new CommandProcess()	// Construct object used to run Ant exec task for external commands
	}
	
	// *************************************************************************************
	// Method to deploy a web application 
	// General flow:
	//	1. Run the installApplication.jy and setClassloader.jy Jython scripts
	public int deploy(String resourceName, ResourceProperties resourceProps) {
		logger.info("=================================================")
		// Required Resource Properties for this method:
		resourcePropsFileName 	= resourceProps.getPropsFileName()	// Use getter to fetch filename for the resource properties	
		servername				= resourceProps.getProperty("servername")
		appname 				= resourceProps.getProperty("appname")
	    pathtoear 				= resourceProps.getProperty("pathtoear")
        loadorder				= resourceProps.getProperty("loadorder")
        startingWeight			= resourceProps.getProperty("startingWeight")
        deployws				= resourceProps.getProperty("deployws")
        deployejb				= resourceProps.getProperty("deployejb")
        filepermission			= resourceProps.getProperty("filepermission")
        classloaderMode			= resourceProps.getProperty("classloaderMode")
        classloaderPolicy		= resourceProps.getProperty("classloaderPolicy")
		config_target_name		= resourceProps.getProperty("config.target.name")
		
		logger.info("Application-deploy: Install web application \"${appname}\"...")
		if (DEBUG) {
			logger.debug("Application-deploy: globalPropsFileName=${globalPropsFileName}")
			logger.debug("Application-deploy: resourcePropsFileName=${resourcePropsFileName}")
			logger.debug("Application-deploy: profilename=${profilename}")
			logger.debug("Application-deploy: appsrvnodename=${appsrvnodename}")
			logger.debug("Application-deploy: servername=${servername}")
			logger.debug("Application-deploy: script_ext=${script_ext}")
			logger.debug("Application-deploy: config_target_name=${config_target_name}")
			logger.debug("Application-deploy: websphere_wsadmin_jython=${websphere_wsadmin_jython}")
			logger.debug("Application-deploy: websphere_install_dir=${websphere_install_dir}")
			logger.debug("Application-deploy: websphere_wsadmin_classpath=${websphere_wsadmin_classpath}")
			logger.debug("Application-deploy: dmgrprotocol=${dmgrprotocol}")
			logger.debug("Application-deploy: dmgrport=${dmgrport}")
			logger.debug("Application-deploy: webappsrv_admin_host=${webappsrv_admin_host}")
			logger.debug("Application-deploy: wsadminOptionsList=${wsadminOptionsList}")
			logger.debug("Application-deploy: appname=${appname}")
			logger.debug("Application-deploy: pathtoear=${pathtoear}")
			logger.debug("Application-deploy: loadorder=${loadorder}") 
			logger.debug("Application-deploy: startingWeight=${startingWeight}")
			logger.debug("Application-deploy: deployws=${deployws}")
			logger.debug("Application-deploy: deployejb=${deployejb}")
			logger.debug("Application-deploy: filepermission=${filepermission}")
			logger.debug("Application-deploy: classloaderMode=${classloaderMode}")
			logger.debug("Application-deploy: classloaderPolicy=${classloaderPolicy}")						
		}
		
		// 1. Run the installApplication.jy and setClassloader.jy Jython scripts ("listOfProcs" property contains these 2 Jython proc names) 
		logger.info("Application-deploy: Run the installApplication.jy and setClassloader.jy Jython scripts using the wsadmin${script_ext} command...")
		command = "${websphere_install_dir}${File.separator}bin${File.separator}wsadmin${script_ext}"
		options = []
		wsadminOptionsList.each{ options.add(it) }
		options.addAll( [ "-f", "${websphere_scripting_jython}${File.separator}callMultJythonFunction.jy",
						  "${websphere_scripting_jython}",
						  "${resourcePropsFileName}",
						  "${globalPropsFileName}" ] )
		optionsArray = options.toArray()
		command_retcode = cmdProc.run(".", command, optionsArray, "ApplicationDeploy_${appname}_output", "ApplicationDeploy_${appname}_result")	// Run wsadmin client to execute Jython procs
		command_output  = cmdProc.process_output	// Get output from the command 				  
		logger.info("Application-deploy: Command Return code: " + command_retcode)
		if (command_retcode == 0) {			// Command return code of zero indicates wsadmin command ran to completion			
			
			// Check if the installApplication.jy Jython proc completed successfully (Message "ADMA5013I: Application {0} installed successfully.")
			if (  command_output.contains("ADMA5013I") ) { 
				method_MSG =  "Application-deploy: ===> installApplication.jy COMPLETED SUCCESSFULLY."
				method_RC = 1
				logger.info("${method_MSG}")
			} else {
				method_MSG = "**SEVERE** Application-deploy: installApplication.jy Jython processing FAILED; unable to continue."
				method_RC = -99
				logger.error("${method_MSG}")
				return method_RC
			}
			
			// Check if the setClassloader.jy Jython proc completed successfully
			if (  command_output.contains("setClassloader: Classloader modes and policy for \"${appname}\" have been set.") ) { 
				method_MSG =  "Application-deploy: ===> setClassloader.jy COMPLETED SUCCESSFULLY."
				method_RC = 1
				logger.info("${method_MSG}")
			} else {
				method_MSG = "**SEVERE** Application-deploy: setClassloader.jy Jython processing FAILED; unable to continue."
				method_RC = -99
				logger.error("${method_MSG}")
			}
			
		} else {
			method_RC = -99
			method_MSG = "**SEVERE** Application-deploy: wsadmin${script_ext} command failed; RC=" + command_retcode
			logger.error("${method_MSG}")
		}
		return method_RC	// Return to driver indicating failure
	}	// END deploy method

	// *************************************************************************************
	// Method to undeploy a web application 
	// General flow:
	//	1. Run the uninstallApplication.jy Jython script
	public int undeploy(String resourceName, ResourceProperties resourceProps) {
		logger.info("=================================================")
		// Required Resource Properties for this method:
		resourcePropsFileName 	= resourceProps.getPropsFileName()	// Use getter to fetch filename for the resource properties
		appname 				= resourceProps.getProperty("appname")
		servername				= resourceProps.getProperty("servername")
		config_target_name		= resourceProps.getProperty("config.target.name")
		
		logger.info("Application-undeploy: Uninstall web application \"${appname}\"...")
		if (DEBUG) {
			logger.debug("Application-undeploy: globalPropsFileName=${globalPropsFileName}")
			logger.debug("Application-undeploy: resourcePropsFileName=${resourcePropsFileName}")
			logger.debug("Application-undeploy: profilename=${profilename}")
			logger.debug("Application-undeploy: appsrvnodename=${appsrvnodename}")
			logger.debug("Application-undeploy: servername=${servername}")
			logger.debug("Application-undeploy: appname=${appname}")
			logger.debug("Application-undeploy: script_ext=${script_ext}")
			logger.debug("Application-undeploy: config_target_name=${config_target_name}")
			logger.debug("Application-undeploy: websphere_wsadmin_jython=${websphere_wsadmin_jython}")
			logger.debug("Application-undeploy: websphere_install_dir=${websphere_install_dir}")
			logger.debug("Application-undeploy: websphere_wsadmin_classpath=${websphere_wsadmin_classpath}")
			logger.debug("Application-undeploy: dmgrprotocol=${dmgrprotocol}")
			logger.debug("Application-undeploy: dmgrport=${dmgrport}")
			logger.debug("Application-undeploy: webappsrv_admin_host=${webappsrv_admin_host}")
			logger.debug("Application-undeploy: wsadminOptionsList=${wsadminOptionsList}")			
		}
		
		// 1. Run the uninstallApplication.jy Jython script 
		command = "${websphere_install_dir}${File.separator}bin${File.separator}wsadmin${script_ext}"
		options = []
		wsadminOptionsList.each{ options.add(it) }
		options.addAll( [ "-f", "${websphere_scripting_jython}${File.separator}callJythonFunction.jy",
						  "${websphere_scripting_jython}",
						  "uninstallApplication",
						  "${resourcePropsFileName}",
						  "${globalPropsFileName}" ] )
		optionsArray = options.toArray()
		command_retcode = cmdProc.run(".", command, optionsArray, "ApplicationUndeploy_${appname}_output", "ApplicationUndeploy_${appname}_result")	// Run wsadmin client to execute Jython procs
		command_output  = cmdProc.process_output	// Get output from the command 				  
		logger.info("Application-undeploy: Command Return code: " + command_retcode)
		if (command_retcode == 0) {			// Command return code of zero indicates wsadmin command ran to completion			
			// Check if the uninstallApplication.jy Jython proc completed successfully (Message "")
			if (  command_output.contains("ADMA5106I") ||
				  command_output.contains("Application \"${appname}\" does not exist") ) { 
				method_MSG =  "Application-undeploy: ===> uninstallApplication.jy COMPLETED SUCCESSFULLY."
				method_RC = 1
				logger.info("${method_MSG}")
			} else {
				method_MSG = "**SEVERE** Application-undeploy: uninstallApplication.jy Jython processing FAILED; unable to continue."
				method_RC = -99
				logger.error("${method_MSG}")
			}
		} else {
			method_RC = -99
			method_MSG = "**SEVERE** Application-undeploy: wsadmin${script_ext} command failed; RC=" + command_retcode
			logger.error("${method_MSG}")
		}
		return method_RC	// Return to driver indicating failure
	}	// END undeploy method

	// *************************************************************************************
	// Method to start a web application 
	// General flow:
	//	1. Run the startApplication.jy Jython script
	public int start(String resourceName, ResourceProperties resourceProps) {
		logger.info("=================================================")
		// Required Resource Properties for this method:
		resourcePropsFileName 	= resourceProps.getPropsFileName()	// Use getter to fetch filename for the resource properties	
		appname 				= resourceProps.getProperty("appname")
		servername				= resourceProps.getProperty("servername")
		config_target_name		= resourceProps.getProperty("config.target.name")

		logger.info("Application-start: Start web application \"${appname}\"...")
		if (DEBUG) {
			logger.debug("Application-start: globalPropsFileName=${globalPropsFileName}")
			logger.debug("Application-start: resourcePropsFileName=${resourcePropsFileName}")
			logger.debug("Application-start: profilename=${profilename}")
			logger.debug("Application-start: appsrvnodename=${appsrvnodename}")
			logger.debug("Application-start: servername=${servername}")
			logger.debug("Application-start: appname=${appname}")
			logger.debug("Application-start: script_ext=${script_ext}")
			logger.debug("Application-start: config_target_name=${config_target_name}")
			logger.debug("Application-start: websphere_wsadmin_jython=${websphere_wsadmin_jython}")
			logger.debug("Application-start: websphere_install_dir=${websphere_install_dir}")
			logger.debug("Application-start: websphere_wsadmin_classpath=${websphere_wsadmin_classpath}")
			logger.debug("Application-start: dmgrprotocol=${dmgrprotocol}")
			logger.debug("Application-start: dmgrport=${dmgrport}")
			logger.debug("Application-start: webappsrv_admin_host=${webappsrv_admin_host}")
			logger.debug("Application-start: wsadminOptionsList=${wsadminOptionsList}")			
		}
		
		// 1. Run the startApplication.jy Jython script 
		logger.info("Application-start: Run the startApplication.jy Jython script to start \"${appname}\" using the wsadmin${script_ext} command...")
		command = "${websphere_install_dir}${File.separator}bin${File.separator}wsadmin${script_ext}"
		options = []
		wsadminOptionsList.each{ options.add(it) }
		options.addAll( [ "-f", "${websphere_scripting_jython}${File.separator}callJythonFunction.jy",
						  "${websphere_scripting_jython}",
						  "startApplication",
						  "${resourcePropsFileName}",
						  "${globalPropsFileName}" ] )
		optionsArray = options.toArray()
		command_retcode = cmdProc.run(".", command, optionsArray, "ApplicationStart_${appname}_output", "ApplicationStart_${appname}_result")	// Run wsadmin client to execute Jython proc
		command_output  = cmdProc.process_output	// Get output from the command 				  
		logger.info("Application-start: Command Return code: " + command_retcode)
		if (command_retcode == 0) {	// Command return code of zero indicates wsadmin command ran to completion			
			// Check if the startApplication.jy Jython proc completed successfully 
			if (  command_output.contains("is already started") ||
				  command_output.contains("has been started.") ||
				  command_output.contains("Application may already have been started") ) { 
				method_MSG =  "Application-start: ===> startApplication.jy COMPLETED SUCCESSFULLY."
				method_RC = 1
				logger.info("${method_MSG}")
			} else {
				method_MSG = "**SEVERE** Application-start: startApplication.jy Jython processing FAILED; unable to continue."
				method_RC = -99
				logger.error("${method_MSG}")
			}
		} else {
			method_RC = -99
			method_MSG = "**SEVERE** Application-start: wsadmin${script_ext} command failed; RC=" + command_retcode
			logger.error("${method_MSG}")
		}
		return method_RC	// Return to driver indicating failure
	}	// END start method

	// *************************************************************************************
	// Method to stop a web application 
	// General flow:
	//	1. Run the stopApplication.jy Jython script
	public int stop(String resourceName, ResourceProperties resourceProps) {
		logger.info("=================================================")
		// Required Resource Properties for this method:
		resourcePropsFileName 	= resourceProps.getPropsFileName()	// Use getter to fetch filename for the resource properties	
		appname 				= resourceProps.getProperty("appname")
		servername				= resourceProps.getProperty("servername")
		config_target_name		= resourceProps.getProperty("config.target.name")
		
		logger.info("Application-stop: stop web application \"${appname}\"...")
		if (DEBUG) {
			logger.debug("Application-stop: globalPropsFileName=${globalPropsFileName}")
			logger.debug("Application-stop: resourcePropsFileName=${resourcePropsFileName}")
			logger.debug("Application-stop: profilename=${profilename}")
			logger.debug("Application-stop: appsrvnodename=${appsrvnodename}")
			logger.debug("Application-stop: servername=${servername}")
			logger.debug("Application-stop: appname=${appname}")
			logger.debug("Application-stop: script_ext=${script_ext}")
			logger.debug("Application-stop: config_target_name=${config_target_name}")
			logger.debug("Application-stop: websphere_wsadmin_jython=${websphere_wsadmin_jython}")
			logger.debug("Application-stop: websphere_install_dir=${websphere_install_dir}")
			logger.debug("Application-stop: websphere_wsadmin_classpath=${websphere_wsadmin_classpath}")
			logger.debug("Application-stop: dmgrprotocol=${dmgrprotocol}")
			logger.debug("Application-stop: dmgrport=${dmgrport}")
			logger.debug("Application-stop: webappsrv_admin_host=${webappsrv_admin_host}")
			logger.debug("Application-stop: wsadminOptionsList=${wsadminOptionsList}")			
		}
		
		// 1. Run the stopApplication.jy Jython script 
		logger.info("Application-stop: Run the stopApplication.jy Jython script using the wsadmin${script_ext} command...")
		command = "${websphere_install_dir}${File.separator}bin${File.separator}wsadmin${script_ext}"
		options = []
		wsadminOptionsList.each{ options.add(it) }
		options.addAll( [ "-f", "${websphere_scripting_jython}${File.separator}callJythonFunction.jy",
						  "${websphere_scripting_jython}",
						  "stopApplication",
						  "${resourcePropsFileName}",
						  "${globalPropsFileName}" ] )
		optionsArray = options.toArray()				  
		command_retcode = cmdProc.run(".", command, optionsArray, "ApplicationStop_${appname}_output", "ApplicationStop_${appname}_result")	// Run wsadmin client to execute Jython proc
		command_output  = cmdProc.process_output	// Get output from the command 				  
		logger.info("Application-stop: Command Return code: " + command_retcode)
		if (command_retcode == 0) {	// Command return code of zero indicates wsadmin command ran to completion			
			// Check if the stopApplication.jy Jython proc completed successfully 
			if (  command_output.contains("has been stopped.") ||
				  command_output.contains("is already stopped.") ||
				  command_output.contains("not started.") ||
				  command_output.contains("does not exist.") ) { 
				method_MSG =  "Application-stop: ===> stopApplication.jy COMPLETED SUCCESSFULLY."
				method_RC = 1
				logger.info("${method_MSG}")
			} else {
				method_MSG = "**SEVERE** Application-stop: stopApplication.jy Jython processing FAILED; unable to continue."
				method_RC = -99
				logger.error("${method_MSG}")
			}
		} else {
			method_RC = -99
			method_MSG = "**SEVERE** Application-stop: wsadmin${script_ext} command failed; RC=" + command_retcode
			logger.error("${method_MSG}")
		}
		return method_RC	// Return to driver indicating failure
	}	// END stop method

}	// END Application class