// Copyright (c) 2002-2011, SAS Institute Inc., Cary, NC, USA, All Rights Reserved

package com.sas.config.websphere 
import com.sas.config.util.CommandProcess
import com.sas.config.util.ResourceProperties

// **************************************************************************************************************
// Class: DmgrProfile
// Purpose: Encapsulate the attributes and methods used to manipulate the dmgr profile for WAS
// Methods:
//		create 		- create the dmgr profile
//		delete 		- delete the dmgr profile
//		start  		- start the dmgr server 
//		stop		- stop the dmgr server
//		update		- update the dmgr server Http ports and JVM generic arguments
//		updateProps	- update the dmgr profile security and timeout properties
//		validate	- validate the profileRegistry
// ****************************************************************************************************************

public class DmgrProfile extends Resource {
	// Class variables available to all methods:
	def config_target_name	// SDW-provided calling Ant target name (configure, updateConfigure, updatePasswords, startMidtierServers, unconfigure, changeHost)
	def globalPropsFileName
	def resourcePropsFileName
	String create_dmgrprofile_response_file
	CommandProcess cmdProc	// Object used to run an external command using Ant exec task
	String command 			// Used to hold the command string that will be executed via the Process class
	String [] optionsArray	// Used to hold the command options 
	String command_output	// Used to hold the stdout output from the command execution
	int    command_retcode  // Used to hold the return code from the command execution
	
	// *************************************************************************************
	// CTOR: Initialize global properties object and obtain required global properties
	// Global properties from the globalProperties file are obtained and initialized by base Resource class 
	public DmgrProfile(GlobalProperties parmGblProps) {
		super(parmGblProps)
		globalPropsFileName = parmGblProps.globalPropsFileName	// Get Global Properties file name
		cmdProc = new CommandProcess()	// Construct object used to run Ant exec task for external commands
	}	// END CTOR
	
	// *************************************************************************************
	// Method to create the dmgr profile
	// General flow:
	//	1. Validate profileRegistry
	//	2. Create dmgr profile
	//  3. Update profile properties for security and timeout values
	//	4. Stop dmgr, if running
	//	5. Update dmgr server Http ports and JVM arguments
	//	6. Start dmgr
	public int create(String resourceName, ResourceProperties resourceProps) {
		logger.info("DmgrProfile-create: Create the dmgr profile \"${dmgrprofilename}\"...")
		// Method variables
		create_dmgrprofile_response_file = resourceProps.getProperty("create.dmgrprofile.response.file")
		config_target_name = resourceProps.getProperty("config.target.name")
		logger.info("DmgrProfile-create: create.dmgrprofile.response.file=${create_dmgrprofile_response_file}")
		
		// 1. Validate profileRegistry
		def method_RC = validate(resourceName,resourceProps)	// Validate the profileRegistry and set profileStatus to "proceed", "reuse", "error", "warning", or "unknown"	
		if (method_RC != 1) {	// If status of profileRegistry indicates an error,
			return method_RC	// Return to driver indicating failure
		}	
		
		// 2. Create dmgr profile
		logger.info("=================================================")
		logger.info("DmgrProfile-create: Issue manageprofiles command to create dmgr profile...")
		command = "${websphere_install_dir}${File.separator}bin${File.separator}manageprofiles${script_ext}"
		optionsArray = ["-response", "${create_dmgrprofile_response_file}"]
		command_retcode = cmdProc.run(".", command, optionsArray, "DmgrProfleCreate_output", "DmgrProfileCreate_result")	// Run the manageprofiles command to create the dmgr profile
		command_output  = cmdProc.process_output				// Get the output from the command
		logger.info("DmgrProfile-create: Command Return code: " + command_retcode)
				
		// Evaluate results from manageprofiles command used to create the dmgr profile:
		// Determine if we can look at text output from the manageprofiles command when locale == "en", or only at the return code when locale != "en"
		def profileStatus = ""
		if (os_localhost_default_locale == "en") {
			if (command_output.contains("INSTCONFSUCCESS: Success: Profile ${dmgrprofilename} now exists."))
				profileStatus = "success"
			else if (command_output.contains("profileName: ${dmgrprofilename} is already in use. Specify another name."))				
				profileStatus = "reuse"
			else if (command_output.contains("profileName: ${dmgrprofilename} is already in use."))
				profileStatus = "validateAndUpdateRegistryRequired" 
			else if (command_output.contains("INSTCONFFAILED: The profile could not be created."))
				profileStatus = "error"
			else
				profileStatus = "unknown" 
		} else {	// locale is non-English
			if (command_retcode == 0)  // Assume if the return code from manageprofiles -create was good, that this is success
				profileStatus = "success"
			else // Otherwise, assume a non-zero return code from manageprofiles -create is an error
				profileStatus = "error"
		}	
		// Prepare any error messages and return codes, if an severe error has occurred.
		logger.info("DmgrProfile-create: manageprofiles create \"${dmgrprofilename}\" profile status is: ${profileStatus}...")
		switch (profileStatus) {
			case ("success"):
				method_MSG = "DmgrProfile-create: Creation of profile \"${dmgrprofilename}\" was SUCCESSFUL; processing can continue."
				method_RC = 1
				break
				
			case ("reuse"):
				method_MSG = "DmgrProfile-create: A valid profile named \"${dmgrprofilename}\" already exists; it is being REUSED."
				method_RC = 1
				break

			case ("validateAndUpdateRegistryRequired"):
				method_MSG = "**SEVERE** DmgrProfile-create: The \"${dmgrprofilename}\" profile is invalid or possibly corrupted.\n" +
							 "           Please run the WebSphere \"${websphere_install_dir}${File.separator}bin${File.separator}manageprofiles${script_ext} -validateAndUpdateRegistry\" command and click \"Retry\" after the profileRegistry has been validated."
				method_RC = -99	
				break

			case ("error"):
				method_MSG = "**SEVERE** DmgrProfile-create: The \"manageprofiles${script_ext}\" command has generated a serious error.\n" +
							 "           Please check the \"${websphere_install_dir}${File.separator}logs${File.separator}manageprofiles\" log directory for the reason before attempting a \"Retry\"."
				method_RC = -99
				break

			default:	// "unknown"
				method_MSG = "**SEVERE** DmgrProfile-create: The WebSphere \"manageprofiles${script_ext}\" command has generated an unknown error.\n" +
							 "           Please ensure the WebSphere \"profileRegistry.xml\" file and the \"profiles\" directory are valid and do not contain references to the \"${dmgrprofilename}\" profile before attempting \"Retry\"\n." +
							 "           Also, ensure your userId has write privileges to the appropriate WebSphere directories."
				method_RC = -99
		}	// END switch
		if (method_RC < 1) { 	// If status of profileRegistry indicates error,
			logger.error("${method_MSG}")
			return method_RC	// Return to driver indicating failure
		} else {
			logger.info("${method_MSG}")
			logger.info("DmgrProfile-create: ===> manageprofiles command to create dmgr profile COMPLETED SUCCESSFULLY.") 
		}	
		
		// 3. Update profile properties for security and timeout values
		method_RC = updateProps(resourceName, resourceProps)
		if (method_RC != 1) {	// If update of security or timeout values indicates an error,
			logger.error("${method_MSG}")
			return method_RC	// Return to driver indicating failure
		} else {
			logger.info("${method_MSG}")
		}
			
		// 4. Stop dmgr, if running
		method_RC = stop(resourceName, resourceProps) 
		if (method_RC != 1) { 	// If status of profileRegistry indicates an error,
			logger.error("${method_MSG}")
			return method_RC	// Return to driver indicating failure
		} else {
			logger.info("${method_MSG}")
		}
		
		// 5. Update dmgr server Http ports and JVM arguments
		method_RC = update(resourceName, resourceProps) 
		if (method_RC != 1) {	// If update of Http ports and JVM args had an error,
			logger.error("${method_MSG}")
			return method_RC	// Return to driver indicating failure
		} else {
			logger.info("${method_MSG}")
		}
			
		// 6. Start dmgr
		method_RC = start(resourceName, resourceProps)
		if (method_RC != 1)		// If startManager failed,
			logger.error("${method_MSG}")
		else
			logger.info("${method_MSG}")
		return method_RC
	}	// END create method	

	// *************************************************************************************
	// Method to delete the dmgr profile
	// General flow:
	//  1. Determine if the profile to be deleted actually exists
	//	2. Stop dmgr, if running
	//	3. Delete dmgr profile
	//	4. Remove dmgr profile directory	
	public int delete(String resourceName, ResourceProperties resourceProps) {
		logger.info("DmgrProfile-delete: Delete profile \"${dmgrprofilename}\"...")
		config_target_name = resourceProps.getProperty("config.target.name")
		
		// 1. Determine if the profile to be deleted actually exists:
		def profileDirectory = "${websphere_profile_dir}${File.separator}${dmgrprofilename}"
		if (!(new File(profileDirectory).exists())) {
			method_MSG = "**SEVERE** DmgrProfile-delete: WAS profile directory \"${profileDirectory}\" does not exist;  Nothing to delete."
			method_RC = -99
 			logger.error("${method_MSG}")
			return method_RC	// Return to driver indicating failure      		
      	}

		//	2. Stop dmgr, if running
		method_RC = stop(resourceName, resourceProps) 
		if (method_RC != 1) {	// If stopManager command failed,
			logger.error("${method_MSG}")
			return method_RC	// Return to driver indicating failure
		} else {
			logger.info("${method_MSG}")
		}					
		
		//	3. Delete dmgr profile
		logger.info("=================================================")
		logger.info("DmgrProfile-delete: Issue manageprofiles command to delete profile \"${dmgrprofilename}\"...")
		command = "${websphere_install_dir}${File.separator}bin${File.separator}manageprofiles${script_ext}" 
		optionsArray = ["-delete", "-profileName", "${dmgrprofilename}"]
		command_retcode = cmdProc.run(".", command, optionsArray, "DmgrProfileDelete_output", "DmgrProfileDelete_result")	// Run the manageprofiles command to delete the dmgr profile
		command_output  = cmdProc.process_output				// Get the output from the command
		
		// Evaluate results from manageprofiles command used to delete the dmgr profile:
		// Determine if we can look at text output from the manageprofiles command when locale == "en", or only at the return code when locale != "en"
		def profileStatus = ""
		if (os_localhost_default_locale == "en") {	// Locale is English, we can look at text in the output message from manageprofiles
			if (command_output.contains("SUCCESS: Success: The profile no longer exists."))
				profileStatus = "success"
			else 
				profileStatus = "error"
		} else {		// Locale is non-English, we have to base success on return codes only
			if (command_retcode == 0)  // Assume if the return code from manageprofiles -delete was good, that this is success
				profileStatus = "success"
			else 		// Otherwise, assume a non-zero return code from manageprofiles -delete is an error
				profileStatus = "error"			
		}
		
		//	4. Remove dmgr profile directory
		logger.info("=================================================")		
		if (profileStatus.equals("success")) {
			logger.info("DmgrProfile-delete: Profile \"${dmgrprofilename}\" has been deleted; now remove the associated profiles directory...")
			try {
				String dmgrProfileDirectoryName = "${websphere_profile_dir}${File.separator}${dmgrprofilename}"
				File dmgrProfileDirectory = new File(dmgrProfileDirectoryName);
				org.apache.commons.io.FileUtils.deleteDirectory(dmgrProfileDirectory);	// Returns nothing whether successful or not
				logger.info("DmgrProfile-delete: dmgr profile directory \"${dmgrProfileDirectoryName}\" has been removed.")
				method_RC = 1;
			}
       		catch (IOException e) {
       			method_RC = -99;
  				logger.error(e.printStackTrace())
       		}		
			method_MSG = "DmgrProfile-delete: ===> Delete of profile \"${dmgrprofilename}\" COMPLETED SUCCESSFULLY" 
		} else {
			method_MSG = "DmgrProfile-delete: ===> Delete of profile \"${dmgrprofilename}\" FAILED; Check the \"${websphere_install_dir}${File.separator}logs${File.separator}manageprofiles\" log directory for the reason."
			method_RC = -99
		}
		if (method_RC != 1) {	// If removal of dmgr profile directory failed,
			logger.error("${method_MSG}")
		} else {
			logger.info("${method_MSG}")
		}
		return method_RC
	}	// END delete method

	// ***********************************************************************************
	// Method to start the dmgr server via startManager command
	public int start(String resourceName, ResourceProperties resourceProps) {
		logger.info("=================================================")
		logger.info("DmgrProfile-start: Start the ${dmgr} server using startManager command...")
		config_target_name = resourceProps.getProperty("config.target.name")
		command = "${websphere_install_dir}${File.separator}bin${File.separator}startManager${script_ext}"
		optionsArray = ["-profileName", "${dmgrprofilename}"]
		command_retcode = cmdProc.run(".", command, optionsArray, "DmgrProfileStart_output", "DmgrProfileStart_result")	// Run the manageprofiles command to start the dmgr server
		command_output  = cmdProc.process_output				// Get the output from the command
				
		// Check if "ADMU3000I: Server {0} open for e-business; process id is {1}" or "ADMU3027E: An instance of the server may already be running: {0}"  
		if (command_output.contains("ADMU3000I") || command_output.contains("ADMU3027E")) { 
			method_MSG = "DmgrProfile-start: ===> startManager${script_ext} COMPLETED SUCCESSFULLY"
			method_RC = 1
		} else { 
			method_MSG = "**SEVERE** DmgrProfile-start: startManager${script_ext} command FAILED; Review WebSphere logs at \"${websphere_install_dir}${File.separator}profiles${File.separator}${dmgrprofilename}${File.separator}logs\" for analysis."
			method_RC = -99
		}		
		return method_RC   		
	}	// END start method
	
	// ***********************************************************************************
	// Method to stop the dmgr server via stopManager command
	public int stop(String resourceName, ResourceProperties resourceProps) {
		logger.info("=================================================")
		logger.info("DmgrProfile-stop: Stop the ${dmgr} server using stopManager command...")
		config_target_name = resourceProps.getProperty("config.target.name")
		command = "${websphere_install_dir}${File.separator}bin${File.separator}stopManager${script_ext}"
		optionsArray = ["-profileName", "${dmgrprofilename}"]
		command_retcode = cmdProc.run(".", command, optionsArray, "DmgrManagerStop_output", "DmgrManagerStop_result")	// Run the manageprofiles command to stop the dmgr server
		command_output  = cmdProc.process_output	// Get output from the command
				
		// Check if "ADMU4000I: Server {0} stop completed." or "ADMU0509I: The {0} "{1}" cannot be reached. It appears to be stopped."  
		if (command_output.contains("ADMU4000I") || command_output.contains("ADMU0509I")) {
			method_MSG = "DmgrProfile-stop: ===> stopManager${script_ext} COMPLETED SUCCESSFULLY"
			method_RC = 1
		} else {
			method_MSG = "**SEVERE** DmgrProfile-stop: stopManager${script_ext} command FAILED; Review WebSphere logs at \"${websphere_install_dir}${File.separator}profiles${File.separator}${dmgrprofilename}${File.separator}logs\" for analysis."
			method_RC = -99
		}
		return method_RC 	
	}	// END stop method

	// ***********************************************************************************
	// Method to validate the profileRegistry, return 1: success, others: failed.
	//		proceed - dmgr profile name is valid
	//		error	- dmgr profile appears in the list of profiles that are not valid
	//		warning	- dmgr profile seems to be ok, but list of profiles that are not valid returned from the manageprofiles -validateRegistry command
	//		unknown - an unknown error has been returned by the manageprofiles -validateRegistry command
	public int validate(String resourceName, ResourceProperties resourceProps) {
		logger.info("=================================================")		
		logger.info("DmgrProfile-validate: Validate the profileRegistry...") 
		config_target_name = resourceProps.getProperty("config.target.name")
		// Issue manageprofiles command to validate the profileRegistry...
		command = "${websphere_install_dir}${File.separator}bin${File.separator}manageprofiles${script_ext}" 
		optionsArray = ["-validateRegistry"]
		command_retcode = cmdProc.run(".", command, optionsArray, "DmgrProfileValidate_output", "DmgrProfileValidate_result")	// Run the manageprofiles command to validate the profile registry		
		command_output  = cmdProc.process_output	// Get output from the command
		logger.info("DmgrProfile-validate: Command Return code: " + command_retcode)
		if (command_retcode == 0)
			logger.info("DmgrProfile-validate: ===> manageprofiles${script_ext} -validateRegistry command COMPLETED SUCCESSFULLY")		

		// Determine if we can look at text output from the manageprofiles command when locale == "en", or only the return code when locale != "en"
		String profileStatus = ""
		if (os_localhost_default_locale == "en")  {
			if (command_output.contains("All of the profiles in the registry are valid."))  {
		        profileStatus = "proceed"
			} else {
			    if (command_output.contains("A list of profiles that are not valid follows:"))  {
			        if (command_output.contains(dmgrprofilename))
			            profileStatus = "error"
			        else
			            profileStatus = "warning"
			    } else {
			        profileStatus = "unknown"
			    }
			}
		} else {	// locale is non-English
			if (command_retcode == 0) 
				profileStatus = "proceed"
			else 
				profileStatus = "error"
		}
		def methodRC = 0;
		switch (profileStatus) {
			case ("proceed"):
				method_MSG = "DmgrProfile-create: WebSphere profile entries, if any, have been validated; An attempt to create \"${dmgrprofilename}\" will proceed..."
				methodRC = 1
				break

			case ("warning"):
				method_MSG = "**WARNING** DmgrProfile-create The WebSphere profiles directory contains one or more possibly invalid profiles.\n" +
							  "            This will not affect continued configuration of WebSphere for SAS, but this problem warrants further investigation."
				methodRC = 1
				break
				
			case ("error"):
				method_MSG = "**SEVERE** DmgrProfile-create \"${dmgrprofilename}\" is invalid or possibly corrupted.\n " +
							 "           Please run the \"${websphere_install_dir}"+File.separator+"bin"+File.separator+"manageprofiles${script_ext} -validateAndUpdateRegistry\" to attempt correction of the WAS profileRegistry.xml file."
				methodRC = -99
				break
				
			default:	// "unknown"
				method_MSG = "**SEVERE** DmgrProfile-create An unknown error has occurred attempting to validate the WAS profileRegistry.xml file.\n" +
							 "           Please run the \"${websphere_install_dir}"+File.separator+"bin"+File.separator+"manageprofiles${script_ext} -validateAndUpdateRegistry\" to attempt correction of the WAS profileRegistry.xml file.\n" +
							 "          Also, ensure your userId has write privileges to the appropriate WebSphere directories."
				methodRC = -99
		}
		if (methodRC != 1) {	// If status of profileRegistry indicates an error,
			logger.error("${method_MSG}")
		} else {
			logger.info("${method_MSG}")
		}
		return methodRC;
	}	// END validate method
	
	// ******************************************************************************************************
	// Method updateProps to update the dmgr profile properties for SOAP and/or RMI security & timeout values
	public int updateProps(String resourceName, ResourceProperties resourceProps) {
		logger.info("=================================================")
		logger.info("DmgrProfile-updateProps: Update the SOAP and/or RMI security and timeout properties for the dmgr profile ...")
		config_target_name = resourceProps.getProperty("config.target.name")
		ResourceProperties credentialsProps		// Object used to read the websphere_scripting_credentials properties file
		String webappsrv_admin_userid = ""		// WAS Admin userId, required if security is enabled
		String webappsrv_admin_passwd = ""		// WAS Admin password, required if security is enabled
		logger.info("DmgrProfile-updateProps: WAS Admin Security is enabled? ${webappsrv_admin_security_is_enabled}")
		if (webappsrv_admin_security_is_enabled) {
			credentialsProps = new ResourceProperties(websphere_scripting_credentials)
			webappsrv_admin_userid = credentialsProps.getProperty("DmgrProfile.create_CELL_${dmgrprofilename}_userid")
			webappsrv_admin_passwd = credentialsProps.getProperty("DmgrProfile.create_CELL_${dmgrprofilename}_passwd")
		}
		
		if (DEBUG)	{
			logger.debug("DmgrProfile-updateProps: webappsrv_admin_security_is_enabled=${webappsrv_admin_security_is_enabled}")
			logger.debug("DmgrProfile-updateProps: dmgrprotocol=${dmgrprotocol}")
			logger.debug("DmgrProfile-updateProps: dmgrsoapprops=${dmgrsoapprops}")
			logger.debug("DmgrProfile-updateProps: dmgrsoaptimeout=${dmgrsoaptimeout}")
			logger.debug("DmgrProfile-updateProps: dmgrrmiprops=${dmgrrmiprops}")
			logger.debug("DmgrProfile-updateProps: dmgrrmitimeout=${dmgrrmitimeout}")
			logger.debug("DmgrProfile-updateProps: dmgripcprops=${dmgripcprops}")
			logger.debug("DmgrProfile-updateProps: dmgripctimeout=${dmgripctimeout}")
			logger.debug("DmgrProfile-updateProps: config_target_name=${config_target_name}")
		}			

		// Update the soap.client.props file (I found you have to update this, even if RMI protocol was selected because the stopManager,
		// stopServer, etc. go looking for the credentials in soap.client.props, although the wsadmin with -conntype RMI only looks in 
		// sas.client.props for the -user, -password credentials.)
		ResourceProperties soapProps = new ResourceProperties(dmgrsoapprops)	// Use the ResourceProperties object to manipulate the WAS soap.client.props file
		// Make a backup of the existing soap.client.props file before alteration:
		logger.info("DmgrProfile-updateProps: Create backup of \"soap.client.props\" file before alteration.")
		soapProps.backup()
		
		// Update the timeout value for SOAP communications
		logger.info("DmgrProfile-updateProps: Update SOAP timeout property in \"${dmgrsoapprops}\" file...")
		soapProps.setPropertyValue("com.ibm.SOAP.requestTimeout", dmgrsoaptimeout)			
		
		// If WAS Admin Security is enabled, update the soap.client.props security properties to avoid anoying userID/password prompts
		if (webappsrv_admin_security_is_enabled) {
			logger.info("DmgrProfile-updateProps: Set the SOAP security properties in \"soap.client.props\" file...")
			soapProps.setPropertyValue("com.ibm.SOAP.securityEnabled", "true")
			soapProps.setPropertyValue("com.ibm.SOAP.loginSource", "none") 
			soapProps.setPropertyValue("com.ibm.SOAP.loginUserid", webappsrv_admin_userid)
			
			// Decode the admin password using the SAS decoder utility
			String decodedPassword = com.sas.util.SasPasswordString.decode(webappsrv_admin_passwd);
			soapProps.setPropertyValue("com.ibm.SOAP.loginPassword", decodedPassword)
		}
				
		// Write the property updates to the soap.client.props file
		logger.info("DmgrProfile-updateProps: Update the SOAP security and/or timeout properties file \"soap.client.props\"...")
		if (soapProps.writeProps() == 1) {
			method_RC = 1
			method_MSG = "DmgrProfile-updateProps: Update of SOAP \"soap.client.props\" file succeeded."
			// Run the WAS PropFilePasswordEncoder utility to ensure no clear-text password appears
			if (webappsrv_admin_security_is_enabled) {
				logger.info("${method_MSG}")
				command = "${websphere_install_dir}${File.separator}bin${File.separator}PropFilePasswordEncoder${script_ext}"
				optionsArray =["${dmgrsoapprops}", "com.ibm.SOAP.loginPassword"]
				command_retcode = cmdProc.run(".", command, optionsArray, "DmgrProfileUpdatePropsSoap_output", "DmgrProfileUpdatePropsSoap_result", "n")	// Run PropFilePasswordEncoder utility to encode password ("n" input string required for response to question on exchange of keystores) 
				command_output  = cmdProc.process_output	// Get output from the command
				logger.info("DmgrProfile-updateProps: Command Return code = " + command_retcode)
				if (command_retcode == 0) {
					method_RC = 1
					method_MSG = "DmgrProfile-updateProps: PropFilePasswordEncoder utility has successfully encoded WAS Admin password."
				} else {
					method_RC = -1
					method_MSG = "**ERROR** DmgrProfile-updateProps: PropFilePasswordEncoder utility produced non-zero return code."
				}					
			}
		} else {
			method_RC = -1
			method_MSG = "**ERROR** DmgrProfile-updateProps: Update of SOAP \"soap.client.props\" file failed."
		}	// END SOAP protocol properties updated
		
		if (method_RC == 1) {
			logger.info("${method_MSG}")
			// Update the ipc.client.props file
			ResourceProperties ipcProps = new ResourceProperties(dmgripcprops)
			logger.info("DmgrProfile-updateProps: Create backup of \"ipc.client.props\" file before alteration.")
			ipcProps.backup()

			// Update the timeout value for IPC communications
			logger.info("DmgrProfile-updateProps: Update IPC timeout property in \"${dmgripcprops}\" file...")
			ipcProps.setPropertyValue("com.ibm.IPC.requestTimeout", dmgripctimeout)

			if (webappsrv_admin_security_is_enabled) {
				logger.info("DmgrProfile-updateProps: Set the IPC security properties in \"ipc.client.props\" file...")
				ipcProps.setPropertyValue("com.ibm.IPC.securityEnabled", "true")
				ipcProps.setPropertyValue("com.ibm.IPC.loginSource", "none")
				ipcProps.setPropertyValue("com.ibm.IPC.loginUserid", webappsrv_admin_userid)

				String decodedPassword = com.sas.util.SasPasswordString.decode(webappsrv_admin_passwd);
				ipcProps.setPropertyValue("com.ibm.IPC.loginPassword", decodedPassword)
			}
	
			logger.info("DmgrProfile-updateProps: Update the IPC security and/or timeout properties file \"ipc.client.props\"...")
			if (ipcProps.writeProps() == 1) {
				method_RC = 1
				method_MSG = "DmgrProfile-updateProps: Update of IPC \"ipc.client.props\" file succeeded."
				if (webappsrv_admin_security_is_enabled) {
					logger.info("${method_MSG}")
					command = "${websphere_install_dir}${File.separator}bin${File.separator}PropFilePasswordEncoder${script_ext}"
					optionsArray =["${dmgripcprops}", "com.ibm.IPC.loginPassword"]
					command_retcode = cmdProc.run(".", command, optionsArray, "DmgrProfileUpdatePropsIPC_output", "DmgrProfileUpdatePropsIPC_result", "n")
					command_output  = cmdProc.process_output
					logger.info("DmgrProfile-updateProps: Command Return code = " + command_retcode)
					if (command_retcode == 0) {
						method_RC = 1
						method_MSG = "DmgrProfile-updateProps: PropFilePasswordEncoder utility has successfully encoded WAS Admin password."
					} else {
						method_RC = -1
						method_MSG = "**ERROR** DmgrProfile-updateProps: PropFilePasswordEncoder utility produced non-zero return code."
					}
				}
			} else {
				method_RC = -1
				method_MSG = "**ERROR** DmgrProfile-updateProps: Update of IPC \"ipc.client.props\" file failed."
			}
		}   // END Update the ipc.client.props file
		
		// Determine if the sas.client.props properties file has to be updated when using the RMI protocol:
		if (method_RC == 1 && dmgrprotocol == "RMI")  {	// If protocol is RMI, then also update the sas.client.props file
			logger.info("${method_MSG}")
			ResourceProperties rmiProps = new ResourceProperties(dmgrrmiprops)	// Use the ResourceProperties object to manipulate the WAS sas.client.props file
			// Make a backup of the existing sas.client.props file before alteration:
			logger.info("DmgrProfile-updateProps: Create backup of \"sas.client.props\" file before alteration.")
			rmiProps.backup()
			
			// Update the timeout value for RMI communications
			logger.info("DmgrProfile-updateProps: Update RMI timeout property in \"${dmgrrmiprops}\" file...")
			rmiProps.setPropertyValue("com.ibm.CORBA.requestTimeout", dmgrrmitimeout)
			
			// If WAS Admin Security is enabled, update the sas.client.props security properties to avoid anoying userID/password prompts
			if (webappsrv_admin_security_is_enabled) {
				logger.info("DmgrProfile-updateProps: Set the RMI security properties in \"sas.client.props\" file...")
				rmiProps.setPropertyValue("com.ibm.CORBA.securityEnabled", "true")
				rmiProps.setPropertyValue("com.ibm.CORBA.loginSource", "none")
				rmiProps.setPropertyValue("com.ibm.CORBA.loginUserid", webappsrv_admin_userid)
			
				// Decode the admin password using the SAS decoder utility
				String decodedPassword = com.sas.util.SasPasswordString.decode(webappsrv_admin_passwd)
				rmiProps.setPropertyValue("com.ibm.CORBA.loginPassword", decodedPassword)
			}
			
			// Write the property updates to the sas.client.props file
			logger.info("DmgrProfile-updateProps: Update the RMI security and/or timeout properties file \"sas.client.props\"...")
			if (rmiProps.writeProps() == 1) {
				method_RC = 1
				method_MSG = "DmgrProfile-updateProps: Update of RMI \"sas.client.props\" file succeeded."
				
				// If security is enabled, run the WAS PropFilePasswordEncoder utility to ensure no clear-text password appears
				if (webappsrv_admin_security_is_enabled) {
					logger.info("${method_MSG}")
					command = "${websphere_install_dir}${File.separator}bin${File.separator}PropFilePasswordEncoder${script_ext}"
					optionsArray = ["${dmgrrmiprops}", "com.ibm.CORBA.loginPassword"]
					command_retcode = cmdProc.run(".", command, optionsArray, "DmgrProfileUpdatePropsRmi_output", "DmgrProfileUpdatePropsRmi_result", "n")	// Run PropFilePasswordEncoder utility to encode password ("n" input string required for response to question on exchange of keystores) 
					command_output  = cmdProc.process_output	// Get output from the command
					logger.info("DmgrProfile-updateProps: Command Return code: " + command_retcode)
					if (command_retcode == 0) {
						method_MSG = "DmgrProfile-updateProps: PropFilePasswordEncoder utility has successfully encoded password."
						method_RC = 1
					} else {
						method_MSG = "**ERROR** DmgrProfile-updateProps: PropFilePasswordEncoder utility produced non-zero return code."
						method_RC = -1
					}	
				}				
			} else {
				method_RC = -1
				method_MSG = "DmgrProfile-updateProps: **ERROR** Update of RMI \"sas.client.props\" file failed."
			}
			// End RMI protocol
		}	
		if (method_RC == 1) {
			logger.info("${method_MSG}")
			method_MSG = "DmgrProfile-updateProps: ===> Update of timeout and security properties COMPLETED SUCCESSFULLY."
			logger.info("${method_MSG}")
		} 
		return method_RC	
	}	// END updateProps method
	
	// ******************************************************************************************
	// Method to update the dmgr server Http ports and JVM generic arguments
	public int update(String resourceName, ResourceProperties resourceProps) {
		logger.info("=================================================")			
		logger.info("DmgrProfile-update: Update the ${dmgr} server Http ports and JVM generic arguments using the wsadmin command...")
		resourcePropsFileName = resourceProps.getPropsFileName()	// Use getter to fetch filename for the resource properties
		config_target_name = resourceProps.getProperty("config.target.name")
		command = "${websphere_install_dir}${File.separator}bin${File.separator}wsadmin${script_ext}"
		optionsArray = [ "-conntype", "NONE", 
						 "-lang", "jython", 
						 "-f", "${websphere_wsadmin_jython}${File.separator}callMultJythonFunction.jy", 
					     "${websphere_wsadmin_jython}", 
						 "${resourcePropsFileName}",
						 "${globalPropsFileName}"  ] 
		command_retcode = cmdProc.run(".", command, optionsArray, "DmgrProfileUpdate_output", "DmgrProfileUpdate_result")	// Run wsadmin client to execute Jython procs to update Http ports and JVM Generic Arguments
		command_output  = cmdProc.process_output	// Get output from the command
		logger.info("DmgrProfile-update: Command Return code: " + command_retcode)
		if (command_retcode == 0) {	// Command return code of zero indicates wsadmin command ran to completion			
			// Check if the setContainerHttpPorts.jy Jython proc completed successfully
			if (  command_output.contains("setHttpPort: Processing complete.") &&
                  command_output.contains("setVirtualHostPort: Processing complete.") && 
                  command_output.contains("setContainerHttpPorts: Processing complete.")  )  { 
				method_MSG =  "DmgrProfile-update: ===> setContainerHttpPorts.jy COMPLETED SUCCESSFULLY; Http ports for the ${dmgr} server are updated."
				method_RC = 1
				logger.info("${method_MSG}")
			} else {
				method_MSG = "**SEVERE** DmgrProfile-update: setContainerHttpPorts.jy Jython processing FAILED; unable to continue."
				method_RC = -99
				return method_RC
			}
			
			// Check if the setJvmArguments.jy Jython proc completed successfully
			if (command_output.contains("setJvmArguments: Modifications to JVM Arguments completed.")) {
				method_MSG = "DmgrProfile-update: ===> setJvmArguments.jy COMPLETED SUCCESSFULLY; JVM generic arguments for the ${dmgr} server are updated."
				method_RC = 1
				logger.info("${method_MSG}")
			} else {
				method_MSG = "**SEVERE** DmgrProfile-update: setJvmArguments.jy Jython processing FAILED; unable to continue."
				method_RC = -99
				return method_RC
			}
			
			// Check if the setJMXClientTimeout.jy Jython proc completed successfully
			if (command_output.contains("setJMXClientTimeout: Set requestTimeout for JMX Clients completed.")) {
				method_MSG = "DmgrProfile-update: ===> setJMXClientTimeout.jy COMPLETED SUCCESSFULLY; SOAP or RMI, IPC requestTimeout values for the ${dmgr} server are updated."
				method_RC = 1
			} else {
				method_MSG = "**SEVERE** DmgrProfile-update: setJMXClientTimeout.jy Jython processing FAILED; unable to continue."
				method_RC = -99
			}
		} else {
			method_RC = -99
			method_MSG = "**SEVERE** DmgrProfile-update: wsadmin${script_ext} command failed; RC=" + command_retcode
		}
		return method_RC	
	}	// END update method

}	// END class DmgrProfile