// Copyright (c) 2002-2011, SAS Institute Inc., Cary, NC, USA, All Rights Reserved

package com.sas.config.websphere
import com.sas.config.util.CommandProcess
import com.sas.config.util.ResourceProperties

// **************************************************************************************************************
// Class: JMSConnectionFactory
// Purpose: Encapsulate the attributes and methods used to manage and configure a JMS Connection Factory
// Methods:
//		create - Create a JMS Connection Factory definition
//		delete - Delete a JMS Connection Factory definition
//		updateProviderEndpoints - Update the Provider Endpoints attribute of an existing JMS Connection Factory
// ****************************************************************************************************************

class JMSConnectionFactory extends Resource {
	// Class variables available to all methods:
	CommandProcess cmdProc
	def globalPropsFileName
	def resourcePropsFileName
	def config_target_name	// SDW-provided calling Ant target name (configure, updateConfigure, updatePasswords, startMidtierServers, unconfigure, changeHost)
	def servername
	def scope
	def SIBusname
	def connectionFactoryId
	def pjndiName
	def pdataSource
	def pschemaName
	def pconnectionFactoryName
	def pconnectionFactoryType
	def pproviderEndPoints
	def pmaxConnections
	def pminConnections
	def pconnectionTimeout
	def preapTime
	def punusedTimeout
	def pagedTimeout
	def ppurgePolicy
	def pjaasAliasName		// JAAS J2C Alias name used in data source definition to access message datastore 

	def command 			// Used to hold the command string that will be executed via the Process class
	def options 			// ArrayList used to accumulate command options
	String [] optionsArray 	// String array used to pass options to command 
	String command_output	// Used to hold the stdout output from the command execution
	int    command_retcode  // Used to hold the return code from the command execution		

	// *************************************************************************************
	// CTOR: Initialize global properties object and obtain required global properties 
	// Global properties are fetched and initialized by the base Resource class
	public JMSConnectionFactory(GlobalProperties parmGblProps) {
		super(parmGblProps)		// Load the global properties using the abstract base class CTOR
		globalPropsFileName = parmGblProps.globalPropsFileName	// Get Global Properties file name
		cmdProc = new CommandProcess()	// Construct object used to run Ant exec task for external commands
	}
	
	// *************************************************************************************
	// Method to create a JMS Connection Factory 
	// General flow:
	//	1. Run the createJMSConnectionFactory.jy Jython script
	public int create(String resourceName, ResourceProperties resourceProps) {
		logger.info("=================================================")
		// Required Resource Properties for this method:
		resourcePropsFileName	= resourceProps.getPropsFileName()	// Use getter to fetch filename for the resource properties
		servername				= resourceProps.getProperty("servername")
		connectionFactoryId		= resourceProps.getProperty("connectionFactoryId")
		pconnectionFactoryName 	= resourceProps.getProperty("pconnectionFactoryName")
		pconnectionFactoryType 	= resourceProps.getProperty("pconnectionFactoryType")
		pjndiName				= resourceProps.getProperty("pjndiName")
		pdataSource				= resourceProps.getProperty("pdataSource")
		pschemaName				= resourceProps.getProperty("pschemaName")
		pjaasAliasName			= resourceProps.getProperty("pjaasAliasName")
		SIBusname				= resourceProps.getProperty("SIBusname")
		pproviderEndPoints		= resourceProps.getProperty("pproviderEndPoints")
		pmaxConnections			= resourceProps.getProperty("pmaxConnections")
		pminConnections			= resourceProps.getProperty("pminConnections")
		pconnectionTimeout		= resourceProps.getProperty("pconnectionTimeout")
		preapTime				= resourceProps.getProperty("preapTime")
		punusedTimeout			= resourceProps.getProperty("punusedTimeout")
		pagedTimeout			= resourceProps.getProperty("pagedTimeout")
		ppurgePolicy			= resourceProps.getProperty("ppurgePolicy")
		scope 					= resourceProps.getProperty("scope")
		config_target_name		= resourceProps.getProperty("config.target.name")
		
		if (DEBUG) {
			logger.debug("JMSConnectionFactory-create: globalPropsFileName=${globalPropsFileName}")
			logger.debug("JMSConnectionFactory-create: resourcePropsFileName=${resourcePropsFileName}")
			logger.debug("JMSConnectionFactory-create: appsrvnodename=${appsrvnodename}")
			logger.debug("JMSConnectionFactory-create: servername=${servername}")
			logger.debug("JMSConnectionFactory-create: connectionFactoryId=${connectionFactoryId}")
			logger.debug("JMSConnectionFactory-create: pconnectionFactoryType=${pconnectionFactoryType}")
			logger.debug("JMSConnectionFactory-create: pconnectionFactoryName=${pconnectionFactoryName}")
			logger.debug("JMSConnectionFactory-create: pjndiName=${pjndiName}")
			logger.debug("JMSConnectionFactory-create: pdataSource=${pdataSource}")
			logger.debug("JMSConnectionFactory-create: pschemaName=${pschemaName}")
			logger.debug("JMSConnectionFactory-create: pjaasAliasName=${pjaasAliasName}")
			logger.debug("JMSConnectionFactory-create: SIBusname=${SIBusname}")
			logger.debug("JMSConnectionFactory-create: pproviderEndPoints=${pproviderEndPoints}")
			logger.debug("JMSConnectionFactory-create: pmaxConnections=${pmaxConnections}")
			logger.debug("JMSConnectionFactory-create: pminConnections=${pminConnections}")
			logger.debug("JMSConnectionFactory-create: pconnectionTimeout${pconnectionTimeout}")
			logger.debug("JMSConnectionFactory-create: preapTime=${preapTime}")
			logger.debug("JMSConnectionFactory-create: punusedTimeout=${punusedTimeout}")
			logger.debug("JMSConnectionFactory-create: pagedTimeout=${pagedTimeout}")
			logger.debug("JMSConnectionFactory-create: ppurgePolicy=${ppurgePolicy}")
			logger.debug("JMSConnectionFactory-create: scope=${scope}")
			logger.debug("JMSConnectionFactory-create: config_target_name=${config_target_name}")
			logger.debug("JMSConnectionFactory-create: script_ext=${script_ext}")
			logger.debug("JMSConnectionFactory-create: websphere_wsadmin_jython=${websphere_wsadmin_jython}")
			logger.debug("JMSConnectionFactory-create: websphere_install_dir=${websphere_install_dir}")
			logger.debug("JMSConnectionFactory-create: websphere_wsadmin_classpath=${websphere_wsadmin_classpath}")
			logger.debug("JMSConnectionFactory-create: dmgrprotocol=${dmgrprotocol}")
			logger.debug("JMSConnectionFactory-create: dmgrport=${dmgrport}")
			logger.debug("JMSConnectionFactory-create: webappsrv_admin_host=${webappsrv_admin_host}")
			logger.debug("JMSConnectionFactory-create: wsadminOptionsList=${wsadminOptionsList}")			
		}

		// Determine if credentials for JAAS-J2C Alias definition for data source connection to a database used for a message datastore are needed
		if (pdataSource != "not-set")  {			// If a data source name has been provided, process the required JAAS-J2C Alias authentication data 
			if (pjaasAliasName != "not-set")  {		// If JAAS-J2C Alias name has been provided, process the credentials 
				ResourceProperties credentialsProps = new ResourceProperties(websphere_scripting_credentials)	// Object used to read the websphere_scripting_credentials properties file
				String jaasAliasUserid = credentialsProps.getProperty("JMSConnectionFactory.create_${scope}_${connectionFactoryId}_userid")
				String jaasAliasPasswd = credentialsProps.getProperty("JMSConnectionFactory.create_${scope}_${connectionFactoryId}_passwd")
				// If userId and/or password are null in the credentials.properties file, can't continue because JAAS J2C Alias for data source needs credentials
				def connectionFactoryErrorType = ""
				if (jaasAliasUserid == "not-set" || jaasAliasUserid == "")
					connectionFactoryErrorType = "userId"
				if (jaasAliasPasswd == "not-set" || jaasAliasPasswd == "")
					connectionFactoryErrorType += " password"
				if (connectionFactoryErrorType) {
					method_MSG = "**SEVERE** JMSConnectionFactory-create: JAAS J2C Alias \"${pjaasAliasName}\" for JMS Connection Factory \"${pconnectionFactoryName}\" message datastore requires credentials, but ${connectionFactoryErrorType} not provided; unable to continue."
					method_RC = -99
					logger.error("${method_MSG}")
					return method_RC
				}
			} else {								// JAAS-J2C Alias name was not provided, can't continue because can't define the requested datasource for the message datastore
				method_MSG = "**SEVERE** JMSConnectionFactory-create: JAAS J2C Alias Name for JMS Connection Factory \"${pconnectionFactoryName}\" has not been provided to create message datastore data source definition; unable to continue."
				method_RC = -99
				logger.error("${method_MSG}")
				return method_RC
			}	
		}

		logger.info("JMSConnectionFactory-create: Create JMS ${pconnectionFactoryType} Connection Factory \"${pconnectionFactoryName}\" at scope \"${scope}\" using wsadmin${script_ext}...")   
		command = "${websphere_install_dir}${File.separator}bin${File.separator}wsadmin${script_ext}"
		options = []
		wsadminOptionsList.each{ options.add(it) }
		options.addAll( [ "-f", "${websphere_scripting_jython}${File.separator}callJythonFunction.jy",
						  "${websphere_scripting_jython}",
						  "createJMSConnectionFactory",
						  "${resourcePropsFileName}",
						  "${globalPropsFileName}" ] )
		optionsArray = options.toArray()
		command_retcode = cmdProc.run(".", command, optionsArray, "JMSConnectionFactoryCreate_${pconnectionFactoryName}_output", "JMSConnectionFactoryCreate_${pconnectionFactoryName}_result")	// Run wsadmin client to execute Jython proc
		command_output  = cmdProc.process_output	// Get output from the command 				  
		logger.info("JMSConnectionFactory-create: Command Return code: " + command_retcode)
		if (command_retcode == 0)  {	// Command return code of zero indicates wsadmin command ran to completion			
			// Check if the createJMSConnectionFactory.jy Jython proc completed successfully 
			if ( command_output.contains("createJMSConnectionFactory: JMS Connection Factory \"${pconnectionFactoryName}\" successfully created.") )  { 
				method_MSG =  "JMSConnectionFactory-create: ===> createJMSConnectionFactory.jy COMPLETED SUCCESSFULLY."
				method_RC = 1
				logger.info("${method_MSG}")
				if (pdataSource != "not-set")  {
					if ( command_output.contains("addDataStoreToSIBEngine: Successfully updated Default Messaging Engine for DATASTORE support by adding data source") )  {
						method_MSG = "JMSConnectionFactory-create: ===> addDataStoreToSIBEngine.jy COMPLETED SUCCESSFULLY."
						method_RC = 1
						logger.info("${method_MSG}")
					} else {
						method_MSG = "**SEVERE** JMSConnectionFactory-create: addDataStoreToSIBEngine.jy Jython processing FAILED; unable to continue."
						method_RC = -99
						logger.error("${method_MSG}")
					}
				}
			} else {
				method_MSG = "**SEVERE** JMSConnectionFactory-create: createJMSConnectionFactory.jy Jython processing FAILED; unable to continue."
				method_RC = -99
				logger.error("${method_MSG}")
			}
		} else {
			method_RC = -99
			method_MSG = "**SEVERE** JMSConnectionFactory-create: wsadmin${script_ext} command FAILED; RC=" + command_retcode
			logger.error("${method_MSG}")
		}
		return method_RC	// Return to driver indicating failure
	}	// END create method	
	
	// *************************************************************************************
	// Method to delete a JMS Connection Factory 
	// General flow:
	//	1. Run the deleteJMSConnectionFactory.jy Jython script
	public int delete(String resourceName, ResourceProperties resourceProps) {
		logger.info("=================================================")
		// Required Resource Properties for this method:
		resourcePropsFileName 	= resourceProps.getPropsFileName()	// Use getter to fetch filename for the resource properties	
		servername 				= resourceProps.getProperty("servername")
		scope					= resourceProps.getProperty("scope")
		config_target_name		= resourceProps.getProperty("config.target.name")
		pconnectionFactoryName 	= resourceProps.getProperty("pconnectionFactoryName")
		pconnectionFactoryType 	= resourceProps.getProperty("pconnectionFactoryType")
		
		if (DEBUG) {
			logger.debug("JMSConnectionFactory-delete: globalPropsFileName=${globalPropsFileName}")		
			logger.debug("JMSConnectionFactory-delete: resourcePropsFileName=${resourcePropsFileName}")
			logger.debug("JMSConnectionFactory-delete: appsrvnodename=${appsrvnodename}")
			logger.debug("JMSConnectionFactory-delete: servername=${servername}")
			logger.debug("JMSConnectionFactory-delete: scope=${scope}")
			logger.debug("JMSConnectionFactory-delete: config_target_name=${config_target_name}")
			logger.debug("JMSConnectionFactory-delete: pconnectionFactoryType=${pconnectionFactoryType}")
			logger.debug("JMSConnectionFactory-delete: pconnectionFactoryName=${pconnectionFactoryName}")
			logger.debug("JMSConnectionFactory-delete: script_ext=${script_ext}")
			logger.debug("JMSConnectionFactory-delete: websphere_wsadmin_jython=${websphere_wsadmin_jython}")
			logger.debug("JMSConnectionFactory-delete: websphere_install_dir=${websphere_install_dir}")
			logger.debug("JMSConnectionFactory-delete: websphere_wsadmin_classpath=${websphere_wsadmin_classpath}")
			logger.debug("JMSConnectionFactory-delete: dmgrprotocol=${dmgrprotocol}")
			logger.debug("JMSConnectionFactory-delete: dmgrport=${dmgrport}")
			logger.debug("JMSConnectionFactory-delete: webappsrv_admin_host=${webappsrv_admin_host}")
			logger.debug("JMSConnectionFactory-delete: wsadminOptionsList=${wsadminOptionsList}")			
		}
	
		logger.info("JMSConnectionFactory-delete: Delete JMS ${pconnectionFactoryType} Connection Factory \"${pconnectionFactoryName}\" using wsadmin${script_ext}...")  
		command = "${websphere_install_dir}${File.separator}bin${File.separator}wsadmin${script_ext}"
		options = []
		wsadminOptionsList.each{ options.add(it) }
		options.addAll( [ "-f", "${websphere_scripting_jython}${File.separator}callJythonFunction.jy",
						  "${websphere_scripting_jython}",
						  "deleteJMSConnectionFactory",
						  "${resourcePropsFileName}",
						  "${globalPropsFileName}" ] )
		optionsArray = options.toArray()				  
		command_retcode = cmdProc.run(".", command, optionsArray, "JMSConnectionFactoryDelete_${pconnectionFactoryName}_output", "JMSConnectionFactoryDelete_${pconnectionFactoryName}_result")	// Run wsadmin client to execute Jython proc
		command_output  = cmdProc.process_output	// Get output from the command 				  
		logger.info("JMSConnectionFactory-delete: Command Return code: " + command_retcode)
		if (command_retcode == 0) {	// Command return code of zero indicates wsadmin command ran to completion			
			// Check if the deleteJMSConnectionFactory.jy Jython proc completed successfully 
			if (  command_output.contains("deleteJMSConnectionFactory: JMS ${pconnectionFactoryType} Connection Factory \"${pconnectionFactoryName}\" was successfully deleted.") ||
				  command_output.contains("deleteJMSConnectionFactory: No JMSConnectionFactory entries to delete.") ) { 
				method_MSG =  "JMSConnectionFactory-delete: ===> deleteJMSConnectionFactory.jy COMPLETED SUCCESSFULLY."
				method_RC = 1
				logger.info("${method_MSG}")
			} else {
				method_MSG = "**SEVERE** JMSConnectionFactory-delete: deleteJMSConnectionFactory.jy Jython processing FAILED; unable to continue."
				method_RC = -99
				logger.error("${method_MSG}")
			}
		} else {
			method_RC = -99
			method_MSG = "**SEVERE** JMSConnectionFactory-delete: wsadmin${script_ext} command FAILED; RC=" + command_retcode
			logger.error("${method_MSG}")
		}
		return method_RC	// Return to driver indicating failure
	}	// END delete method
	
	// *************************************************************************************
	// Method to update the Provider Endpoints attribute of a JMS Connection Factory definition 
	// General flow:
	//	1. Run the updateProviderEndpoints.jy Jython script
	public int updateProviderEndpoints(String resourceName, ResourceProperties resourceProps) {
		logger.info("=================================================")
		// Required Resource Properties for this method:
		resourcePropsFileName 	= resourceProps.getPropsFileName()	// Use getter to fetch filename for the resource properties
		servername				= resourceProps.getProperty("servername")
		scope					= resourceProps.getProperty("scope")
		config_target_name		= resourceProps.getProperty("config.target.name")
		pconnectionFactoryName 	= resourceProps.getProperty("pconnectionFactoryName")
		pconnectionFactoryType 	= resourceProps.getProperty("pconnectionFactoryType")
		pproviderEndPoints		= resourceProps.getProperty("pproviderEndPoints")
		
		if (DEBUG) {
			logger.debug("JMSConnectionFactory-create: globalPropsFileName=${globalPropsFileName}")
			logger.debug("JMSConnectionFactory-updateProviderEndpoints: resourcePropsFileName=${resourcePropsFileName}")
			logger.debug("JMSConnectionFactory-updateProviderEndpoints: appsrvnodename=${appsrvnodename}")
			logger.debug("JMSConnectionFactory-updateProviderEndpoints: servername=${servername}")
			logger.debug("JMSConnectionFactory-updateProviderEndpoints: scope=${scope}")
			logger.debug("JMSConnectionFactory-updateProviderEndpoints: config_target_name=${config_target_name}")
			logger.debug("JMSConnectionFactory-updateProviderEndpoints: pconnectionFactoryType=${pconnectionFactoryType}")
			logger.debug("JMSConnectionFactory-updateProviderEndpoints: pconnectionFactoryName=${pconnectionFactoryName}")
			logger.debug("JMSConnectionFactory-updateProviderEndpoints: pproviderEndPoints=${pproviderEndPoints}")
			logger.debug("JMSConnectionFactory-updateProviderEndpoints: script_ext=${script_ext}")
			logger.debug("JMSConnectionFactory-updateProviderEndpoints: websphere_wsadmin_jython=${websphere_wsadmin_jython}")
			logger.debug("JMSConnectionFactory-updateProviderEndpoints: websphere_install_dir=${websphere_install_dir}")
			logger.debug("JMSConnectionFactory-updateProviderEndpoints: websphere_wsadmin_classpath=${websphere_wsadmin_classpath}")
			logger.debug("JMSConnectionFactory-updateProviderEndpoints: dmgrprotocol=${dmgrprotocol}")
			logger.debug("JMSConnectionFactory-updateProviderEndpoints: dmgrport=${dmgrport}")
			logger.debug("JMSConnectionFactory-updateProviderEndpoints: webappsrv_admin_host=${webappsrv_admin_host}")
			logger.debug("JMSConnectionFactory-updateProviderEndpoints: wsadminOptionsList=${wsadminOptionsList}")			
		}
	
		logger.info("JMSConnectionFactory-updateProviderEndpoints: Update the Provider Endpoints attribute for the JMS ${pconnectionFactoryType} Connection Factory \"${pconnectionFactoryName}\" using wsadmin${script_ext}...")
		command = "${websphere_install_dir}${File.separator}bin${File.separator}wsadmin${script_ext}"
		options = []
		wsadminOptionsList.each{ options.add(it) }
		options.addAll( [ "-f", "${websphere_scripting_jython}${File.separator}callJythonFunction.jy",
						  "${websphere_scripting_jython}",
						  "updateProviderEndpoints",
						  "${resourcePropsFileName}",
						  "${globalPropsFileName}" ] )
		optionsArray = options.toArray()
		command_retcode = cmdProc.run(".", command, optionsArray, "JMSConnectionFactoryUpdateProviderEndpoints_${pconnectionFactoryName}_output", "JMSConnectionFactoryUpdateProviderEndpoints_${pconnectionFactoryName}_result")	// Run wsadmin client to execute Jython proc
		command_output  = cmdProc.process_output	// Get output from the command 				  
		logger.info("JMSConnectionFactory-updateProviderEndpoints: Command Return code: " + command_retcode)
		if (command_retcode == 0) {	// Command return code of zero indicates wsadmin command ran to completion			
			// Check if the updateProviderEndpoints.jy Jython proc completed successfully 
			if (  command_output.contains("ProviderEndpoints attribute has been successfully updated for ${pconnectionFactoryType} Connection Factory \"${pconnectionFactoryName}\".") ) { 
				method_MSG =  "JMSConnectionFactory-updateProviderEndpoints: ===> updateProviderEndpoints.jy COMPLETED SUCCESSFULLY; Provider Endpoints have been set to \"${pproviderEndPoints}\"."
				method_RC = 1
				logger.info("${method_MSG}")
			} else {
				method_MSG = "**SEVERE** JMSConnectionFactory-updateProviderEndpoints: updateProviderEndpoints.jy Jython processing FAILED; unable to continue."
				method_RC = -99
				logger.error("${method_MSG}")
			}
		} else {
			method_RC = -99
			method_MSG = "**SEVERE** JMSConnectionFactory-updateProviderEndpoints: wsadmin${script_ext} command FAILED; RC=" + command_retcode
			logger.error("${method_MSG}")
		}
		return method_RC	// Return to driver indicating failure
	}	// END updateProviderEndpoints method	
	
}	// END JMSConnectionFactory class