// Copyright (c) 2002-2011, SAS Institute Inc., Cary, NC, USA, All Rights Reserved

package com.sas.config.websphere
import com.sas.config.util.CommandProcess
import com.sas.config.util.ResourceProperties

// **************************************************************************************************************
// Class: JMSQueue
// Purpose: Encapsulate the attributes and methods used to manage and configure a JMS Queue
// Methods:
//		create - Create a JMS Queue definition
//		delete - Delete a JMS Queue definition
// ****************************************************************************************************************

class JMSQueue extends Resource {
	// Class variables available to all methods:
	CommandProcess cmdProc
	def globalPropsFileName
	def resourcePropsFileName
	def servername
	def jndiName
	def queueName
	def topicName
	def SIBusDestName
	def SIBusDestType
	def SIBusname
	def deliveryMode
	def readAhead
	def timeToLive
	def priority
	def scope
	def scope_string
	def config_target_name	// SDW-provided calling Ant target name (configure, updateConfigure, updatePasswords, startMidtierServers, unconfigure, changeHost)
	def command 			// Used to hold the command string that will be executed via the Process class
	def options 			// ArrayList used to accumulate command options
	String [] optionsArray 	// String array used to pass options to command  
	String command_output	// Used to hold the stdout output from the command execution
	int    command_retcode  // Used to hold the return code from the command execution			

	// *************************************************************************************
	// CTOR: Initialize global properties object and obtain required global properties 
	// Global properties are fetched and initialized by the base Resource class
	public JMSQueue(GlobalProperties parmGblProps) {
		super(parmGblProps)		// Load the global properties using the abstract base class CTOR
		globalPropsFileName = parmGblProps.globalPropsFileName	// Get Global Properties file name
		cmdProc = new CommandProcess()	// Construct object used to run Ant exec task for external commands
	}
	
	// *************************************************************************************
	// Method to create a JMS Queue 
	// General flow:
	//	1. Run the createSIBusDestination.jy and createJMSQueue.jy Jython scripts
	public int create(String resourceName, ResourceProperties resourceProps) {
		logger.info("=================================================")
		// Required Resource Properties for this method:
		resourcePropsFileName 	= resourceProps.getPropsFileName()	// Use getter to fetch filename for the resource properties	
		servername 				= resourceProps.getProperty("servername")
		jndiName				= resourceProps.getProperty("pjndiName")
		queueName 				= resourceProps.getProperty("pqueueName")
		topicName				= resourceProps.getProperty("ptopicname")
		SIBusname 				= resourceProps.getProperty("SIBusname")
		SIBusDestName 			= resourceProps.getProperty("psibusdestname")
		SIBusDestType 			= resourceProps.getProperty("psibusdesttype")
		deliveryMode			= resourceProps.getProperty("pdeliveryMode")
		readAhead				= resourceProps.getProperty("preadAhead")
		timeToLive				= resourceProps.getProperty("ptimeToLive")
		priority				= resourceProps.getProperty("ppriority")
		scope 					= resourceProps.getProperty("scope")
		scope_string 			= resourceProps.getProperty("scope_string")
		config_target_name		= resourceProps.getProperty("config.target.name")
		if (DEBUG) {
			logger.debug("JMSQueue-create: globalPropsFileName=${globalPropsFileName}")
			logger.debug("JMSQueue-create: resourcePropsFileName=${resourcePropsFileName}")
			logger.debug("JMSQueue-create: servername=${servername}")
			logger.debug("JMSQueue-create: appsrvnodename=${appsrvnodename}")
			logger.debug("JMSQueue-create: jndiName=${jndiName}")
			logger.debug("JMSQueue-create: queueName=${queueName}")
			logger.debug("JMSQueue-create: topicName=${topicName}")
			logger.debug("JMSQueue-create: SIBusname=${SIBusname}")
			logger.debug("JMSQueue-create: SIBusDestName=${SIBusDestName}")
			logger.debug("JMSQueue-create: SIBusDestType=${SIBusDestType}")
			logger.debug("JMSQueue-create: deliveryMode=${deliveryMode}")
			logger.debug("JMSQueue-create: readAhead=${readAhead}")
			logger.debug("JMSQueue-create: timeToLive=${timeToLive}")
			logger.debug("JMSQueue-create: priority=${priority}")
			logger.debug("JMSQueue-create: scope=${scope}")
			logger.debug("JMSQueue-create: scope_string=${scope_string}")
			logger.debug("JMSQueue-create: config_target_name=${config_target_name}")
			logger.debug("JMSQueue-create: script_ext=${script_ext}")
			logger.debug("JMSQueue-create: websphere_wsadmin_jython=${websphere_wsadmin_jython}")
			logger.debug("JMSQueue-create: websphere_install_dir=${websphere_install_dir}")
			logger.debug("JMSQueue-create: websphere_wsadmin_classpath=${websphere_wsadmin_classpath}")
			logger.debug("JMSQueue-create: dmgrprotocol=${dmgrprotocol}")
			logger.debug("JMSQueue-create: dmgrport=${dmgrport}")
			logger.debug("JMSQueue-create: webappsrv_admin_host=${webappsrv_admin_host}")
			logger.debug("JMSQueue-create: wsadminOptionsList=${wsadminOptionsList}")			
		}
	
		logger.info("JMSQueue-create: Create JMS Queue \"${queueName}\" at scope \"${scope_string}\" using wsadmin${script_ext}...")
		logger.info("JMSQueue-create: Run createSIBusDestination.jy and createJMSQueue.jy scripts to accomplish this task.")  
		command = "${websphere_install_dir}${File.separator}bin${File.separator}wsadmin${script_ext}"
		options = []
		wsadminOptionsList.each{ options.add(it) }
		options.addAll( [ "-f", "${websphere_scripting_jython}${File.separator}callMultJythonFunction.jy",
						  "${websphere_scripting_jython}",
						  "${resourcePropsFileName}",
						  "${globalPropsFileName}" ] )
		optionsArray = options.toArray()				  
		command_retcode = cmdProc.run(".", command, optionsArray, "JMSQueueCreate_${queueName}_output", "JMSQueueCreate_${queueName}_result")	// Run wsadmin client to execute Jython proc
		command_output  = cmdProc.process_output	// Get output from the command 				  
		logger.info("JMSQueue-create: Command Return code: " + command_retcode)
		if (command_retcode == 0) {		// Command return code of zero indicates wsadmin command ran to completion			
			// Check if the createSIBusDestination.jy Jython proc completed successfully 
			if (  command_output.contains("createSIBusDestination: SIBus destination \"${SIBusDestName}\" successfully created.") ||
				  command_output.contains("createSIBusDestination: No need to create new SIBus destination \"${SIBusDestName}\"; definition already exists.") ) { 
				method_MSG =  "JMSQueue-create: ===> createSIBusDestination.jy COMPLETED SUCCESSFULLY."
				method_RC = 1
				logger.info("${method_MSG}")
			} else {
				method_MSG = "**SEVERE** JMSQueue-create: createSIBusDestination.jy Jython processing FAILED; unable to continue."
				method_RC = -99
				logger.error("${method_MSG}")
				return method_RC
			}
			// Check if the createJMSQueue.jy Jython proc completed successfully 
			if (  command_output.contains("createJMSQueue: JMS Queue \"${queueName}\" successfully created.") ) { 
				method_MSG =  "JMSQueue-create: ===> createJMSQueue.jy COMPLETED SUCCESSFULLY."
				method_RC = 1
				logger.info("${method_MSG}")
			} else {
				method_MSG = "**SEVERE** JMSQueue-create: createJMSQueue.jy Jython processing FAILED; unable to continue."
				method_RC = -99
				logger.error("${method_MSG}")
			}
		} else {
			method_RC = -99
			method_MSG = "**SEVERE** JMSQueue-create: wsadmin${script_ext} command FAILED; RC=" + command_retcode
			logger.error("${method_MSG}")
		}
		return method_RC	// Return to driver indicating failure
	}	// END create method	 
	
	// *************************************************************************************
	// Method to delete a JMS Queue 
	// General flow:
	//	1. Run the deleteSIBusDestination.jy and deleteJMSQueue.jy Jython scripts
	public int delete(String resourceName, ResourceProperties resourceProps) {
		logger.info("=================================================")
		// Required Resource Properties for this method:
		resourcePropsFileName 	= resourceProps.getPropsFileName()	// Use getter to fetch filename for the resource properties	
		queueName 				= resourceProps.getProperty("pqueueName")
		topicName				= resourceProps.getProperty("ptopicname")	// The value of this will be "not-set" for JMS queues
		SIBusname 				= resourceProps.getProperty("SIBusname")
		SIBusDestName 			= resourceProps.getProperty("psibusdestname")
		SIBusDestType 			= resourceProps.getProperty("psibusdesttype")
		scope = resourceProps.getProperty("scope")
		scope_string = resourceProps.getProperty("scope_string")
		config_target_name		= resourceProps.getProperty("config.target.name")
		if (DEBUG) {
			logger.debug("JMSQueue-delete: globalPropsFileName=${globalPropsFileName}")
			logger.debug("JMSQueue-delete: resourcePropsFileName=${resourcePropsFileName}")
			logger.debug("JMSQueue-delete: servername=${servername}")
			logger.debug("JMSQueue-delete: appsrvnodename=${appsrvnodename}")
			logger.debug("JMSQueue-delete: queueName=${queueName}")
			logger.debug("JMSQueue-delete: topicName=${topicName}")
			logger.debug("JMSQueue-create: SIBusname=${SIBusname}")
			logger.debug("JMSQueue-create: SIBusDestName=${SIBusDestName}")
			logger.debug("JMSQueue-create: SIBusDestType=${SIBusDestType}")
			logger.debug("JMSQueue-delete: scope=${scope}")
			logger.debug("JMSQueue-delete: scope_string=${scope_string}")
			logger.debug("JMSQueue-delete: config_target_name=${config_target_name}")
			logger.debug("JMSQueue-delete: script_ext=${script_ext}")
			logger.debug("JMSQueue-delete: websphere_wsadmin_jython=${websphere_wsadmin_jython}")
			logger.debug("JMSQueue-delete: websphere_install_dir=${websphere_install_dir}")
			logger.debug("JMSQueue-delete: websphere_wsadmin_classpath=${websphere_wsadmin_classpath}")
			logger.debug("JMSQueue-delete: dmgrprotocol=${dmgrprotocol}")
			logger.debug("JMSQueue-delete: dmgrport=${dmgrport}")
			logger.debug("JMSQueue-delete: webappsrv_admin_host=${webappsrv_admin_host}")
			logger.debug("JMSQueue-delete: wsadminOptionsList=${wsadminOptionsList}")			
		}
	
		logger.info("JMSQueue-delete: Delete JMS Queue \"${queueName}\" at scope \"${scope_string}\" using wsadmin${script_ext}...") 
		logger.info("JMSQueue-delete: Run deleteSIBusDestination.jy and deleteJMSQueue.jy scripts to accomplish this task.") 
		command = "${websphere_install_dir}${File.separator}bin${File.separator}wsadmin${script_ext}"
		options = []
		wsadminOptionsList.each{ options.add(it) }
		options.addAll( [ "-f", "${websphere_scripting_jython}${File.separator}callMultJythonFunction.jy",
						  "${websphere_scripting_jython}",
						  "${resourcePropsFileName}",
						  "${globalPropsFileName}" ] )
		optionsArray = options.toArray()				  
		command_retcode = cmdProc.run(".", command, optionsArray, "JMSQueueDelete_${queueName}_output", "JMSQueueDelete_${queueName}_result")	// Run wsadmin client to execute Jython proc
		command_output  = cmdProc.process_output	// Get output from the command 				  
		logger.info("JMSQueue-delete: Command Return code: " + command_retcode)
		if (command_retcode == 0) {	// Command return code of zero indicates wsadmin command ran to completion			
			// Check if the deleteSIBusDestination.jy Jython proc completed successfully  
			if (  command_output.contains("deleteSIBusDestination: SIBus destination \"${SIBusDestName}\" successfully deleted.") ||
				  command_output.contains("deleteSIBusDestination: No SIBus Destination entries to delete.") ) { 
				method_MSG =  "JMSQueue-delete: ===> deleteSIBusDestination.jy COMPLETED SUCCESSFULLY."
				method_RC = 1
				logger.info("${method_MSG}")
			} else {
				method_MSG = "**SEVERE** JMSQueue-delete: deleteSIBusDestination.jy Jython processing FAILED; unable to continue."
				method_RC = -99
				logger.error("${method_MSG}")
				return method_RC
			}
			// Check if the deleteJMSQueue.jy Jython proc completed successfully
			if (  command_output.contains("deleteJMSQueue: JMS Queue \"${queueName}\" successfully deleted.") ||
				  command_output.contains("deleteJMSQueue: No JMS Queue entries to delete.") ) { 
				method_MSG =  "JMSQueue-delete: ===> deleteJMSQueue.jy COMPLETED SUCCESSFULLY."
				method_RC = 1
				logger.info("${method_MSG}")
			} else {
				method_MSG = "**SEVERE** JMSQueue-delete: deleteJMSQueue.jy Jython processing FAILED; unable to continue."
				method_RC = -99
				logger.error("${method_MSG}")
			}	
		} else {
			method_RC = -99
			method_MSG = "**SEVERE** JMSQueue-delete: wsadmin${script_ext} command FAILED; RC=" + command_retcode
			logger.error("${method_MSG}")
		}
		return method_RC	// Return to driver indicating failure
	}	// END delete method

}	// End JMSQueue class