// Copyright (c) 2002-2011, SAS Institute Inc., Cary, NC, USA, All Rights Reserved

package com.sas.config.websphere
import com.sas.config.util.CommandProcess
import com.sas.config.util.ResourceProperties

// **************************************************************************************************************
// Class: JMSTopic
// Purpose: Encapsulate the attributes and methods used to manage and configure a JMS Topic
// Methods:
//		create - Create a JMS Topic definition
//		delete - Delete a JMS Topic definition
// ****************************************************************************************************************

class JMSTopic extends Resource {
	// Class variables available to all methods:
	CommandProcess cmdProc
	def globalPropsFileName
	def resourcePropsFileName
	def servername
	def jndiName
	def queueName
	def topicName
	def topicSpace
	def SIBusDestName
	def SIBusDestType
	def SIBusname
	def deliveryMode
	def readAhead
	def timeToLive
	def priority
	def scope
	def scope_string
	def config_target_name	// SDW-provided calling Ant target name (configure, updateConfigure, updatePasswords, startMidtierServers, unconfigure, changeHost)
	def command 			// Used to hold the command string that will be executed via the Process class
	def options 			// ArrayList used to accumulate command options
	String [] optionsArray 	// String array used to pass options to command  
	String command_output	// Used to hold the stdout output from the command execution
	int    command_retcode  // Used to hold the return code from the command execution			

	// *************************************************************************************
	// CTOR: Initialize global properties object and obtain required global properties 
	// Global properties are fetched and initialized by the base Resource class
	public JMSTopic(GlobalProperties parmGblProps) {
		super(parmGblProps)		// Load the global properties using the abstract base class CTOR
		globalPropsFileName = parmGblProps.globalPropsFileName	// Get Global Properties file name
		cmdProc = new CommandProcess()	// Construct object used to run Ant exec task for external commands
	}
	
	// *************************************************************************************
	// Method to create a JMS Topic 
	// General flow:
	//	1. Run the createSIBusDestination.jy and createJMSTopic.jy Jython scripts
	public int create(String resourceName, ResourceProperties resourceProps) {
		logger.info("=================================================")
		// Required Resource Properties for this method:
		resourcePropsFileName 	= resourceProps.getPropsFileName()	// Use getter to fetch filename for the resource properties
		servername				= resourceProps.getProperty("servername")
		jndiName				= resourceProps.getProperty("pjndiName")	
		topicName 				= resourceProps.getProperty("ptopicname")
		queueName				= resourceProps.getProperty("pqueueName")	// This value will be "not-set" for create topic
		topicSpace 				= resourceProps.getProperty("ptopicSpace")
		SIBusname 				= resourceProps.getProperty("SIBusname")
		SIBusDestName 			= resourceProps.getProperty("psibusdestname")
		SIBusDestType 			= resourceProps.getProperty("psibusdesttype")
		deliveryMode			= resourceProps.getProperty("pdeliveryMode")
		readAhead				= resourceProps.getProperty("preadAhead")
		timeToLive				= resourceProps.getProperty("ptimeToLive")
		priority				= resourceProps.getProperty("ppriority")
		scope 					= resourceProps.getProperty("scope")
		scope_string 			= resourceProps.getProperty("scope_string")
		config_target_name		= resourceProps.getProperty("config.target.name")
		if (DEBUG) {
			logger.debug("JMSTopic-create: globalPropsFileName=${globalPropsFileName}")
			logger.debug("JMSTopic-create: resourcePropsFileName=${resourcePropsFileName}")
			logger.debug("JMSTopic-create: servername=${servername}")
			logger.debug("JMSTopic-create: appsrvnodename=${appsrvnodename}")
			logger.debug("JMSTopic-create: topicName=${topicName}")
			logger.debug("JMSTopic-create: queueName=${queueName}")
			logger.debug("JMSTopic-create: topicSpace=${topicSpace}")
			logger.debug("JMSTopic-create: SIBusname=${SIBusname}")
			logger.debug("JMSTopic-create: SIBusDestName=${SIBusDestName}")
			logger.debug("JMSTopic-create: SIBusDestType=${SIBusDestType}")
			logger.debug("JMSTopic-create: deliveryMode=${deliveryMode}")
			logger.debug("JMSTopic-create: readAhead=${readAhead}")
			logger.debug("JMSTopic-create: timeToLive=${timeToLive}")
			logger.debug("JMSTopic-create: priority=${priority}")
			logger.debug("JMSTopic-create: scope=${scope}")
			logger.debug("JMSTopic-create: scope_string=${scope_string}")
			logger.debug("JMSTopic-create: config_target_name=${config_target_name}")
			logger.debug("JMSTopic-create: script_ext=${script_ext}")
			logger.debug("JMSTopic-create: websphere_wsadmin_jython=${websphere_wsadmin_jython}")
			logger.debug("JMSTopic-create: websphere_install_dir=${websphere_install_dir}")
			logger.debug("JMSTopic-create: websphere_wsadmin_classpath=${websphere_wsadmin_classpath}")
			logger.debug("JMSTopic-create: dmgrprotocol=${dmgrprotocol}")
			logger.debug("JMSTopic-create: dmgrport=${dmgrport}")
			logger.debug("JMSTopic-create: webappsrv_admin_host=${webappsrv_admin_host}")
			logger.debug("JMSTopic-create: wsadminOptionsList=${wsadminOptionsList}")			
		}
	
		logger.info("JMSTopic-create: Create JMS Topic \"${topicName}\" at scope \"${scope_string}\" using wsadmin${script_ext}...")
		logger.info("JMSTopic-create: Run the createSIBusDestination.jy and createJMSTopic.jy Jython scripts to accomplish this task.")  
		command = "${websphere_install_dir}${File.separator}bin${File.separator}wsadmin${script_ext}"
		options = []
		wsadminOptionsList.each{ options.add(it) }
		options.addAll( [ "-f", "${websphere_scripting_jython}${File.separator}callMultJythonFunction.jy",
						  "${websphere_scripting_jython}",
						  "${resourcePropsFileName}",
						  "${globalPropsFileName}" ] )
		optionsArray = options.toArray()				  
		command_retcode = cmdProc.run(".", command, optionsArray, "JMSTopicCreate_${topicName}_output", "JMSTopicCreate_${topicName}_result")	// Run wsadmin client to execute Jython proc
		command_output  = cmdProc.process_output	// Get output from the command 				  
		logger.info("JMSTopic-create: Command Return code: " + command_retcode)
		if (command_retcode == 0) {	// Command return code of zero indicates wsadmin command ran to completion			
			// Check if the createSIBusDestination.jy Jython proc completed successfully 
			if (  command_output.contains("createSIBusDestination: SIBus destination \"${SIBusDestName}\" successfully created.") ||
				  command_output.contains("createSIBusDestination: No need to create new SIBus destination \"${SIBusDestName}\"; definition already exists.") ) { 
				method_MSG =  "JMSTopic-create: ===> createSIBusDestination.jy COMPLETED SUCCESSFULLY."
				method_RC = 1
				logger.info("${method_MSG}")
			} else {
				method_MSG = "**SEVERE** JMSTopic-create: createSIBusDestination.jy Jython processing FAILED; unable to continue."
				method_RC = -99
				logger.error("${method_MSG}")
				return method_RC
			}
			// Check if the createJMSTopic.jy Jython proc completed successfully 
			if (  command_output.contains("createJMSTopic: JMS Topic \"${topicName}\" successfully created.") ) { 
				method_MSG =  "JMSTopic-create: ===> createJMSTopic.jy COMPLETED SUCCESSFULLY."
				method_RC = 1
				logger.info("${method_MSG}")
			} else {
				method_MSG = "**SEVERE** JMSTopic-create: createJMSTopic.jy Jython processing FAILED; unable to continue."
				method_RC = -99
				logger.error("${method_MSG}")
			}
		} else {
			method_RC = -99
			method_MSG = "**SEVERE** JMSTopic-create: wsadmin${script_ext} command FAILED; RC=" + command_retcode
			logger.error("${method_MSG}")
		}
		return method_RC	// Return to driver indicating failure
	}	// END create method
	
	// *************************************************************************************
	// Method to delete a JMS Topic 
	// General flow:
	//	1. Run the deleteSIBusDestination.jy and deleteJMSTopic.jy Jython scripts
	public int delete(String resourceName, ResourceProperties resourceProps) {
		logger.info("=================================================")
		// Required Resource Properties for this method:
		resourcePropsFileName 	= resourceProps.getPropsFileName()	// Use getter to fetch filename for the resource properties	
		servername				= resourceProps.getProperty("servername")
		topicName 				= resourceProps.getProperty("ptopicname")
		queueName				= resourceProps.getProperty("pqueueName")	// This value will be "not-set" for create topic
		topicSpace 				= resourceProps.getProperty("topicSpace")
		SIBusname 				= resourceProps.getProperty("SIBusname")
		SIBusDestName 			= resourceProps.getProperty("psibusdestname")
		SIBusDestType 			= resourceProps.getProperty("psibusdesttype")		
		scope 					= resourceProps.getProperty("scope")
		scope_string 			= resourceProps.getProperty("scope_string")
		config_target_name		= resourceProps.getProperty("config.target.name")
		
		if (DEBUG) {
			logger.debug("JMSTopic-delete: globalPropsFileName=${globalPropsFileName}")
			logger.debug("JMSTopic-delete: resourcePropsFileName=${resourcePropsFileName}")
			logger.debug("JMSTopic-delete: appsrvnodename=${appsrvnodename}")
			logger.debug("JMSTopic-delete: topicName=${topicName}")
			logger.debug("JMSTopic-delete: queueName=${queueName}")
			logger.debug("JMSTopic-delete: topicSpace=${topicSpace}")
			logger.debug("JMSTopic-delete: SIBusname=${SIBusname}")
			logger.debug("JMSTopic-delete: SIBusDestName=${SIBusDestName}")
			logger.debug("JMSTopic-delete: SIBusDestType=${SIBusDestType}")
			logger.debug("JMSTopic-delete: scope=${scope}")
			logger.debug("JMSTopic-delete: scope_string=${scope_string}")
			logger.debug("JMSTopic-create: config_target_name=${config_target_name}")
			logger.debug("JMSTopic-delete: script_ext=${script_ext}")
			logger.debug("JMSTopic-delete: websphere_wsadmin_jython=${websphere_wsadmin_jython}")
			logger.debug("JMSTopic-delete: websphere_install_dir=${websphere_install_dir}")
			logger.debug("JMSTopic-delete: websphere_wsadmin_classpath=${websphere_wsadmin_classpath}")
			logger.debug("JMSTopic-delete: dmgrprotocol=${dmgrprotocol}")
			logger.debug("JMSTopic-delete: dmgrport=${dmgrport}")
			logger.debug("JMSTopic-delete: webappsrv_admin_host=${webappsrv_admin_host}")
			logger.debug("JMSTopic-delete: wsadminOptionsList=${wsadminOptionsList}")			
		}
	
		logger.info("JMSTopic-delete: Delete JMS Topic \"${topicName}\" at scope \"${scope_string}\" using wsadmin${script_ext}...") 
		logger.info("JMSTopic-delete: Run the deleteSIBusDestination.jy and deleteJMSTopic.jy Jython scripts to accomplish this task.") 
		command = "${websphere_install_dir}${File.separator}bin${File.separator}wsadmin${script_ext}"
		options = []
		wsadminOptionsList.each{ options.add(it) }
		options.addAll( [ "-f", "${websphere_scripting_jython}${File.separator}callMultJythonFunction.jy",
						  "${websphere_scripting_jython}",
						  "${resourcePropsFileName}",
						  "${globalPropsFileName}" ] )
		optionsArray = options.toArray()				  
		command_retcode = cmdProc.run(".", command, optionsArray, "JMSTopicDelete_${topicName}_output", "JMSTopicDelete_${topicName}_result")	// Run wsadmin client to execute Jython proc
		command_output  = cmdProc.process_output	// Get output from the command
		logger.info("JMSTopic-delete: Command Return code: " + command_retcode)
		if (command_retcode == 0) {	// Command return code of zero indicates wsadmin command ran to completion			
			// Check if the deleteSIBusDestination.jy Jython proc completed successfully  
			if (  command_output.contains("deleteSIBusDestination: SIBus destination \"${SIBusDestName}\" successfully deleted.") ||
				  command_output.contains("deleteSIBusDestination: No SIBus Destination entries to delete.") ) { 
				method_MSG =  "JMSTopic-delete: ===> deleteSIBusDestination.jy COMPLETED SUCCESSFULLY."
				method_RC = 1
				logger.info("${method_MSG}")
			} else {
				method_MSG = "**SEVERE** JMSTopic-delete: deleteSIBusDestination.jy Jython processing FAILED; unable to continue."
				method_RC = -99
				logger.error("${method_MSG}")
				return method_RC
			}
			// Check if the deleteJMSTopic.jy Jython proc completed successfully
			if (  command_output.contains("deleteJMSTopic: JMS Topic \"${topicName}\" successfully deleted.") ||
				  command_output.contains("deleteJMSTopic: No JMS Topic entries to delete.") ) { 
				method_MSG =  "JMSTopic-delete: ===> deleteJMSTopic.jy COMPLETED SUCCESSFULLY."
				method_RC = 1
				logger.info("${method_MSG}")
			} else {
				method_MSG = "**SEVERE** JMSTopic-delete: deleteJMSTopic.jy Jython processing FAILED; unable to continue."
				method_RC = -99
				logger.error("${method_MSG}")
			}	
		} else {
			method_RC = -99
			method_MSG = "**SEVERE** JMSTopic-delete: wsadmin${script_ext} command FAILED; RC=" + command_retcode
			logger.error("${method_MSG}")
		}
		return method_RC	// Return to driver indicating failure
	}	// END delete method

}	// END JMSTopic class