// Copyright (c) 2002-2011, SAS Institute Inc., Cary, NC, USA, All Rights Reserved

package com.sas.config.websphere
import com.sas.config.util.CommandProcess
import com.sas.config.util.ResourceProperties

// **************************************************************************************************************
// Class: MailSession
// Purpose: Encapsulate the attributes and methods used to manage and configure a MailSession
// Methods:
//		create 		- create a MailSession
//		delete 		- delete a MailSession
// ****************************************************************************************************************

class MailSession extends Resource {
	// Class variables available to all methods:
	CommandProcess cmdProc
	def globalPropsFileName
	def resourcePropsFileName
	def	servername
	def mailSessionId
	def mailSessionName
	def mailSessionJndiName
	def mailSessionSmtpHost
	def mailSessionUser
	def mailSessionPassword
	Boolean server_mailsrv_requires_authentication
	def scope
	def config_target_name	// SDW-provided calling Ant target name (configure, updateConfigure, updatePasswords, startMidtierServers, unconfigure, changeHost)
	def command 			// Used to hold the command string that will be executed via the Process class
	def options 			// ArrayList used to accumulate command options
	String [] optionsArray 	// String array used to pass options to command  
	String command_output	// Used to hold the stdout output from the command execution
	int    command_retcode  // Used to hold the return code from the command execution					

	// *************************************************************************************
	// CTOR: Initialize global properties object and obtain required global properties 
	public MailSession(GlobalProperties parmGblProps) {
		super(parmGblProps)
		globalPropsFileName = parmGblProps.globalPropsFileName	// Get Global Properties file name
		cmdProc = new CommandProcess()	// Construct object used to run Ant exec task for external commands			
	}	// END CTOR

	// *************************************************************************************
	// Method to create a MailSession 
	// General flow:
	//	1. Create a MailSession using the createMailSession.jy script
	public int create(String resourceName, ResourceProperties resourceProps) {
		logger.info("=================================================")
		// Required Resource Properties for this method:
		resourcePropsFileName 	= resourceProps.getPropsFileName()	// Use getter to fetch filename for the resource properties
		scope					= resourceProps.getProperty("scope")
		config_target_name		= resourceProps.getProperty("config.target.name")
		servername 				= resourceProps.getProperty("servername")
		mailSessionId			= resourceProps.getProperty("mailSessionId")
		mailSessionName 		= resourceProps.getProperty("mailSessionName")
		mailSessionJndiName		= resourceProps.getProperty("mailSessionJndiName")
        mailSessionSmtpHost		= resourceProps.getProperty("mailSessionSmtpHost")
        server_mailsrv_requires_authentication = resourceProps.getProperty("server.mailsrv.requires.authentication")		
		
		if (DEBUG) {
			logger.debug("MailSession-create: globalPropsFileName=${globalPropsFileName}")
			logger.debug("MailSession-create: resourcePropsFileName=${resourcePropsFileName}")
			logger.debug("MailSession-create: servername=${servername}")
			logger.debug("MailSession-create: appsrvnodename=${appsrvnodename}")
			logger.debug("MailSession-create: scope=${scope}")
			logger.debug("MailSession-create: config_target_name=${config_target_name}")
			logger.debug("MailSession-create: script_ext=${script_ext}")
			logger.debug("MailSession-create: websphere_wsadmin_jython=${websphere_wsadmin_jython}")
			logger.debug("MailSession-create: websphere_install_dir=${websphere_install_dir}")
			logger.debug("MailSession-create: websphere_wsadmin_classpath=${websphere_wsadmin_classpath}")
			logger.debug("MailSession-create: dmgrprotocol=${dmgrprotocol}")
			logger.debug("MailSession-create: dmgrport=${dmgrport}")
			logger.debug("MailSession-create: webappsrv_admin_host=${webappsrv_admin_host}")
			logger.debug("MailSession-create: wsadminOptionsList=${wsadminOptionsList}")
			logger.debug("MailSession-create: mailSessionId=${mailSessionId}")
			logger.debug("MailSession-create: mailSessionName=${mailSessionName}")
			logger.debug("MailSession-create: mailSessionJndiName=${mailSessionJndiName}")
			logger.debug("MailSession-create: mailSessionSmtpHost=${mailSessionSmtpHost}")
			logger.debug("MailSession-create: server_mailsrv_requires_authentication=${server_mailsrv_requires_authentication}")						
		}		
		
		// Determine if userId and password are required and solicit them from the user, if they do not exist in the credentials.properties file
		if (server_mailsrv_requires_authentication) {
			logger.info("MailSession-create: Mail server requires authentication; obtain credentials from \"${websphere_scripting_credentials}\"")
			ResourceProperties credentialsProps = new ResourceProperties(websphere_scripting_credentials)	// Object used to read the websphere_scripting_credentials properties file
			String mailSessionUserid = credentialsProps.getProperty("MailSession.create_${scope}_SMTP-Mail-Server_userid")
			String mailSessionPasswd = credentialsProps.getProperty("MailSession.create_${scope}_SMTP-Mail-Server_passwd")
			// If userId and/or password are null in the credentials.properties file, can't continue because Mail Server needs credentials 
			def mailSessionErrorType = ""
			if (mailSessionUserid == "") 
				mailSessionErrorType = "userId"
			if (mailSessionPasswd == "")
				mailSessionErrorType += " password"
			if (mailSessionErrorType) {
				method_MSG = "**SEVERE** MailSession-create: Mail Server requires authentication but ${mailSessionErrorType} not provided; unable to continue."
				method_RC = -99
				logger.error("${method_MSG}")
				return method_RC 
			}
		}	// END Mail POP server requires authentication
		
		// 1. Create a MailSession 
		logger.info("MailSession-create: Create MailSession \"${mailSessionName}\" using the wsadmin${script_ext} command at scope \"${scope}\"...")
		command = "${websphere_install_dir}${File.separator}bin${File.separator}wsadmin${script_ext}"
		options = []
		wsadminOptionsList.each{ options.add(it) }
		options.addAll( [ "-f", "${websphere_scripting_jython}${File.separator}callJythonFunction.jy",
						  "${websphere_scripting_jython}",
						  "createMailSession",
						  "${resourcePropsFileName}",
						  "${globalPropsFileName}" ] )
		optionsArray = options.toArray()
		command_retcode = cmdProc.run(".", command, optionsArray, "MailSessionCreate_${mailSessionName}_output", "MailSessionCreate_${mailSessionName}_result")	// Run wsadmin client to execute Jython proc
		command_output  = cmdProc.process_output	// Get output from the command 				  
		logger.info("MailSession-create: Command Return code: " + command_retcode)
		if (command_retcode == 0) {	// Command return code of zero indicates wsadmin command ran to completion			
			// Check if the createMailSession.jy Jython proc completed successfully
			if (  command_output.contains("createMailSession: MailSession \"${mailSessionName}\" has been successfully created.") ) { 
				method_MSG =  "MailSession-create: ===> createMailSession.jy COMPLETED SUCCESSFULLY."
				method_RC = 1
			} else {
				method_MSG = "**SEVERE** MailSession-create: createMailSession.jy Jython processing FAILED; unable to continue."
				method_RC = -99
				logger.error("${method_MSG}")
				return method_RC
			}
		} else {
			method_RC = -1
			method_MSG = "**ERROR** MailSession-create: wsadmin${script_ext} command failed; RC=" + command_retcode
			logger.error("${method_MSG}")
		}
		logger.info("${method_MSG}")
		return method_RC	// Return to driver indicating failure
	}	// END create method	

	// *************************************************************************************
	// Method to delete a MailSession 
	// General flow:
	//	1. Delete a MailSession using the deleteMailSession.jy script
	public int delete(String resourceName, ResourceProperties resourceProps) {
		logger.info("=================================================")
		// Required Resource Properties for this method:
		resourcePropsFileName 	= resourceProps.getPropsFileName()	// Use getter to fetch filename for the resource properties
		servername 				= resourceProps.getProperty("servername")
		mailSessionName 		= resourceProps.getProperty("mailSessionName")
		scope 					= resourceProps.getProperty("scope")
		config_target_name		= resourceProps.getProperty("config.target.name")

		if (DEBUG) {
			logger.debug("MailSession-delete: globalPropsFileName=${globalPropsFileName}")
			logger.debug("MailSession-delete: resourcePropsFileName=${resourcePropsFileName}")		
			logger.debug("MailSession-delete: servername=${servername}")
			logger.debug("MailSession-delete: appsrvnodename=${appsrvnodename}")
			logger.debug("MailSession-delete: mailSessionName=${mailSessionName}")
			logger.debug("MailSession-delete: scope=${scope}")
			logger.debug("MailSession-delete: config_target_name=${config_target_name}")
			logger.debug("MailSession-delete: script_ext=${script_ext}")
			logger.debug("MailSession-delete: websphere_wsadmin_jython=${websphere_wsadmin_jython}")
			logger.debug("MailSession-delete: websphere_install_dir=${websphere_install_dir}")
			logger.debug("MailSession-delete: websphere_wsadmin_classpath=${websphere_wsadmin_classpath}")
			logger.debug("MailSession-delete: dmgrprotocol=${dmgrprotocol}")
			logger.debug("MailSession-delete: dmgrport=${dmgrport}")
			logger.debug("MailSession-delete: webappsrv_admin_host=${webappsrv_admin_host}")
			logger.debug("MailSession-delete: wsadminOptionsList=${wsadminOptionsList}")						
		}
				
		// 1. Delete a MailSession 
		logger.info("MailSession-delete: Delete MailSession \"${mailSessionName}\" using the wsadmin${script_ext} command...")
		command = "${websphere_install_dir}${File.separator}bin${File.separator}wsadmin${script_ext}"
		options = []
		wsadminOptionsList.each{ options.add(it) }
		options.addAll( [ "-f", "${websphere_scripting_jython}${File.separator}callJythonFunction.jy",
						  "${websphere_scripting_jython}",
						  "deleteMailSession",
						  "${resourcePropsFileName}",
						  "${globalPropsFileName}" ] )
		optionsArray = options.toArray()				  
		command_retcode = cmdProc.run(".", command, optionsArray, "MailSessionDelete_${mailSessionName}_output", "MailSessionDelete_${mailSessionName}_result")	// Run wsadmin client to execute Jython proc
		command_output  = cmdProc.process_output	// Get output from the command 				  
		logger.info("MailSession-delete: Command Return code: " + command_retcode)
		if (command_retcode == 0) {	// Command return code of zero indicates wsadmin command ran to completion			
			// Check if the deleteMailSession.jy Jython proc completed successfully
			if (  command_output.contains("deleteMailSession: MailSession \"${mailSessionName}\" has been removed.") ) { 
				method_MSG =  "MailSession-delete: ===> deleteMailSession.jy COMPLETED SUCCESSFULLY."
				method_RC = 1
			} else {
				method_MSG = "**SEVERE** MailSession-delete: deleteMailSession.jy Jython processing FAILED; unable to continue."
				method_RC = -99
				logger.error("${method_MSG}")
				return method_RC
			}
		} else {
			method_RC = -1
			method_MSG = "**ERROR** MailSession-delete: wsadmin${script_ext} command failed; RC=" + command_retcode
			logger.error("${method_MSG}")
		}
		logger.info("${method_MSG}")
		return method_RC	// Return to driver indicating failure
	}	// END delete method			
	
}	// END MailSession class