// Copyright (c) 2002-2011, SAS Institute Inc., Cary, NC, USA, All Rights Reserved

package com.sas.config.websphere
import com.sas.config.util.CommandProcess
import com.sas.config.util.ResourceProperties

// **************************************************************************************************************
// Class: MultiCast
// Purpose: Encapsulate the attributes and methods used to reset the Multicast Authentication Token  
// Methods:
//		reset - reset the Multicast Authentication Token for the desired server for updatePasswords processing
// ****************************************************************************************************************

class MultiCast extends Resource {
	// Class variables available to all methods:
	CommandProcess cmdProc
	def globalPropsFileName
	def resourcePropsFileName
	def servername
	def jvmOptions
	def config_target_name	// SDW-provided calling Ant target name (configure, updateConfigure, updatePasswords, startMidtierServers, unconfigure, changeHost)
	def command 			// Used to hold the command string that will be executed via the Process class
	def options 			// ArrayList used to accumulate command options
	String [] optionsArray 	// String array used to pass options to command  
	String command_output	// Used to hold the stdout output from the command execution
	int    command_retcode  // Used to hold the return code from the command execution						

	// *************************************************************************************
	// CTOR: Initialize global properties object and obtain required global properties 
	// Global properties are fetched and initialized by the base Resource class
	public MultiCast(GlobalProperties parmGblProps) {
		super(parmGblProps)		// Load the global properties using the abstract base class CTOR
		globalPropsFileName = parmGblProps.globalPropsFileName	// Get Global Properties file name
		cmdProc = new CommandProcess()	// Construct object used to run Ant exec task for external commands	
	}
	
	// ***********************************************************************************************************************************
	// Method to reset the Multicast Authentication Token for the desired server during updatePasswords processing 
	// Required properties:
	//		appsrvnodename 	- from GlobalProperties file
	//		servername 		- from ResourceProperties file
	//		jvmOptions		- from ResourceProperties file
	public int reset(String resourceName, ResourceProperties resourceProps) {
		logger.info("=================================================")
		// Required Resource Properties for this method:
		resourcePropsFileName 	= resourceProps.getPropsFileName()	// Use getter to fetch filename for the resource properties
		servername 				= resourceProps.getProperty("servername")
		jvmOptions				= resourceProps.getProperty("jvmOptions")
		config_target_name		= resourceProps.getProperty("config.target.name")
		
		// Obtain the Multicast token (password) from the credentials.properties file and add it to the JVM argument
		ResourceProperties credentialsProps	= new ResourceProperties(websphere_scripting_credentials)	// Object used to read the websphere_scripting_credentials properties file
		String midtier_multicast_auth_token_passwd = credentialsProps.getProperty("MultiCast.reset_SERVER_${servername}_passwd")  // Multicast password token
		jvmOptions += midtier_multicast_auth_token_passwd	// Add Multicast auth token password value to JVM option
		
		logger.info("MultiCast-reset: Reset the Multicast Authentication Token for server \"${servername}\"...")
		command = "${websphere_install_dir}${File.separator}bin${File.separator}wsadmin${script_ext}"
		options = []
		wsadminOptionsList.each{ options.add(it) }
		options.addAll( [ "-f", "${websphere_scripting_jython}${File.separator}callJythonFunction.jy",
						  "${websphere_scripting_jython}",
						  "setJvmArguments",
						  "${resourcePropsFileName}",
						  "${globalPropsFileName}" ] )
		optionsArray = options.toArray()				  
		command_retcode = cmdProc.run(".", command, optionsArray, "MultiCastReset_${servername}_output", "MultiCastReset_${servername}_result")	// Run wsadmin client to execute Jython proc
		command_output  = cmdProc.process_output	// Get output from the command 				  
		logger.info("MultiCast-reset: Command Return code: " + command_retcode)			

		if (command_retcode == 0) {	// Command return code of zero indicates wsadmin command ran to completion	
			// Check if the setJvmArguments.jy Jython proc completed successfully
			if (command_output.contains("setJvmArguments: Modifications to JVM Arguments completed.")) {
				method_MSG = "MultiCast-reset: ===> setJvmArguments.jy COMPLETED SUCCESSFULLY; JVM generic arguments for server \"${servername}\" have been updated."
				method_RC = 1
				logger.info("${method_MSG}")
			} else {
				method_MSG = "**SEVERE** MultiCast-reset: setJvmArguments.jy Jython processing FAILED; unable to continue."
				method_RC = -99
				logger.error("${method_MSG}")
			}	
		} else {
			method_RC = -99
			method_MSG = "**SEVERE** Server-update: wsadmin${script_ext} command failed; RC=" + command_retcode
			logger.error("${method_MSG}")
		}
		return method_RC	
	}	// END reset method
		
}	// END MultiCast class