// Copyright (c) 2002-2011, SAS Institute Inc., Cary, NC, USA, All Rights Reserved

package com.sas.config.websphere
import com.sas.config.util.CommandProcess
import com.sas.config.util.ResourceProperties

// **************************************************************************************************************
// Class: ObjectCache
// Purpose: Encapsulate the attributes and methods used to manage and configure an Object Cache 
// Methods:
//		create - Create an Object Cache definition
//		delete - Delete an Object Cache definition
// ****************************************************************************************************************

class ObjectCache extends Resource {
	// Class variables available to all methods:
	CommandProcess cmdProc	// Object used to run an external command using Ant exec task
	def globalPropsFileName
	def resourcePropsFileName
	def servername
	def scope
	def scope_string
	def cacheObjName
	def cacheObjJndiName
	def cacheObjCacheSize
	def cacheObjDefaultPriority
	def config_target_name	// SDW-provided calling Ant target name (configure, updateConfigure, updatePasswords, startMidtierServers, unconfigure, changeHost)
	String command 			// Used to hold the command string that will be executed via the Process class
	def options 			// ArrayList used to accumulate command options
	String [] optionsArray 	// String array used to pass options to command  
	String command_output	// Used to hold the stdout output from the command execution
	int    command_retcode  // Used to hold the return code from the command execution		

	// *************************************************************************************
	// CTOR: Initialize global properties object and obtain required global properties 
	// Global properties are fetched and initialized by the base Resource class
	public ObjectCache(GlobalProperties parmGblProps) {
		super(parmGblProps)		// Load the global properties using the abstract base class CTOR
		globalPropsFileName = parmGblProps.globalPropsFileName	// Get Global Properties file name
		cmdProc = new CommandProcess()	// Construct object used to run Ant exec task for external commands	
	}

	// *************************************************************************************
	// Method to create an Object Cache 
	// General flow:
	//	1. Run the createObjectCache.jy Jython script
	public int create(String resourceName, ResourceProperties resourceProps) {
		logger.info("=================================================")
		// Required Resource Properties for this method:
		resourcePropsFileName 	= resourceProps.getPropsFileName()	// Use getter to fetch filename for the resource properties	
		servername 				= resourceProps.getProperty("servername")
		cacheObjName 			= resourceProps.getProperty("cacheObjName")
		cacheObjJndiName 		= resourceProps.getProperty("cacheObjJndiName")
		cacheObjCacheSize		= resourceProps.getProperty("cacheObjCacheSize")
		cacheObjDefaultPriority = resourceProps.getProperty("cacheObjDefaultPriority")
		scope 					= resourceProps.getProperty("scope")
		scope_string 			= resourceProps.getProperty("scope_string")
		config_target_name		= resourceProps.getProperty("config.target.name")
		if (DEBUG) {
			logger.debug("ObjectCache-create: globalPropsFileName=${globalPropsFileName}")
			logger.debug("ObjectCache-create: resourcePropsFileName=${resourcePropsFileName}")
			logger.debug("ObjectCache-create: appsrvnodename=${appsrvnodename}")
			logger.debug("ObjectCache-create: cacheObjName=${cacheObjName}")
			logger.debug("ObjectCache-create: cacheObjJndiName=${cacheObjJndiName}")
			logger.debug("ObjectCache-create: cacheObjCacheSize=${cacheObjCacheSize}")
			logger.debug("ObjectCache-create: cacheObjDefaultPriority=${cacheObjDefaultPriority}")
			logger.debug("ObjectCache-create: scope=${scope}")
			logger.debug("ObjectCache-create: scope_string=${scope_string}")
			logger.debug("ObjectCache-create: config_target_name=${config_target_name}")
			logger.debug("ObjectCache-create: script_ext=${script_ext}")
			logger.debug("ObjectCache-create: websphere_wsadmin_jython=${websphere_wsadmin_jython}")
			logger.debug("ObjectCache-create: websphere_install_dir=${websphere_install_dir}")
			logger.debug("ObjectCache-create: websphere_wsadmin_classpath=${websphere_wsadmin_classpath}")
			logger.debug("ObjectCache-create: dmgrprotocol=${dmgrprotocol}")
			logger.debug("ObjectCache-create: dmgrport=${dmgrport}")
			logger.debug("ObjectCache-create: webappsrv_admin_host=${webappsrv_admin_host}")
			logger.debug("ObjectCache-create: wsadminOptionsList=${wsadminOptionsList}")						
		}		
		
		logger.info("ObjectCache-create: Create Object Cache \"${cacheObjName}\" with JNDI name \"${cacheObjJndiName}\"  at scope \"${scope_string}\"...")  
		command = "${websphere_install_dir}${File.separator}bin${File.separator}wsadmin${script_ext}"
		options = []
		wsadminOptionsList.each{ options.add(it) }
		options.addAll( [ "-f", "${websphere_scripting_jython}${File.separator}callJythonFunction.jy",
						  "${websphere_scripting_jython}",
						  "createObjectCache",
						  "${resourcePropsFileName}",
						  "${globalPropsFileName}" ] )
		optionsArray = options.toArray()
		command_retcode = cmdProc.run(".", command, optionsArray, "ObjectCacheCreate_${cacheObjName}_output", "ObjectCacheCreate_${cacheObjName}_result")	// Run wsadmin client to execute Jython proc
		command_output  = cmdProc.process_output	// Get output from the command 				  
		logger.info("ObjectCache-create: Command Return code: " + command_retcode)
		if (command_retcode == 0) {	// Command return code of zero indicates wsadmin command ran to completion			
			// Check if the createObjectCache.jy Jython proc completed successfully 
			if (  command_output.contains("createObjectCache: Object Cache \"${cacheObjName}\" was successfully created.") ) { 
				method_MSG =  "ObjectCache-create: ===> createObjectCache.jy COMPLETED SUCCESSFULLY."
				method_RC = 1
				logger.info("${method_MSG}")
			} else {
				method_MSG = "**SEVERE** ObjectCache-create: createObjectCache.jy Jython processing FAILED; unable to continue."
				method_RC = -99
				logger.error("${method_MSG}")
			}
		} else {
			method_RC = -99
			method_MSG = "**SEVERE** ObjectCache-create: wsadmin${script_ext} command FAILED; RC=" + command_retcode
			logger.error("${method_MSG}")
		}
		return method_RC	// Return to driver indicating failure
	}	// END create method

	// *************************************************************************************
	// Method to delete an Object Cache 
	// General flow:
	//	1. Run the deleteObjectCache.jy Jython script
	public int delete(String resourceName, ResourceProperties resourceProps) {
		logger.info("=================================================")
		resourcePropsFileName 	= resourceProps.getPropsFileName()	// Use getter to fetch filename for the resource properties	
		servername 				= resourceProps.getProperty("servername")		
		cacheObjName 			= resourceProps.getProperty("cacheObjName")
		scope 					= resourceProps.getProperty("scope")
		scope_string 			= resourceProps.getProperty("scope_string")
		config_target_name		= resourceProps.getProperty("config.target.name")
		if (DEBUG) {
			logger.debug("ObjectCache-create: globalPropsFileName=${globalPropsFileName}")
			logger.debug("ObjectCache-delete: resourcePropsFileName=${resourcePropsFileName}")
			logger.debug("ObjectCache-delete: appsrvnodename=${appsrvnodename}")
			logger.debug("ObjectCache-delete: cacheObjName=${cacheObjName}")
			logger.debug("ObjectCache-delete: scope=${scope}")
			logger.debug("ObjectCache-delete: scope_string=${scope_string}")
			logger.debug("ObjectCache-delete: config_target_name=${config_target_name}")
			logger.debug("ObjectCache-delete: script_ext=${script_ext}")
			logger.debug("ObjectCache-delete: websphere_wsadmin_jython=${websphere_wsadmin_jython}")
			logger.debug("ObjectCache-delete: websphere_install_dir=${websphere_install_dir}")
			logger.debug("ObjectCache-delete: websphere_wsadmin_classpath=${websphere_wsadmin_classpath}")
			logger.debug("ObjectCache-delete: dmgrprotocol=${dmgrprotocol}")
			logger.debug("ObjectCache-delete: dmgrport=${dmgrport}")
			logger.debug("ObjectCache-delete: webappsrv_admin_host=${webappsrv_admin_host}")
			logger.debug("ObjectCache-delete: wsadminOptionsList=${}")						
		}		
		
		logger.info("ObjectCache-delete: Delete Object Cache \"${cacheObjName}\" at scope \"${scope_string}\"...")  
		command = "${websphere_install_dir}${File.separator}bin${File.separator}wsadmin${script_ext}"
		options = []
		wsadminOptionsList.each{ options.add(it) }
		options.addAll( [ "-f", "${websphere_scripting_jython}${File.separator}callJythonFunction.jy",
						  "${websphere_scripting_jython}",
						  "deleteObjectCache",
						  "${resourcePropsFileName}",
						  "${globalPropsFileName}" ] )
		optionsArray = options.toArray()				  
		command_retcode = cmdProc.run(".", command, optionsArray, "ObjectCacheDelete_${cacheObjName}_output", "ObjectCacheDelete_${cacheObjName}_result")	// Run wsadmin client to execute Jython proc
		command_output  = cmdProc.process_output	// Get output from the command 				  
		logger.info("ObjectCache-delete: Command Return code: " + command_retcode)
		if (command_retcode == 0) {	// Command return code of zero indicates wsadmin command ran to completion			
			// Check if the deleteObjectCache.jy Jython proc completed successfully 
			if (  command_output.contains("deleteObjectCache: Object Cache \"${cacheObjName}\" has been removed.") ) { 
				method_MSG =  "ObjectCache-delete: ===> deleteObjectCache.jy COMPLETED SUCCESSFULLY."
				method_RC = 1
				logger.info("${method_MSG}")
			} else {
				method_MSG = "**SEVERE** ObjectCache-delete: deleteObjectCache.jy Jython processing FAILED; unable to continue."
				method_RC = -99
				logger.error("${method_MSG}")
			}
		} else {
			method_RC = -99
			method_MSG = "**SEVERE** ObjectCache-delete: wsadmin${script_ext} command FAILED; RC=" + command_retcode
			logger.error("${method_MSG}")
		}
		return method_RC	// Return to driver indicating failure
	}	// END delete method

}	// END ObjectCache class