// Copyright (c) 2002-2011, SAS Institute Inc., Cary, NC, USA, All Rights Reserved

package com.sas.config.websphere
import org.apache.log4j.Logger

// *********************************************************************************************************************
// Abstract Class: Resource
// Purpose: Encapsulate the attributes and methods used by all Resource classes
// Implemented Methods:
//		Constructor - instantiate the Resource object, which assigns the Global Properties values to class fields. 
//					  The class fields are used by all Resource types.
// *********************************************************************************************************************
 
abstract class Resource {
	final static Logger logger = Logger.getLogger("com.sas.config");	// Instantiate singleton of log4j object for logging
	// Class variables available to all methods:
	int    method_RC						// Method return code
	String method_MSG 						// Method success or failure message
	String method_VALUE						// Method result value 
	
	// Ensure all child classes have access to all Global properties 
	GlobalProperties gblProps   			// Object for global properties	
	String  globalPropsFileName				// Filename for global properties file
	String  scriptingDirectory				// Scripting directory path
	Boolean DEBUG							// DEBUG flag 
	Boolean DEBUG_WASDriver					// DEBUG flag for the WAS driver program
	String  File_separator					// File separation character for use in Velocity scripts
	String  appsrvnodename					// WAS managed node name
	String  appsrvnodermiport				// WAS nodeagent RMI port number
	String  appsrvnodesoapport				// WAS nodeagent SOAP port number
	String  cellname						// WAS cell name
	String  config_lev_web_common_dir		// SDW-provided <SAS-Config>/Lev1/Web directory
	String  config_lev_web_staging_dir		// SDW-provided <SAS-Config>/Lev1/Web/Staging directory
	String  config_target_name				// SDW-provided calling Ant target name (configure, updateConfigure, updatePasswords, startMidtierServers, unconfigure, changeHost)
	String  defaultsibusname				// WAS default SIBus name
	String  dmgr							// WAS dmgr server name
	String  dmgrhttpport					// WAS dmgr HTTP port
	String  dmgrhttpsport					// WAS dmgr HTTPS port
	String  dmgrnodename					// WAS dmgr node name
	String  dmgrport						// WAS SOAP or RMI port used for communications between node and dmgr
	String  dmgrprofilename					// WAS dmgr profile name
	String  dmgrprotocol					// WAS dmgr communications protocol (SOAP or RMI) used between node and dmgr
	String  dmgrrmiport						// WAS dmgr RMI port number
	String  dmgrrmiprops					// WAS dmgr RMI client properties filename ("sas.client.props")
	String  dmgrrmitimeout					// WAS dmgr RMI timeout value
	String  dmgrsoapport					// WAS dmgr SOAP port number
	String  dmgrsoapprops					// WAS dmgr SOAP client properties filename ("soap.client.props")
	String  dmgrsoaptimeout					// WAS dmgr SOAP timeout value
	String  dmgripcprops					// WAS dmgr IPC client properties filename ("ipc.client.props")
	String  dmgripctimeout					// WAS dmgr IPC timeout value
	Boolean mustCreateDmgrProfile			// WAS create dmgr profile if "true", otherwise, unnecessary
	String  nodeagent						// WAS name of nodeagent server
	String  nodermiprops					// WAS node RMI properties file
	String  nodermitimeout					// WAS node RMI timeout value
	String  nodesoapprops					// WAS node SOAP properties file
	String  nodesoaptimeout					// WAS node SOAP timeout value
	String  nodeipcprops					// WAS node IPC properties file
	String  nodeipctimeout					// WAS node IPC timeout value
	String  os_localhost_default_locale 	// Locale ("en" or "ja", "zn", etc.)
	String  os_localhost_machine_type		// Machine architecture ("lax", "win", "wx6", "r64", "sax", "s64", and "zos")	
	String  os_localhost_type				// Host type ("unx", "win" or "zos")
	String  profilename						// WAS node profile name
	String  script_executable				// Script executable pgm "cmd" or "/bin/sh" or "/bin/ksh"
	String  script_ext						// Script extension ".bat" or ".sh"	
	String  script_flag						// Script flag "/c" or "-c"
	Boolean webapp_auto_deploy				// SDW auto-deploy (true) or manual deploy (false)
	String  webappsrv_admin_host			// WAS Admin dmgr host
	Boolean webappsrv_admin_security_is_enabled	// WAS Admin Security is enabled (true) or disabled (false)	
	Boolean webappsrv_auto_config			// SDW auto-config (true) or manual config (false)
	Boolean webappsrv_policy_use_restrictive	// WAS Java2 Security is enforced (true)
	Boolean webappsrv_scripting_cache_credentials	// Whether it is ok to save userIds and passwords in a file
	String  websphere_appsrv_logs			// WAS node logs
	String  websphere_dmgr_logs				// WAS dmgr logs
	String  websphere_full_version			// WAS full version with fixpack (e.g. "7.0.0.11")	
	String  websphere_install_dir			// WAS install directory
	String  websphere_profile_dir			// WAS profiles directory
	String  websphere_scripting_classpath	// Scripting classpath for Groovy compiles and runtime
	String  websphere_scripting_dir			// Scripting directory name
	String  websphere_scripting_jython		// Scripting Jython scripts location
	String  websphere_scripting_lib			// Scripting Java lib containing all necessary jars
	String  websphere_scripting_logs		// Scripting logs directory
	String  websphere_scripting_props		// Scripting properties directory containing the global and resource properties files
	String  websphere_scripting_scripts		// Scripting scripts to drive the WAS config
	String  websphere_scripting_src			// Scripting Groovy source directory
	String  websphere_scripting_tasks		// Scripting tasks directory containing the websphere.configuration.tasks file
	String  websphere_scripting_tasks_file	// Scripting default task file full path name
	String 	websphere_scripting_credentials	// Scripting filename for WAS credentials file 
	String  websphere_temp_dir				// SDW temporary directory for WebSphere 
	String  websphere_wsadmin_classpath		// Classpath for use by wsadmin command
	String  websphere_wsadmin_jython		// Location of SAS Jython code for WAS configs
	Boolean webspherend_is_installed		// WAS is installed (true) or not installed (false)
	def		wsadminOptionsList				// ArrayList of common wsadmin options

	// *************************************************************************************
	// CTOR: Initialize global properties object and obtain required global properties 
	public Resource(GlobalProperties parmGblProps) {
		gblProps 									= parmGblProps									// Save Global Properties object reference
		scriptingDirectory 							= gblProps.scriptingDirectory					// Scripting directory path				
		globalPropsFileName							= gblProps.globalPropsFileName					// File name of Global Properties file	
		DEBUG 										= gblProps.DEBUG								// DEBUG flag 
		DEBUG_WASDriver 							= gblProps.DEBUG_WASDriver						// DEBUG flag for WAS driver program
		File_separator 								= gblProps.File_separator						// File separator character for use in Velocity scripts
		config_lev_web_common_dir					= gblProps.config_lev_web_common_dir			// SDW-provided <SAS-Config>/Lev1/Web directory
		config_lev_web_staging_dir					= gblProps.config_lev_web_staging_dir			// SDW-provided <SAS-Config>/Lev1/Web/Staging directory
		appsrvnodename 								= gblProps.appsrvnodename						// WAS managed node name
		appsrvnodermiport 							= gblProps.appsrvnodermiport 					// WAS nodeagent RMI port number
		appsrvnodesoapport 							= gblProps.appsrvnodesoapport					// WAS nodeagent SOAP port number
		cellname 									= gblProps.cellname								// WAS cell name
		defaultsibusname 							= gblProps.defaultsibusname						// WAS default SIBus name
		dmgr 										= gblProps.dmgr									// WAS dmgr server name
		dmgrnodename 								= gblProps.dmgrnodename							// WAS dmgr node name
		dmgrprofilename 							= gblProps.dmgrprofilename						// WAS dmgr profile name		
		dmgrhttpport 								= gblProps.dmgrhttpport							// WAS dmgr HTTP port
		dmgrhttpsport 								= gblProps.dmgrhttpsport						// WAS dmgr HTTPS port
		dmgrport 									= gblProps.dmgrport								// WAS SOAP or RMI port used for communications between node and dmgr						
		dmgrprotocol 								= gblProps.dmgrprotocol							// WAS dmgr communications protocol (SOAP or RMI) used between node and dmgr
		dmgrrmiport 								= gblProps.dmgrrmiport							// WAS dmgr RMI port number
		dmgrrmiprops 								= gblProps.dmgrrmiprops							// WAS dmgr RMI client properties filename ("sas.client.props")
		dmgrrmitimeout 								= gblProps.dmgrrmitimeout						// WAS dmgr RMI timeout value
		dmgrsoapport 								= gblProps.dmgrsoapport							// WAS dmgr SOAP port number
		dmgrsoapprops								= gblProps.dmgrsoapprops						// WAS dmgr SOAP client properties filename ("soap.client.props")
		dmgrsoaptimeout 							= gblProps.dmgrsoaptimeout						// WAS dmgr SOAP timeout value
		dmgripcprops								= gblProps.dmgripcprops							// WAS dmgr IPC client properties filename ("ipc.client.props")
		dmgripctimeout								= gblProps.dmgripctimeout						// WAS dmgr IPC timeout value
		mustCreateDmgrProfile 						= gblProps.mustCreateDmgrProfile				// WAS create dmgr profile if "true", otherwise unnecessary 		
		nodeagent 									= gblProps.nodeagent							// WAS name of nodeagent server
		nodermiprops 								= gblProps.nodermiprops							// WAS node RMI properties file
		nodermitimeout 								= gblProps.nodermitimeout						// WAS node RMI timeout value
		nodesoapprops 								= gblProps.nodesoapprops						// WAS node SOAP properties file
		nodesoaptimeout 							= gblProps.nodesoaptimeout						// WAS node SOAP timeout value
		nodeipcprops 								= gblProps.nodeipcprops							// WAS node IPC properties file
		nodeipctimeout								= gblProps.nodeipctimeout						// WAS node IPC timeout value
		os_localhost_default_locale 				= gblProps.os_localhost_default_locale			// Locale ("en" or "ja", "zn", etc.)	
		os_localhost_type 							= gblProps.os_localhost_type					// Host type ("unx", "win" or "zos")
		os_localhost_machine_type 					= gblProps.os_localhost_machine_type			// Machine architecture ("lax", "win", "wx6", "r64", "sax", "s64", and "zos")  
		profilename 								= gblProps.profilename							// WAS node profile name
		script_executable 							= gblProps.script_executable					// Script executable pgm "cmd" or "/bin/sh" or "/bin/ksh"
		script_ext 									= gblProps.script_ext							// Script extension ".bat" or ".sh"	
		script_flag 								= gblProps.script_flag							// Script flag "/c" or "-c"		
		webapp_auto_deploy 							= gblProps.webapp_auto_deploy					// SDW auto-deploy (true) or manual deploy (false)
		webappsrv_admin_security_is_enabled 		= gblProps.webappsrv_admin_security_is_enabled	// WAS Admin Security is enabled (true) or disabled (false)
		webappsrv_auto_config 						= gblProps.webappsrv_auto_config				// SDW auto-config (true) or manual config (false)
		webappsrv_scripting_cache_credentials		= gblProps.webappsrv_scripting_cache_credentials  // Whether it is ok to save userIds and passwords in a file
		webspherend_is_installed					= gblProps.webspherend_is_installed				// WAS is installed (true) or not installed (false)			
		webappsrv_admin_host 						= gblProps.webappsrv_admin_host					// WAS Admin dmgr host
		websphere_appsrv_logs 						= gblProps.websphere_appsrv_logs				// WAS node logs location
		websphere_dmgr_logs 						= gblProps.websphere_dmgr_logs					// WAS dmgr logs location
		webappsrv_policy_use_restrictive 			= gblProps.webappsrv_policy_use_restrictive		// WAS Java2 Security is enforced (true)
		websphere_full_version 						= gblProps.websphere_full_version				// WAS full version with fixpack (e.g. "7.0.0.11")	
		websphere_install_dir 						= gblProps.websphere_install_dir				// WAS install directory
		websphere_profile_dir 						= gblProps.websphere_profile_dir				// WAS profiles directory
		websphere_scripting_dir 					= gblProps.websphere_scripting_dir				// Scripting directory name
		websphere_scripting_lib 					= gblProps.websphere_scripting_lib				// Scripting Java lib containing all necessary jars
		websphere_scripting_logs 					= gblProps.websphere_scripting_logs				// Scripting logs directory
		websphere_scripting_props 					= gblProps.websphere_scripting_props			// Scripting properties directory containing the global and resource properties files
		websphere_scripting_src						= gblProps.websphere_scripting_src				// Scripting Groovy source directory
		websphere_scripting_tasks 					= gblProps.websphere_scripting_tasks			// Scripting tasks directory containing the websphere.configuration.tasks file
		websphere_scripting_tasks_file				= gblProps.websphere_scripting_tasks_file		// Scripting default task file full path name 
		websphere_scripting_scripts					= gblProps.websphere_scripting_scripts			// Scripting scripts used to driver WAS config
		websphere_scripting_jython					= gblProps.websphere_scripting_jython			// Scripting Jython scripts location 
		websphere_scripting_classpath				= gblProps.websphere_scripting_classpath		// Scripting Groovy compile and runtime classpath
		websphere_scripting_credentials				= gblProps.websphere_scripting_credentials		// Scripting filename for WAS credentials file
		websphere_temp_dir 							= gblProps.websphere_temp_dir					// SDW temporary directory for WebSphere 
		websphere_wsadmin_classpath					= gblProps.websphere_wsadmin_classpath			// Classpath for use by wsadmin command
		websphere_wsadmin_jython 					= gblProps.websphere_wsadmin_jython				// Location of SAS Jython code for WAS configs
		wsadminOptionsList 							= gblProps.wsadminOptionsList					// ArrayList of common wsadmin options		
	
		if (DEBUG) {
			logger.debug("Resource: ============= Begin All Property Values ============")
			logger.debug("Resource: globalPropsFileName=${globalPropsFileName}")		
			logger.debug("Resource: scriptingDirectory=${scriptingDirectory}")
			logger.debug("Resource: websphere_scripting_props=${websphere_scripting_props}") 
			logger.debug("Resource: DEBUG=${DEBUG}")						
			logger.debug("Resource: DEBUG_WASDriver=${DEBUG_WASDriver}")		
			logger.debug("Resource: appsrvnodename=${appsrvnodename}")
			logger.debug("Resource: appsrvnodermiport=${appsrvnodermiport}")
			logger.debug("Resource: appsrvnodesoapport${appsrvnodesoapport}")
			logger.debug("Resource: cellname=${cellname}")			
			logger.debug("Resource: defaultsibusname=${defaultsibusname}")
			logger.debug("Resource: mustCreateDmgrProfile=${mustCreateDmgrProfile}")
			logger.debug("Resource: dmgr=${dmgr}")						
			logger.debug("Resource: dmgrnodename=${dmgrnodename}")			
			logger.debug("Resource: dmgrprofilename=${dmgrprofilename}")		
			logger.debug("Resource: dmgrhttpport=${dmgrhttpport}")		
			logger.debug("Resource: dmgrhttpsport=${dmgrhttpsport}")
			logger.debug("Resource: dmgrport=${dmgrport}")							
			logger.debug("Resource: dmgrprotocol=${dmgrprotocol}")	
			logger.debug("Resource: dmgrrmiport=${dmgrrmiport}")		
			logger.debug("Resource: dmgrrmiprops=${dmgrrmiprops}")			
			logger.debug("Resource: dmgrrmitimeout=${dmgrrmitimeout}")		
			logger.debug("Resource: dmgrsoapport=${dmgrsoapport}")
			logger.debug("Resource: dmgrsoapprops=${dmgrsoapprops}")
			logger.debug("Resource: dmgrsoaptimeout=${dmgrsoaptimeout}")
			logger.debug("Resource: dmgripcprops=${dmgripcprops}")
			logger.debug("Resource: dmgripctimeout=${dmgripctimeout}")
			logger.debug("Resource: nodeagent=${nodeagent}")
			logger.debug("Resource: nodermiprops=${nodermiprops}")
			logger.debug("Resource: nodermitimeout=${nodermitimeout}")
			logger.debug("Resource: nodesoapprops=${nodesoapprops}")		
			logger.debug("Resource: nodesoaptimeout=${nodesoaptimeout}")
			logger.debug("Resource: nodeipcprops=${nodeipcprops}")
			logger.debug("Resource: nodeipctimeout=${nodeipctimeout}")
			logger.debug("Resource: os_localhost_default_locale=${os_localhost_default_locale}")	
			logger.debug("Resource: os_localhost_type=${os_localhost_type}")
			logger.debug("Resource: os_localhost_machine_type=${os_localhost_machine_type}")
			logger.debug("Resource: profilename=${profilename}")				
			logger.debug("Resource: script_executable=${script_executable}")
			logger.debug("Resource: script_ext=${script_ext}")	
			logger.debug("Resource: script_flag=${script_flag}")		
			logger.debug("Resource: webappsrv_admin_security_is_enabled=${webappsrv_admin_security_is_enabled}")
			logger.debug("Resource: webappsrv_policy_use_restrictive=${webappsrv_policy_use_restrictive}")
			logger.debug("Resource: webappsrv_scripting_cache_credentials=${webappsrv_scripting_cache_credentials}")	
			logger.debug("Resource: webappsrv_auto_config=${webappsrv_auto_config}")
			logger.debug("Resource: webapp_auto_deploy=${webapp_auto_deploy}")
			logger.debug("Resource: webspherend_is_installed=${webspherend_is_installed}")
			logger.debug("Resource: webappsrv_admin_host=${webappsrv_admin_host}")
			logger.debug("Resource: websphere_appsrv_logs=${websphere_appsrv_logs}")
			logger.debug("Resource: websphere_dmgr_logs=${websphere_dmgr_logs}")
			logger.debug("Resource: websphere_full_version=${websphere_full_version}")
			logger.debug("Resource: websphere_install_dir=${websphere_install_dir}")
			logger.debug("Resource: websphere_profile_dir=${websphere_profile_dir}")
			logger.debug("Resource: websphere_scripting_dir=${websphere_scripting_dir}")
			logger.debug("Resource: websphere_scripting_lib=${websphere_scripting_lib}")
			logger.debug("Resource: websphere_scripting_logs=${websphere_scripting_logs}")
			logger.debug("Resource: websphere_scripting_props=${websphere_scripting_props}")
			logger.debug("Resource: websphere_scripting_src=${websphere_scripting_src}")
			logger.debug("Resource: websphere_scripting_tasks=${websphere_scripting_tasks}")
			logger.debug("Resource: websphere_scripting_tasks_file=${websphere_scripting_tasks_file}")
			logger.debug("Resource: websphere_scripting_scripts=${websphere_scripting_scripts}")
			logger.debug("Resource: websphere_scripting_jython=${websphere_scripting_jython}")
			logger.debug("Resource: websphere_scripting_classpath=${websphere_scripting_classpath}")
			logger.debug("Resource: websphere_scripting_credentials=${websphere_scripting_credentials}")
			logger.debug("Resource: websphere_temp_dir=${websphere_temp_dir}")
			logger.debug("Resource: websphere_wsadmin_classpath=${websphere_wsadmin_classpath}")
			logger.debug("Resource: websphere_wsadmin_jython=${websphere_wsadmin_jython}")
			logger.debug("Resource: File_separator=${File_separator}")
			logger.debug("Resource: config_lev_web_common_dir=${config_lev_web_common_dir}")
			logger.debug("Resource: config_lev_web_staging_dir=${config_lev_web_staging_dir}")
			logger.debug("Resource: wsadminOptionsList=${wsadminOptionsList}")
			logger.debug("Resource: ============= End   All Property Values ============")				
		} 			
	}	// END CTOR
	
}	// END interface Resource	
	