# Copyright (c) 2002-2011, SAS Institute Inc., Cary, NC, USA, All Rights Reserved

import sys

#**************************************************************************************************
# Name: SAScreateDataSource
# Purpose: Create or update a WebLogic DataSource.
# Input:
#   credentialsFile - The full path name of the file containing user credentials.
#                     Credentials used in this operation:
#@@ credentials          datasource.create_<em>resource</em>_userid: The data source user name.
#@@ credentials          datasource.create_<em>resource</em>_passwd: The data source user password.
#   resourcePropertiesFile - A Java properties file that contains values for configuring this resource.
#@@ datasource resourceDescription: Datasource properties are used to configure data source resources in WebLogic. The resource files are named in the pattern <tt>datasource.<em>datasource_name</em>.properties</tt>.  For information about how these properties control WebLogic Server configuration, see the online help in the WebLogic Administration Console.  More information is available in the WebLogic Server documentation found on the Oracle Web site.  The WebLogic Server 10.3.3 documentation for this resource can be found at <a href="http://download.oracle.com/docs/cd/E14571_01/web.1111/e13737/toc.htm">http://download.oracle.com/docs/cd/E14571_01/web.1111/e13737/toc.htm</a>.   
#@@ datasource capacityIncrement: The number of connections created when new connections are added to the connection pool.
#@@ datasource classPath: The classpath consisting of the JAR files required for JDBC driver.  This is only used to keep track of jdbc jars used by various data sources and jdbc providers.
#@@ datasource databaseServerURL: The database server's URL.
#@@ datasource datasourceName: The data source name.  This name must be unique in a WebLogic Server domain.
#@@ datasource driverName: The fully qualified JDBC driver class name.
#@@ datasource globalTransactionsProtocol: The global transactions protocol.  Valid options include <tt>TwoPhaseCommit</tt>, <tt>LoggingLastResource</tt>, <tt>OnePhaseCommit</tt>, <tt>EmulateTwoPhaseCommit</tt>, and <tt>None</tt>. SAS applications depend on correct settings.  Do not change this property.
#@@ datasource initialCapacity: The number of physical connections to create when creating the connection pool.
#@@ datasource inputJarLocation: The fully qualified path for the location where the JDBC driver JAR files have been staged.
#@@ datasource jdbcDriverJarDir: The fully qualified path for the location where the JDBC driver JAR files will be copied and used by the configuration.
#@@ datasource jndiName: The data source JNDI name.  This name is configured in application configuration files and should not be changed without corresponding changes to the applications that use this datasource.
#@@ datasource keepLogicalConnOpenOnRelease: A boolean value.  If <tt>true</tt>, then it enables WebLogic Server to keep the logical JDBC connection open for a global transaction when the physical XA connection is returned to the connection pool.
#@@ datasource keepXaConnTillTxComplete: A boolean value.  If <tt>true</tt>, then it enables WebLogic Server to keep the logical JDBC connection open for a global transaction when the physical XA connection is returned to the connection pool.   SAS applications might depend on particular JDBC transaction settings.  Do not change this property.
#@@ datasource loginDelaySeconds: The number of seconds to delay before creating each physical database connection. This delay supports database servers that cannot handle multiple connection requests in rapid succession.
#@@ datasource maxCapacity: The maximum number of physical connections that this connection pool can contain.
#@@ datasource needTxCtxOnClose: A boolean value.  If <tt>true</tt> it specifies whether the XA driver requires a distributed transaction context when closing various JDBC objects (result sets, statements, connections, and so on). Only applies to connection pools that use an XA driver.   SAS applications may depend on particular jdbc transaction settings.  Do not change this property.
#@@ datasource options:  The list of properties passed to the JDBC driver that are used to create physical database connections. 
#@@ datasource shrinkFrequencySeconds: The number of seconds to wait before shrinking a connection pool that has incrementally increased to meet demand.
#@@ datasource testConnectionsOnReserve: Enables WebLogic Server to test a connection before giving it to a client. (Requires that you specify a Test Table Name.)
#@@ datasource testTableName: The name of the database table to use when testing physical database connections. This name is required when you specify a Test Frequency and enable Test Reserved Connections.
#@@ datasource xaEndOnlyOnce: A boolean value.  If <tt>true</tt>, then it specifies that XAResource.end() is called only once for each pending XAResource.start(). This option prevents the XA driver from calling XAResource.end(TMSUSPEND) and XAResource.end(TMSUCCESS) successively. This property applies to data sources that use an XA driver only. SAS applications might depend on particular JDBC transaction settings.  Do not change this property.
#
# Global properties used - see createDomain.py for descriptions of global resources:
#!! adminUrl
#!! domainDir
#
# Output:
#   None
#**************************************************************************************************
def SAScreateDataSource(credentialsFile, resourcePropertiesFile):
    loadProperties(resourcePropertiesFile)

    # Debug Statements
    logger.debug("adminUrl=" + adminUrl)
    logger.debug("jndiName=" + jndiName)
    logger.debug("targets=" + targets)
    logger.debug("driverName=" + driverName)
    logger.debug("databaseServerURL=" + databaseServerURL)
    logger.debug("options=" + options)
    logger.debug("credentialsFile=" + credentialsFile)
    logger.debug("initialCapacity=" + initialCapacity)
    logger.debug("maxCapacity=" + maxCapacity)
    logger.debug("capacityIncrement=" + capacityIncrement)
    logger.debug("loginDelaySeconds=" + loginDelaySeconds)
    logger.debug("testConnectionsOnReserve=" + testConnectionsOnReserve)
    logger.debug("shrinkFrequencySeconds=" + shrinkFrequencySeconds)
    logger.debug("globalTransactionsProtocol=" + globalTransactionsProtocol)
    logger.debug("keepXaConnTillTxComplete=" + keepXaConnTillTxComplete)
    logger.debug("needTxCtxOnClose=" + needTxCtxOnClose)
    logger.debug("xaEndOnlyOnce=" + xaEndOnlyOnce)
    logger.debug("keepLogicalConnOpenOnRelease=" + keepLogicalConnOpenOnRelease)
    logger.debug('domainDir=' + domainDir)
    logger.debug('jdbcDriverJarDir=' + jdbcDriverJarDir)
    logger.debug('inputJarLocation=' + inputJarLocation)

    editing = None
    rc = RC_FAILURE
    
    copyJDBCDriverJars(inputJarLocation, jdbcDriverJarDir)
    
    try:
        datasourceUsername = getCredentialUser(credentialsFile, "datasource.create_" + datasourceName)
        datasourcePassword = getCredentialPassword(credentialsFile, "datasource.create_" + datasourceName)
        try:
            from com.sas.util import SasPasswordString
            datasourcePassword = SasPasswordString.decode(datasourcePassword)
        except ImportError, e:
            logger.error("Can't find sas.core.jar")
            writeRC(RC_NO_SAS_CORE_JAR)
            return RC_NO_SAS_CORE_JAR
        logger.debug("testTableName=" + testTableName)
        logger.debug("datasourceName=" + datasourceName)

        # Connect to administration server
        connect(url=adminUrl, adminServerName='AdminServer', userConfigFile=nmUserConfigFile, userKeyFile=nmUserKeyFile)

        logger.debug('Creating new JDBC Data Source named ' + datasourceName + '.')
        edit()
        startEdit()
        editing = true
        cd('/')

        # Create data source
        jdbcSystemResource = getMBean('/JDBCSystemResources/' + datasourceName)
        if jdbcSystemResource is None:
            jdbcSystemResource = create(datasourceName, 'JDBCSystemResource')

        jdbcResource = getMBean('/JDBCSystemResources/' + datasourceName + '/JDBCResource/' + datasourceName)
        jdbcResource.setName(datasourceName)

        # Set JNDI name
        jdbcResourceParameters = jdbcResource.getJDBCDataSourceParams()
        jdbcResourceParameters.setJNDINames([jndiName])
  
        if globalTransactionsProtocol <> "not-set":
            jdbcResourceParameters.setGlobalTransactionsProtocol(globalTransactionsProtocol)
        else:
            jdbcResourceParameters.setGlobalTransactionsProtocol("OnePhaseCommit")       

        # Create connection pool
        connectionPool = jdbcResource.getJDBCConnectionPoolParams()
  
        if initialCapacity <> "not-set":
            connectionPool.setInitialCapacity(int(initialCapacity))
        else:
            connectionPool.setInitialCapacity(1)
    
        if maxCapacity <> "not-set":
            connectionPool.setMaxCapacity(int(maxCapacity))
        else: 
            connectionPool.setMaxCapacity(15)
    
        if capacityIncrement <> "not-set":
            connectionPool.setCapacityIncrement(int(capacityIncrement))
        else:
            connectionPool.setCapacityIncrement(1)
  
        if loginDelaySeconds <> "not-set":
            connectionPool.setLoginDelaySeconds(int(loginDelaySeconds))  
        else:
            connectionPool.unSet("LoginDelaySeconds")
    
        if testConnectionsOnReserve <> "not-set":
            if testConnectionsOnReserve.lower() == "true":
                connectionPool.setTestConnectionsOnReserve(1)
            else:
                connectionPool.setTestConnectionsOnReserve(0)
        else:
            connectionPool.unSet("TestConnectionsOnReserve")
    
        if testTableName <> "not-set":
            connectionPool.setTestTableName(testTableName)
        else:
            connectionPool.unSet("TestTableName")
  
        if shrinkFrequencySeconds <> "not-set":
            connectionPool.setShrinkFrequencySeconds(int(shrinkFrequencySeconds))
        else:
            connectionPool.unSet("ShrinkFrequencySeconds")


        jdbcXAParams = jdbcResource.getJDBCXAParams()
  
        if keepXaConnTillTxComplete <> "not-set":
            if keepXaConnTillTxComplete.lower() == "true":
                jdbcXAParams.setKeepXaConnTillTxComplete(1)
            else:
                jdbcXAParams.setKeepXaConnTillTxComplete(0)
        else:
            jdbcXAParams.unSet("KeepXaConnTillTxComplete")
    
        if needTxCtxOnClose <> "not-set":
            if needTxCtxOnClose.lower() == "true":
                jdbcXAParams.setNeedTxCtxOnClose(1)
            else:
                jdbcXAParams.setNeedTxCtxOnClose(0)
        else:
            jdbcXAParams.unSet("NeedTxCtxOnClose")
    
        if xaEndOnlyOnce <> "not-set":
            if xaEndOnlyOnce.lower() == "true":
                jdbcXAParams.setXaEndOnlyOnce(1)
            else:
                jdbcXAParams.setXaEndOnlyOnce(0)
        else:
            jdbcXAParams.unSet("XaEndOnlyOnce")
    
        if keepLogicalConnOpenOnRelease <> "not-set":
            if keepLogicalConnOpenOnRelease.lower() == "true":
                jdbcXAParams.setKeepLogicalConnOpenOnRelease(1)
            else:
                jdbcXAParams.setKeepLogicalConnOpenOnRelease(0)
        else:
            jdbcXAParams.unSet("KeepLogicalConnOpenOnRelease")
             
	
        # Create driver settings
        driver = jdbcResource.getJDBCDriverParams()
        driver.setDriverName(driverName)
  
        if databaseServerURL <> "not-set":
            driver.setUrl(databaseServerURL)
        else:
            rc = RC_FAILURE
            raise Exception, 'JDBC Driver URL is required.'
    
        if datasourcePassword <> "not-set":
            driver.setPasswordEncrypted(encrypt(datasourcePassword, domainDir))
        else:
            driver.unSet("PasswordEncrypted")
  
        driverProperties = driver.getProperties()
        userProperty = driverProperties.lookupProperty('user')
        if datasourceUsername <> "not-set":
            if userProperty is None:
                userProperty = driverProperties.createProperty('user')
            userProperty.setValue(datasourceUsername)
        else:
            if userProperty is not None:
                driverProperties.destroyProperty(userProperty)

        # parse and set options
        if options <> "not-set":
            for p in options.split(","):
                pair = p.split("=", 1)
                logger.debug(pair[0] + "=" + pair[1])
                property = driverProperties.lookupProperty(pair[0])
                if property is None:
                    property = driverProperties.createProperty(pair[0])
                property.setValue(pair[1])

        targetList = String.split(targets, ',')
        for t in targetList:
            targetServer = getMBean('/Servers/' + t)
            if targetServer is None:
                rc = RC_TARGET_DOES_NOT_EXIST
                raise Exception, 'Target Server does not exist: ' + targetServerName + '.'    
            jdbcSystemResource.addTarget(targetServer)

        save()
        activate(block='true')  # Activate changes
        disconnect()
  
        logger.debug('Exiting Data Source creation.')
        writeRC(RC_OK_NEW)
        return RC_OK_NEW
    except Exception, e:
        logger.error('Failed to create Datasource ' + datasourceName + '.')
        logger.error(e)

        writeRC(rc)
    
        # Cancel edit.
        try:
            if editing is not None:
                logger.debug('Canceling edit...')
                cancelEdit('y')
        except WLSTException:
            pass
        return rc
    
from org.apache.commons.io import FileUtils

def copyJDBCDriverJars(srcDir, destDir):
    destDirFile = File(destDir)
    for i in classPath.split(';'):
        path = File(i).getName()
        s = File(srcDir, path)
        logger.debug('copying driver file ' + `s` + ' to ' + destDir)
        FileUtils.copyFileToDirectory(s, destDirFile)

#**************************************************************************************************
# Name: SAScreateJDBCProvider
# Purpose: Copy JDBC JAR files that are required by application that do not setup system data sources to the location that will be added to the system classpath
# Input:
#   credentialsFile - The full path name of the file containing user credentials.
#                     Credentials are not used in this operation.
#   resourcePropertiesFile - A Java properties file that contains values for configuring this resource.
#@@ jdbcprovider resourceDescription: JDBCProvider properties are used to configure JDBC drivers in WebLogic Server. The resource files are named in the patttern <tt>jdbcprovider.<em>resource_name</em>.properties</tt>. WebLogic Server does not use the concept of a JDBCProvider.  These properties are used by the SAS Deployment Wizard to manage the JDBC driver JAR files only.
#@@ jdbcprovider classPath: The classpath consisting of the JAR files required for the JDBC driver.  This property is only used to keep track of the JDBC driver JAR files that are used by various data sources and JDBC providers.
#@@ jdbcprovider jdbcProvider: A name that is used to keep track of the JAR files related to this JDBC driver by a particular application.  This name must be unique in a WebLogic Server domain.
#@@ jdbcprovider inputJarLocation: The fully qualified path for the location where the JDBC driver JAR files have been staged.
#@@ jdbcprovider jdbcDriverJarDir: The fully qualified path for the location where the JDBC driver JAR files will be copied and used by the configuration.
#
# Global properties used - see createDomain.py for descriptions of global resources:
#!! domainDir
#
# Output:
#   None
#**************************************************************************************************
def SAScreateJDBCProvider(credentialsFile, resourcePropertiesFile):
    loadProperties(resourcePropertiesFile)
    logger.debug('inputJarLocation=' + inputJarLocation)
    logger.debug('jdbcDriverJarDir=' + jdbcDriverJarDir)
    copyJDBCDriverJars(inputJarLocation, jdbcDriverJarDir)
    writeRC(RC_OK)
    return RC_OK

