# Copyright (c) 2002-2011, SAS Institute Inc., Cary, NC, USA, All Rights Reserved

import sys

def lookupResource(module, resourceType, resourceName):
    if resourceType == 'ConnectionFactory':
        return module.lookupConnectionFactory(resourceName)
    elif resourceType == 'Queue':
        return module.lookupQueue(resourceName)
    elif resourceType == 'Topic':
        return module.lookupTopic(resourceName)
    else:
        logger.error('Unsupported resource Type: ' + resourceType)
        return null
 
def createResource(module, resourceType, resourceName):
    if resourceType == 'ConnectionFactory':
        return module.createConnectionFactory(resourceName)
    elif resourceType == 'Queue':
        return module.createQueue(resourceName)
    elif resourceType == 'Topic':
        return module.createTopic(resourceName)
    else:
        logger.error('Unsupported resource Type: ' + resourceType)
        return null

#**************************************************************************************************
# Name: SAScreateJMSResource
# Purpose: Create or update a JMS Resource (Connection Factory, Queue, Topic) for Weblogic.
# Input:
#   credentialsFile - The full path name of the file containing user credentials.
#                     Credentials are not used in this operation:
#   resourcePropertiesFile - A Java properties file that contains values for configuring this resource.
#@@ jmsresource resourceDescription: Three type of JMS resources are supported.  Depending on the type of resource, they are stored in properties files that are named in the pattern <tt>connectionfactory.<em>resource</em>.properties</tt>, <tt>queue.<em>resource</em>.properties</tt>, or <tt>topic.<em>resource</em>.properties</tt>.  For information about how these properties control WebLogic Server configuration, see the online help in the WebLogic Administration Console.  More information is available in the WebLogic Server documentation found on the Oracle Web site.  The WebLogic Server 10.3.3 documentation for this resource can be found at <a href="http://download.oracle.com/docs/cd/E14571_01/web.1111/e13738/toc.htm">http://download.oracle.com/docs/cd/E14571_01/web.1111/e13738/toc.htm</a>.
#@@ jmsresource cfXAEnabled: A boolean value.  If <tt>true</tt>, then it indicates whether an XA queue or XA topic connection factory is returned, instead of a queue or topic connection factory. This property applies to connection factories only. SAS applications might depend on particular JDBC transaction settings.  Do not change this property. 
#@@ jmsresource jmsModuleName: The name of the JMS system module to target this resource to.
#@@ jmsresource jmsServerName: The name of the JMS server instance to which this module will be targeted.
#@@ jmsresource jndiName: The global JNDI name used to look up the destination within the JNDI namespace.  This name is configured in application configuration files and should not be changed without corresponding changes to the applications that use this JMS resource.
#@@ jmsresource resourceName: The name of this JMS resource.
#@@ jmsresource resourceType: The type of JMS resource to be configured.  Supported values are <tt>ConnectionFactory</tt>, <tt>Queue</tt>, and <tt>Topic</tt>.
#@@ jmsresource subdeploymentName: Name of the subdeployment whith which this resource will be associated.
#
# Global properties used - see createDomain.py for descriptions of global resources:
#!! adminUrl
#
# Output:
#   None
#**************************************************************************************************

def SAScreateJMSResource(credentialsFile, resourcePropertiesFile):
    loadProperties(resourcePropertiesFile)

    global subdeploymentName
    
    # Debug Statements
    logger.debug('adminUrl=' + adminUrl)
    logger.debug('resourceName=' + resourceName)
    logger.debug('targets=' + targets)
    logger.debug('jndiName=' + jndiName)
    logger.debug('jmsServerName=' + jmsServerName)
    logger.debug('subdeploymentName:' + subdeploymentName)
    logger.debug('jmsModuleName=' + jmsModuleName)
    logger.debug('resourceType=' + resourceType)
    
    if resourceType == "ConnectionFactory":
        logger.debug('cfXAEnabled=' + cfXAEnabled)
    

    # if subdeploymentName == "not-set":
    subdeploymentName=jmsServerName + 'SubDeployment'
    logger.debug("Using default subdeploymentName: " + subdeploymentName)

    editing = None

    try:
        # Connect to administration server
        connect(url=adminUrl, adminServerName='AdminServer', userConfigFile=nmUserConfigFile, userKeyFile=nmUserKeyFile)  

        logger.debug('Creating new ' + resourceType + ' named ' + resourceName + '.')
        edit()
        startEdit()
        editing = true
        cd('/')

        # create a JMS System Resource
        jmsServerMBean=getMBean('/JMSServers/' + jmsServerName)
        if jmsServerMBean is None:
            raise Exception, 'Unable to create ' + resourceType + ' on target ' + targets + '. No JMS Server MBean found for ' + jmsServerName  + '.'    
        else:      
            # create a JMS System Resource
            jmsSASSystemResource = getMBean('/JMSSystemResources/' + jmsModuleName )
            if jmsSASSystemResource is None:
                logger.debug('Creating ' + jmsModuleName + '.')
                jmsSASSystemResource = create(jmsModuleName,"JMSSystemResource")
            else:
                logger.debug('The ' + jmsModuleName + ' already exists.')
                pass
            
            # target the system resource to WLS managed servers
            targetList = String.split(targets, ',')
            for t in targetList:
                targetServerMBean = getMBean('/Servers/' + t)
                if targetServerMBean is None:        
                    cancelEdit('y')
                    disconnect()
                    logger.error("target " + t + "does not exist")
                    writeRC(RC_TARGET_DOES_NOT_EXIST)
                    return RC_TARGET_DOES_NOT_EXIST
                jmsSASSystemResource.addTarget(targetServerMBean)

            # get the system resource object
            theJMSResource = jmsSASSystemResource.getJMSResource()
            # check to see if the resource already exists
            rsrc = lookupResource(theJMSResource, resourceType, resourceName)
            if rsrc is None:
                # create the resource 
                rsrc = createResource(theJMSResource, resourceType, resourceName)
                # set the resource's jndi name
                rsrc.setJNDIName(jndiName)   
                if resourceType == "ConnectionFactory":     
                    if (cfXAEnabled == "true"):
                        rsrc.transactionParams.setXAConnectionFactoryEnabled(1)
                    else:
                        rsrc.transactionParams.setXAConnectionFactoryEnabled(0)                    
            else:
                logger.debug('The ' + resourceType + ' ' + resourceName + ' already exists.')
               
                       
            # configure a subdeployment for the resources
            rsrc.setSubDeploymentName(subdeploymentName)
            subDep1mb = getMBean('/JMSSystemResources/' + jmsModuleName + '/SubDeployments/' + subdeploymentName)
            if subDep1mb is None:
                logger.debug('Creating SubDeployment module')
                subDep1mb = jmsSASSystemResource.createSubDeployment(subdeploymentName)
            else:        
                logger.debug('SubDeployment module already exists.')
	
            # target the sub-deployment object to the JMS server
            jmsServerMBean=getMBean('/JMSServers/' + jmsServerName)
            subDep1mb.addTarget(jmsServerMBean)

            save()
            activate(block='true') # Activate changes
            logger.debug(resourceType + ' ' + resourceName + " has been successfully created and configured.")
            writeRC(RC_OK_NEW)
            disconnect()
            return RC_OK_NEW
    except Exception, e:
        logger.error('Failed to create ' + resourceType + ' ' + resourceName + '.')
        logger.error(e)
        writeRC(RC_FAILURE)
    
    # Cancel edit.
    try:
        if editing is not None:
            logger.debug("Canceling edit...")
            cancelEdit('y')
    except WLSTException:
        pass
    return RC_FAILURE
