# Copyright (c) 2002-2011, SAS Institute Inc., Cary, NC, USA, All Rights Reserved

#***********************************************************************************
# Name: SASdeleteDataSource
# Purpose - delete a JDBC datasource within WebLogic
# Input: arguments passed in from Ant:
#   credentialsFile - The full path name of the file containing user credentials.
#                     Credentials are not used in this operation.
#   resourcePropertiesFile - A Java properties file that contains values for configuring this resource.
#
# resource properties used - see createDataSource.py for descriptions of these resources:
#!! datasource datasourceName
#!! datasource deletedTargets
#
# Global properties used - see createDomain.py for descriptions of global resources:
#!! global adminUrl
#
# Output: 
#     None
#***********************************************************************************
def SASdeleteDataSource(credentialsFile, resourcePropertiesFile):
    loadProperties(resourcePropertiesFile)

    logger.debug("adminUrl=" + adminUrl)
    logger.debug("deletedTargets=" + deletedTargets)
    logger.debug("datasourceName=" + datasourceName)

    try:
        connect(url=adminUrl, adminServerName='AdminServer', userConfigFile=nmUserConfigFile, userKeyFile=nmUserKeyFile)
    except WLSTException:
        writeRC(RC_FAIL_ADM_CONNECT)
        return RC_FAIL_ADM_CONNECT

    if  find(datasourceName, 'JDBCSystemResource'):
        logger.debug(datasourceName + ' exists')
    else:
        logger.debug(datasourceName + ' is not configured')
        writeRC(RC_OK_NOT_CONFIGURED)
        return RC_OK_NOT_CONFIGURED

    try:
        edit()
        startEdit()

        # Save reference to system resource
        jdbcSystemResource = getMBean('/JDBCSystemResources/' + datasourceName)
        if jdbcSystemResource is None:
            logger.debug('jdbc System Resource does not exist: ' + datasourceName + '.')  
            writeRC(RC_OK_NOT_CONFIGURED)
            cancelEdit('y')
            disconnect()
            return RC_OK_NOT_CONFIGURED
        
        deletedTargetList = String.split(deletedTargets, ',')
        for t in deletedTargetList:
            # Save reference to target server
            targetServer = getMBean('/Servers/' + t)
            if targetServer is None:
                logger.debug('Target Server does not exist: ' + t + '.')
                writeRC(RC_OK_NOT_CONFIGURED)
                cancelEdit('y')
                disconnect()
                return RC_OK_NOT_CONFIGURED
            # Remove the dependency in JdbcStore, where the data source may be referenced
            removeJDBCStorebyDatasource(datasourceName,targetServer)

            # remove data source target  
            jdbcSystemResource.removeTarget(targetServer)
            
        if len(jdbcSystemResource.getTargets()) == 0:
            logger.debug('No more targets.  Removing data source.')
            delete(datasourceName,'JDBCSystemResource')
        
        save()
        activate(block="true")
        disconnect()
        removeJDBCJars()
    except  WLSTException:
        logger.error('Exception deleting datasource, canceling Edit session to release lock..')
        cancelEdit('y')
        writeRC(RC_FAILURE)
        return RC_FAILURE

    writeRC(RC_OK)
    logger.debug('Exiting Data Source deletion.')
    return RC_OK

#***********************************************************************************
# Name: SASdeleteJDBCProvider
# Purpose - delete a JDBC provider
# Input: arguments passed in from Ant:
#   credentialsFile - The full path name of the file containing user credentials.
#                     Credentials are not used in this operation.
#   resourcePropertiesFile - A java properties file that contains values for configuring this resource.
#
# resource properties used - see createDataSource.py for descriptions of these resources:
#!! jdbcprovider datasourceName
#!! jdbcprovider deletedTargets
#
# Global properties used - see createDomain.py for descriptions of global resources:
#
# Output: 
#     None
#***********************************************************************************
def SASdeleteJDBCProvider(credentialsFile, resourcePropertiesFile):
    loadProperties(resourcePropertiesFile)
    logger.debug('inputJarLocation=' + inputJarLocation)
    logger.debug('jdbcDriverJarDir=' + jdbcDriverJarDir)
    removeJDBCJars()
    writeRC(RC_OK)
    return RC_OK

from com.sas.config.weblogic import WebLogicContextBuilder
from com.sas.config.weblogic import WebLogicDeletedResourceLoaderFilter
from com.sas.config import Context
from com.sas.config import DirectoryResourceLoader
from java.io import File

# Remove any jars that are part of this resource and are not in use by any other resource
def removeJDBCJars():
    jarsInUse = getAllJDBCJarsInUse()
    print jarsInUse
    print classPath
    for i in classPath.split(File.pathSeparatorChar):
        path = File(i).getName()
        if not (path in jarsInUse):
            jar = File(jdbcDriverJarDir, path)
            print "removing " + `jar`
            jar.delete()

# Look at properties files to collect names of all jars in use by dataSources and jdbcproviders
def getAllJDBCJarsInUse():
    pcb = WebLogicContextBuilder()
    resourceDir = File(domainPropertiesFile).getParentFile()
    directoryResourceLoader = DirectoryResourceLoader(resourceDir)
    directoryResourceLoader.setFilter(WebLogicDeletedResourceLoaderFilter())
    pcb.loadResources(directoryResourceLoader)
    ctx = pcb.build(File(resourceDir, "foo"))
    jarsInUse = []
    getAllJDBCJarsForResourceType("datasources", ctx, jarsInUse)
    getAllJDBCJarsForResourceType("jdbcproviders", ctx, jarsInUse)
    return jarsInUse

def getAllJDBCJarsForResourceType(resourceName, ctx, jarList):
    resources = ctx.getProperty(resourceName)
    if resources != None:
        for t in resources:
            clsPath = t.getProperty("classPath")
            for i in clsPath.split(File.pathSeparatorChar):
                path = File(i).getName()
                if not (path in jarList):
                    jarList.append(path)
#function to help remove all related JdbcStore mbean(s) which reference the data source.
# that match a specific name
def removeJDBCStorebyDatasource(dsName,targetServ):
    cd('/JDBCStores')
    jdbcStoreList = cmo.getJDBCStores()
    if jdbcStoreList is not None:
       for js in jdbcStoreList:
          jdbcDS=js.getDataSource()
          if jdbcDS is None:
              continue
          if jdbcDS.getName()==dsName and (targetServ in js.getTargets()):
              logger.debug('Removing JDBCStore Resource: ' + js.getName() + ' referred the data source: ' + dsName)
              jsName = js.getName()
              removeJMSServerbyJDBCStore(jsName)
              logger.debug('Removing JDBCStore Resource: ' + js.getName() + ' referred the data source: ' + dsName)
              delete(js.getName(),'JDBCStore')     

def removeJMSServerbyJDBCStore(storeName):
    try:
        if(storeName.endswith('_JDBCStore')):
            logger.debug('JMS Server named ' + storeName.rstrip('_JDBCStore') + '.')
            # jmsServerName=storeName.rstrip('_JDBCStore')
            jmsServerName=storeName[0:len(storeName) - len('_JDBCStore')]
            logger.debug('Attempting to delete the JMS Server named ' + jmsServerName + '.')
            cd('/JMSServers/' + jmsServerName)
            logger.debug('Deleting JMS Server named ' + jmsServerName + '.')
            cd('/')  
            jmsServerMBean = delete(jmsServerName,'JMSServer')
            logger.debug('Exiting JMS Server deletion.')
            writeRC(RC_OK)
            return RC_OK
    except WLSTException:
            writeRC(RC_OK_NOT_CONFIGURED)
            logger.debug('The JMS Server with name ' + jmsServerName + ' does not exist.')
            return RC_OK_NOT_CONFIGURED
   