# Copyright (c) 2002-2011, SAS Institute Inc., Cary, NC, USA, All Rights Reserved

import sys
import string

#**************************************************************************************************
# Name: SASdeployApplication
# Purpose: Deploy application to specified targets
# Input: arguments passed in from Ant:
#   credentialsFile - The full path name of the file containing user credentials.
#                     Credentials are not used in this operation.
#   resourcePropertiesFile - A Java properties file that contains values for configuring this resource.
#
#@@ application resourceDescription: These resources represent applications deployed in a WebLogic Server domain.  The resource files are named in the pattern <tt>application.<em>application_name</em>.properties</tt>.  For information about how these properties control WebLogic Server configuration, see the online help in the WebLogic Administration Console.  More information is available in the WebLogic Server documentation found on the Oracle Web site.  The WebLogic Server 10.3.3 documentation for this resource can be found at <a href="http://download.oracle.com/docs/cd/E14571_01/apirefs.1111/e13952/taskhelp/applications/DeployEnterpriseApplications.html">http://download.oracle.com/docs/cd/E14571_01/apirefs.1111/e13952/taskhelp/applications/DeployEnterpriseApplications.html</a>.
#@@ application applicationName: The name of the SAS Web application.
#@@ application applicationPath: The fully qualified path to the application archive file or directory.
#@@ application loadOrder: The application load order.
#@@ application docApplicationName: This property is used by SAS Deployment Wizard to generate documentation.  It is not used by the SAS Configuration Scripting Tools.
#
# Global properties used - see createDomain.py for descriptions of global resources:
#!! adminHost
#!! adminUrl
#!! host
#
#**************************************************************************************************
def SASdeployApplication(credentialsFile, resourcePropertiesFile):
    loadProperties(resourcePropertiesFile)
    
    # Debug Statements
    logger.debug("applicationName=" + applicationName)
    logger.debug("applicationPath=" + applicationPath)
    logger.debug("targets=" + targets)
    logger.debug("adminUrl=" + adminUrl)
    logger.debug('host:' + host)
    logger.debug('adminHost:' + adminHost)
    logger.debug('loadOrder:' + loadOrder)

    errno = RC_FAILURE
    contexts = None
    editing = None
    try:
        try:
            connect(url=adminUrl, adminServerName='AdminServer', userConfigFile=nmUserConfigFile, userKeyFile=nmUserKeyFile)
        except WLSTException:
            logger.error('Failed to connect to the Admin Server.')
            writeRC(RC_FAIL_ADM_CONNECT)
            return RC_FAIL_ADM_CONNECT
  
        targetList = String.split(targets, ',')

        # Check to see if the application is already deployed on any of the targets - if so, fail
        appMBean = getMBean('/AppDeployments/' + applicationName)
        if appMBean != None:
            for t in targetList:
                targetMBean = getMBean('/AppDeployments/' + applicationName + '/Targets/' + t)         
                if targetMBean != None:
                    logger.error('The application ' + applicationName + ' is already deployed.')
                    writeRC(RC_APP_ALREADY_DEPLOYED)
                    return RC_APP_ALREADY_DEPLOYED
        
        # Deploy to all targets
        for t in targetList:        
            logger.debug("deploying to target " + t)
            if host.lower() == adminHost.lower():
                progress = deploy(applicationName, applicationPath, t, stageMode='nostage', timeout=1800000)
            else:
                progress = deploy(applicationName, applicationPath, t, stageMode='stage', timeout=1800000, upload='true')    
      
        logger.debug("state=" + progress.getState())
        if progress.isFailed():
                    errno = RC_APP_DEPLOYMENT_FAILED
                    writeRC(RC_APP_DEPLOYMENT_FAILED)
                    raise Exception, 'Failed to deploy ' + applicationName + ' on targets ' + t + '.' 
    
        edit()
        startEdit()
        editing = true
        cd("AppDeployments/" + applicationName)
        cmo.setDeploymentOrder(int(loadOrder))      
        save()
        activate(block="true") 
      
        logger.debug('Deployed application successfully.')   
        writeRC(RC_OK)
        rc = RC_OK
        disconnect()
        return rc
    except Exception, e:
        errorMsg = 'Failed to deploy application: ' + applicationName + '.'
        logger.error(errorMsg)
        logger.error(e)

    # Cancel edit.
    try:
        if editing is not None:
            logger.debug("Canceling edit...")
            cancelEdit('y')
    except WLSTException:
        pass  
  
    writeRC(RC_FAILURE)
    return RC_FAILURE

#**************************************************************************************************
# Name: SAScompileApplication
# Purpose: Compile an application (compile JSP's and EJBs)
# Input: arguments passed in from Ant:
#   credentialsFile - The full path name of the file containing user credentials.
#                     Credentials are not used in this operation.
#   resourcePropertiesFile - A Java properties file that contains values for configuring this resource.
#
#@@ application compileMaxMemory: The -Xmx option used on the <tt>weblogic.appc</tt> command line when compiling a Web application.
#
# Global properties used - see createDomain.py for descriptions of global resources:
#!! javaHome
#!! osType
#
#**************************************************************************************************
def SAScompileApplication(credentialsFile, resourcePropertiesFile):
    loadProperties(resourcePropertiesFile)
    logger.debug("compileMaxMemory" + compileMaxMemory)
    logger.debug("applicationPath=" + applicationPath)

    if osType == 'win':
        rc = execScript('"' + javaHome + '\\bin\\java.exe" -Xmx' + compileMaxMemory + " weblogic.appc " + applicationPath + " -forceGeneration -verbose")
    else:
        rc = execScript(javaHome + "/bin/java -Xmx" + compileMaxMemory + " weblogic.appc " + applicationPath + " -forceGeneration -verbose")
       
    writeRC(rc)
    return rc