#!/bin/sh


#########################
# Environment Variables #
#########################

if [ -x /usr/bin/echo ] ; then
   echo=/usr/bin/echo
elif [ -x /bin/echo ] ; then
   echo=/bin/echo
else
   echo=echo
fi


#############
# Functions #
#############

###########################################################################
#
#  detect_host
#    Identify the host on which this script is running.  In the event the
#    host can not be identified, print out an error message to the user.
#
###########################################################################

detect_host () {
   hosttype=`uname`
   case "$hosttype" in
       OSF1) $echo "Error: The application cannot be run on this host: $hosttype."
             exit
             ;;

      SunOS) proc_type=`uname -p` 
             if [ "$proc_type" = "sparc" ] ; then
                platform="s64"
             elif [ "$proc_type" = "i386" ] ; then
                platform="sax"
             else
                $echo "Error: The application cannot be run on this host: $hosttype."
                exit
             fi
             ;;

      HP-UX) model_type=`model`
             if [ "`set -f ; $echo $model_type | sed -n '/ia64/ {p;}'`" ] ; then
                platform="h6i"
             else
                platform="h64"
             fi
             ;;

      Linux) machine_type=`uname -m`
             if [ "$machine_type" = "x86_64" ] ; then
                platform="lax"
             elif [ "$machine_type" != "ia64" ] ; then
                platform="lnx"
             else
                $echo "Error: The application cannot be run on this host: $hosttype."
                exit
             fi
             ;;

        AIX) platform="r64"
             ;;

     Darwin) proc_type=`uname -p`
             if [ "$proc_type" = "powerpc" ] ; then
                platform="mpp"
             elif [ "$proc_type" = "i386" ] ; then
                if [ "`sysctl -n hw.optional.x86_64 2> /dev/null`" = 1 ] ; then
                   platform="m64"
                else
                   platform="mac"
                fi
             else
                $echo "Error: The application cannot be run on this host: $hosttype."
                exit
             fi
             ;;
   esac
}

###########################################################################
#
#  locate_data_file
#    Locate the data file associated with the script passed in.  A value
#    is only returned from this function if a valid data file is found to
#    exist for use by the script.
#
###########################################################################
locate_data_file () {
   script=$1
   extension=$2

   dir="`dirname "$script"`"
   base="`basename "$script"`"

   file="$dir/`set -f ; $echo "$base" | sed 's/^\([^\.]*\).*$/\1/'`$extension"

   if [ ! -f "$file" ] ; then
      if [ "`set -f ; $echo $base | sed -n '/^[^\.]*_console\..*$/ {p;}'`" ] ; then
         file="$dir/`set -f ; $echo "$base" | sed 's/^\([^\.]*\)_console\..*$/\1/'`$extension"
      elif [ "`set -f ; $echo $base | sed -n '/^[^\.]*_console$/ {p;}'`" ] ; then
         file="$dir/`set -f ; $echo "$base" | sed 's/^\([^\.]*\)_console$/\1/'`$extension"
      fi
   fi

   if [ ! -f "$file" ] ; then
      file=""
   fi

   $echo $file
   return
}

###########################################################################
#
#  load_data
#    Load all of the data in the given section of a given file.  After
#    each value is loaded, perform property substitutions on variables
#    in the data strings with "<name>".  If a property name is not
#    available to replace a given "<name>", the literal string is left
#    in the data string unaltered.
#
###########################################################################

load_data () {
   file=$1
   section=$2

   exec 9<&0 < "$file"

   found=0
   while read line ; do
      line=`set -f ; $echo $line | tr -d '\15\32'`

      if [ $found = 0 ] ; then
         if [ "$line" = "[$section]" ] ; then
            found=1
         fi
      elif [ "`set -f ; $echo $line | sed -n '/^\[..*\]$/ {p;}'`" ] ; then
         break
      elif [ "`set -f ; $echo $line | sed -n '/^[ \t\v\f]*[A-Za-z0-9_][A-Za-z0-9_]*[ \t\v\f]*=.*$/ {p;}'`" ] ; then
         property=`set -f ; $echo $line | sed 's/^[ \t\v\f]*\([A-Za-z0-9_][A-Za-z0-9_]*\)[ \t\v\f]*=\(.*\)$/\1="\2"/'`
         property=`set -f ; $echo $property | sed 's/\(<\([A-Za-z0-9_][A-Za-z0-9_]*\)>\)/${PROPERTY_\2-\1}/g'`
         eval $property
      fi
   done 

   exec 0<&9 9<&-
}

###########################################################################
#
#  load_properties
#    Load all of the properties in the given section of a given file.
#
###########################################################################

load_properties () {
   file=$1
   section=$2

   exec 9<&0 < "$file"

   found=0
   while read line ; do
      line=`set -f ; $echo $line | tr -d '\15\32'`

      if [ $found = 0 ] ; then
         if [ "$line" = "[$section]" ] ; then
            found=1
         fi
      elif [ "`set -f ; $echo $line | sed -n '/^\[..*\]$/ {p;}'`" ] ; then
         break
      elif [ "`set -f ; $echo $line | sed -n '/^[ \t\v\f]*[A-Za-z0-9_][A-Za-z0-9_]*[ \t\v\f]*=.*$/ {p;}'`" ] ; then
         property=`set -f ; $echo $line | sed 's/^[ \t\v\f]*\([A-Za-z0-9_][A-Za-z0-9_]*\)[ \t\v\f]*=\(.*\)$/PROPERTY_\1="\2"/'`
         eval $property
      fi
   done 

   exec 0<&9 9<&-
}

###########################################################################
#
#  set_umask
#    Set the umask according to the rules defined in the data.
#
###########################################################################

set_umask () {
   mask=`umask`

   if [ "$umask_user" ] ; then
      mask=`set -f ; echo $mask | sed "s/[0-7]\([0-7][0-7]\)$/$umask_user\1/"`
   fi

   if [ "$umask_group" ] ; then
      mask=`set -f ; echo $mask | sed "s/[0-7]\([0-7]\)$/$umask_group\1/"`
   fi

   if [ "$umask_other" ] ; then
      mask=`set -f ; echo $mask | sed "s/[0-7]$/$umask_other/"`
   fi

   umask $mask
}

###########################################################################
#
#  simplify_path
#    Simplifies a fully qualified path which may contain extra / . or ..
#    characters.  Otherwise, the ".."s may be processed relative to
#    the target of the symbolic link target thus throwing the path off the
#    desired trail.
#
#    NOTE: This does not deal with absolutely every case, but should be
#    safe with all fully qualified paths which may need simplification.
#
###########################################################################

simplify_path () {
   formatted="$1"

   # Strip out instances of //
   formatted="`set -f ; $echo "$formatted" | sed 's/\/\/*/\//g'`"

   # Start looping and stripping out instances of /./ until there are no more
   previous=""
   while [ "$previous" != "$formatted" ] ; do
      previous="$formatted"
      formatted="`set -f ; $echo "$formatted" | sed 's/\/\.\//\//g'`"
   done

   # Strip out instances of ./ at the begiing of the path
   formatted="`set -f ; $echo "$formatted" | sed 's/^\.\///g'`"

   # Start looping and stripping out instances of .. until there are no more
   previous=""
   while [ "$previous" != "$formatted" ] ; do
      previous="$formatted"

      # Replace "[^/]+/../" at the start of the path with ""
      formatted="`set -f ; $echo "$previous" | sed 's/^[^\/][^\/]*\/\.\.\///'`"

      # Replace the first instance of "/[^/]+/../" anywhere in the path with "/"
      formatted="`set -f ; $echo "$formatted" | sed 's/\/[^\/][^\/]*\/\.\.\//\//'`"
   done

   # Replace a path fitting the pattern "[^/]+/.." with ""
   formatted="`set -f ; $echo "$formatted" | sed 's/^[^\/][^\/]*\/\.\.$//'`"

   # Replace a trailing "/[^/]+/.." with ""
   formatted="`set -f ; $echo "$formatted" | sed 's/\/[^\/][^\/]*\/\.\.$//'`"

   $echo "$formatted"
   return
}

###########################################################################
#
#  check_free_space
#    Check to be sure there is enough disk space in the TEMP location
#
###########################################################################

check_free_space () {
   # Calculate required space only if it hasn't already been done
   if [ ! "$spacereq" ] ; then
      spacereq=0

      copynum=1
      while [ "`eval $echo "$"{copy$copynum"+0}"`" ] ; do
         value="`eval $echo "$"copy$copynum""`"

         if [ "$value" ] ; then
            if [ -d "$startuplocation/$value" -o -f "$startuplocation/$value" ] ; then
               size=`du -s "$startuplocation/$value" | awk '{print $1}'`
               spacereq=`expr $spacereq + $size`
            fi
         fi

         copynum=`expr $copynum + 1`
      done

      if [ "$jre" ] ; then
         size=`zcat < "$startuplocation/$jre" | wc -c`
         size=`expr $size + 1024`
         size=`expr $size / 1024`
         spacereq=`expr $spacereq + $size`
      fi
   fi

   if [ -w "$templocation" -a -x "$templocation" ] ; then
      case "$platform" in
         h64) dfcmd="/usr/bin/bdf"
              ;;
         h6i) dfcmd="/usr/bin/bdf"
              ;;
         lax) dfcmd="/bin/df -k"
              ;;
         lnx) dfcmd="/bin/df -k"
              ;;
         mpp) dfcmd="/bin/df -k"
              ;;
         mac) dfcmd="/bin/df -k"
              ;;
         m64) dfcmd="/bin/df -k"
              ;;
         r64) dfcmd="/usr/bin/df -kI"
              ;;
         s64) dfcmd="/usr/bin/df -k"
              ;;
         sax) dfcmd="/usr/bin/df -k"
              ;;
           *) dfcmd="df -k"
              ;;
      esac

      diskfree=`$dfcmd "$templocation" | awk '{line=$0} ; END {print line}' | sed 's/^[^ ]* *[0-9]* *[0-9]* *\([0-9]*\).*/\1/'`

      case "$?","$diskfree" in
         0,[0-9] | 0,[0-9]*[0-9] )
            if [ $diskfree -lt $spacereq ] ; then
               disk_space="notenough"
               $echo "This application requires $spacereq KB of free space in $templocation to execute."
               get_alt_dir
            else
               disk_space="enough"
            fi
            ;;
      esac
   else
      $echo "Warning: $templocation does not have the appropriate permissions."
      get_alt_dir
   fi
}

###########################################################################
#
# get_alt_dir
#    Prompt the user for an alternate temporary directory
#
###########################################################################
get_alt_dir () {
   valid_alt_dir="false"

   until  [ "$valid_alt_dir" = "true" ] ; do
      $echo "Enter alternate directory or (q) to quit: "
      read altdir

      templocation=$altdir
      case "$templocation" in
         q|Q|quit|Quit|QUIT)
            exit 0
            ;;
         *)
            if [ -n "$templocation" ] ; then
               check_free_space
               if [ "$disk_space" = "enough" ] ; then
                  valid_alt_dir=true
               else
                  continue
               fi
            else
               read state
               case "$state" in
                  q|Q|quit|Quit|QUIT)
                     exit 0
                     ;;
                esac
            fi
            ;;
      esac
   done

   return
}

###########################################################################
#
#  copy_files
#    Copy all of the files needed to run the application to TEMP
#
###########################################################################

copy_files () {
   copynum=1
   while [ "`eval $echo "$"{copy$copynum"+0}"`" ] ; do
      value="`eval $echo "$"copy$copynum""`"

      if [ "$value" ] ; then
         if [ -d "$startuplocation/$value" -o -f "$startuplocation/$value" ] ; then
            mkdir -p "`dirname "$templocation/$value"`" 1> /dev/null 2>&1
            cp -r "$startuplocation/$value" "$templocation/$value"
         fi
      fi

      copynum=`expr $copynum + 1`
   done

   # Extract a JRE if one is present in the data
   if [ "$jre" ] ; then
      mkdir -p "$templocation/`dirname "$jre"`" 1> /dev/null 2>&1
      cd "$templocation/`dirname "$jre"`"
      zcat < "$startuplocation/$jre" | tar xvf - > /dev/null 2>&1
      cd "$pwd"
   fi
}

###########################################################################
#
#  parse_arguments
#    Parse the command line arguments and set the necessary variables.
#
###########################################################################

parse_arguments () {
   # Parse command line arguments
   while [ "$1" != "" ] ; do
      case $1 in
         -quiet)
            headless=1
            shift
            continue;;
         -console)
            mode="console"
            shift
            continue;;
         -installsas)
            headless=1
            shift
            continue;;
         -templocation)
            alttmp=1
            shift
            if [ -d "$1" ] ; then
               templocation=$1
            fi
            shift
            continue;;
         -datalocation)
            shift
            if [ -d "$1" ] ; then
               datalocation=$1
            fi
            shift
            continue;;
         *)
            shift
            continue;;
      esac
   done
}


########
# Main #
########

pwd="`pwd`"
PROPERTY_WORKINGDIR="$pwd"

exe="$0"

# Dereference any symbolic links to the real executable if there is no paired DAT file
if [ ! "`locate_data_file "$exe" ".dat"`" ] ; then
   while [ -h "$exe" ]; do
      link="`ls -ld "$exe"`"
      link="`expr "$link" : '.*-> \(.*\)'`"
      if [ "`expr "$link" : '/.*'`" = 0 ]; then
         exe="`dirname "$exe"`/$link"
      else
         exe="$link"
      fi
   done
fi

# Determine the script name (less any extensions)
script="`basename "$exe" | sed 's/^\([^\.]*\).*$/\1/'`"

# Determine the startup location
cd "`dirname "$exe"`"
startuplocation="`pwd`"
PROPERTY_LAUNCHERDIR="$startuplocation"
cd "$pwd"

# Determine if we are running in console mode
if [ "`set -f ;  $echo $script | sed -n '/_console$/ {p;}'`" ] ; then
   mode="console"
fi


# Establish the templocation
templocation="/tmp"

# Test for a DAT file paired with the shell script (by script name)
datfile="`locate_data_file "$startuplocation/$script" ".dat"`"
if [ "$datfile" ] ; then
   load_data "$datfile" default
else
   $echo "Error: Missing $script.dat."
   exit 1
fi

# Test for an INI file paired with the shell script (by script name)
if [ "$inifile" ] ; then
   if [ ! "`set -f ; $echo $inifile | sed -n '/^\// {p;}'`" ] ; then
      inifile="$startuplocation/$inifile"
   fi

   if [ ! -f "$inifile" ] ; then
      $echo "Error: Missing $inifile."
      exit 1
   fi
fi

parse_arguments "$@"

# Establish the default log
timestamp="`date "+%Y-%m-%d-%H.%M.%S"`"
echo ${datalocation}
if [ -d "$datalocation" ] ; then
    logfile="${datalocation}/${script}_${timestamp}.log"
else
    logfile="$HOME/.SASAppData/SASDeploymentWizard/${script}_${timestamp}.log"
fi

detect_host

# Determine if there is a master properties file to read first by parsing the INI
load_properties "$inifile" properties

if [ "$PROPERTY_MASTERPROP" ] ; then
   # The path to the master property file may be relative or absolute
   if [ ! "`set -f ; $echo $PROPERTY_MASTERPROP | sed -n '/^\// {p;}'`" ] ; then
      inilocation="`dirname "$inifile"`"
      PROPERTY_MASTERPROP=`simplify_path "$inilocation/$PROPERTY_MASTERPROP"`
   fi

   if [ -f "$PROPERTY_MASTERPROP" ] ; then
      load_properties "$PROPERTY_MASTERPROP" properties
   else
      $echo "Error: Missing $PROPERTY_MASTERPROP."
      exit 1
   fi
fi

# Parse the INI file properties again to override any properties from the master
load_properties "$inifile" properties

# Parse the INI file for data specific to the detected platform
load_data "$inifile" default
load_data "$inifile" "$platform"

# Test for a DISPLAY
if [ "$mode" != "console" -a ! "$headless" -a "$platform" != "mpp" -a "$platform" != "mac" -a "$platform" != "m64" -a ! "$DISPLAY" ] ; then
   $echo "Error: Your DISPLAY environmental variable has not been set."
   $echo "Please set it and run this application again."
   exit 2
fi

$echo "Preparing the SAS Deployment Wizard..."

# Set the umask based upon the data
set_umask

# Present an error if there is no launch command
if [ ! "$launch" ] ; then
   $echo "Error: This application cannot be run on this platform: $platform."
   exit 3
fi

# Determine whether or not the permissions are consistent between the bootstrap and the launcher
if [ "`ls -l "$startuplocation/$launch" | awk '$1 !~ /x/'`" ] ; then
   $echo "Error: There appear to be discrepancies in the permissions on your SAS Software Depot."
   $echo "Please change the permissions, as follows, and run this application again."
   $echo "   chmod -R 0755 \"$startuplocation\""
   exit 2
fi

# Perform Disk Space Checking
check_free_space

templocation="$templocation/_setup$$"
mkdir "$templocation" 1> /dev/null 2>&1

# Process any copy statements enumerated in the data
copy_files

$echo "Launching the SAS Deployment Wizard..."

if [ "$mode" = "console" ] ; then
   exec "$templocation/$launch" "$@" -startuplocation "$startuplocation" -templocation "$templocation"
else
   # Set up the logging directory and initialize the log
   mkdir -p "`dirname "$logfile"`" 1> /dev/null 2>&1

   $echo "---------------------------------------------------------------" >> "$logfile"
   date >> "$logfile"

   # Run the command
   if [ "$alttmp" ] ; then
      exec "$templocation/$launch" "$@" -startuplocation "$startuplocation"  1>> "$logfile" 2>&1
   else
      exec "$templocation/$launch" "$@" -startuplocation "$startuplocation" -templocation "$templocation" 1>> "$logfile" 2>&1
   fi

fi

exit $?
